@@ -1,22 +1,21 @@
 """Define handlers for communicating with a buildlogger server."""
 
-from __future__ import absolute_import
-
 import functools
 import json
+import os
 import threading
 
 import requests
 
-from . import handlers
-from .. import config as _config
+from buildscripts.resmokelib import config as _config
+from buildscripts.resmokelib.logging import handlers
 
 CREATE_BUILD_ENDPOINT = "/build"
 APPEND_GLOBAL_LOGS_ENDPOINT = "/build/%(build_id)s"
 CREATE_TEST_ENDPOINT = "/build/%(build_id)s/test"
 APPEND_TEST_LOGS_ENDPOINT = "/build/%(build_id)s/test/%(test_id)s"
 
-_BUILDLOGGER_CONFIG = "mci.buildlogger"
+_BUILDLOGGER_CONFIG = os.getenv("BUILDLOGGER_CREDENTIALS", "mci.buildlogger")
 
 _SEND_AFTER_LINES = 2000
 _SEND_AFTER_SECS = 10
@@ -84,7 +83,7 @@ class _LogsSplitter(object):
             2 is added to each string size to account for the array representation of the logs,
             as each line is preceded by a '[' or a space and followed by a ',' or a ']'.
             """
-            return len(json.dumps(line, encoding="utf-8")) + 2
+            return len(json.dumps(line)) + 2
 
         curr_logs = []
         curr_logs_size = 0
@@ -116,7 +115,11 @@ class _BaseBuildloggerHandler(handlers.BufferedHandler):
 
         self.endpoint = endpoint
         self.retry_buffer = []
-        self.max_size = None
+        # Set a reasonable max payload size in case we don't get a HTTP 413 from LogKeeper
+        # before timing out. This limit is intentionally slightly larger than LogKeeper's
+        # limit of 32MB so we can still receive a 413 where appropriate but won't cause
+        # side effects.
+        self.max_size = 33 * 1024 * 1024
 
     def process_record(self, record):
         """Return a tuple of the time the log record was created, and the message.
@@ -199,8 +202,8 @@ class _BaseBuildloggerHandler(handlers.BufferedHandler):
             # writing the messages to the fallback logkeeper to avoid putting additional pressure on
             # the Evergreen database.
             BUILDLOGGER_FALLBACK.warning(
-                "Failed to flush all log output (%d messages) to logkeeper.", len(
-                    self.retry_buffer))
+                "Failed to flush all log output (%d messages) to logkeeper.",
+                len(self.retry_buffer))
 
             # We set a flag to indicate that we failed to flush all log output to logkeeper so
             # resmoke.py can exit with a special return code.
@@ -225,10 +228,11 @@ class BuildloggerTestHandler(_BaseBuildloggerHandler):
     @_log_on_error
     def _finish_test(self, failed=False):
         """Send a POST request to the APPEND_TEST_LOGS_ENDPOINT with the test status."""
-        self.post(self.endpoint, headers={
-            "X-Sendlogs-Test-Done": "true",
-            "X-Sendlogs-Test-Failed": "true" if failed else "false",
-        })
+        self.post(
+            self.endpoint, headers={
+                "X-Sendlogs-Test-Done": "true",
+                "X-Sendlogs-Test-Failed": "true" if failed else "false",
+            })
 
     def close(self):
         """Close the buildlogger handler."""
@@ -261,7 +265,9 @@ class BuildloggerServer(object):
         """Initialize BuildloggerServer."""
         tmp_globals = {}
         self.config = {}
-        execfile(_BUILDLOGGER_CONFIG, tmp_globals, self.config)
+        exec(
+            compile(open(_BUILDLOGGER_CONFIG, "rb").read(), _BUILDLOGGER_CONFIG, 'exec'),
+            tmp_globals, self.config)
 
         # Rename "slavename" to "username" if present.
         if "slavename" in self.config and "username" not in self.config:
@@ -284,11 +290,12 @@ class BuildloggerServer(object):
         handler = handlers.HTTPHandler(url_root=_config.BUILDLOGGER_URL, username=username,
                                        password=password, should_retry=True)
 
-        response = handler.post(CREATE_BUILD_ENDPOINT, data={
-            "builder": builder,
-            "buildnum": build_num,
-            "task_id": _config.EVERGREEN_TASK_ID,
-        })
+        response = handler.post(
+            CREATE_BUILD_ENDPOINT, data={
+                "builder": builder,
+                "buildnum": build_num,
+                "task_id": _config.EVERGREEN_TASK_ID,
+            })
 
         return response["id"]
 
