@@ -11,10 +11,11 @@ import pymongo.errors
 
 from buildscripts.resmokelib import errors
 from buildscripts.resmokelib import utils
-from buildscripts.resmokelib.testing.hooks import interface
+from buildscripts.resmokelib.testing.fixtures import interface as fixture_interface
 from buildscripts.resmokelib.testing.fixtures import replicaset
 from buildscripts.resmokelib.testing.fixtures import shardedcluster
-from buildscripts.resmokelib.testing.fixtures import interface as fixture_interface
+from buildscripts.resmokelib.testing.fixtures import tenant_migration
+from buildscripts.resmokelib.testing.hooks import interface
 
 
 class ContinuousStepdown(interface.Hook):  # pylint: disable=too-many-instance-attributes
@@ -27,12 +28,12 @@ class ContinuousStepdown(interface.Hook):  # pylint: disable=too-many-instance-a
             self, hook_logger, fixture, config_stepdown=True, shard_stepdown=True,
             stepdown_interval_ms=8000, terminate=False, kill=False,
             use_stepdown_permitted_file=False, wait_for_mongos_retarget=False,
-            stepdown_via_heartbeats=True, background_reconfig=False):
+            stepdown_via_heartbeats=True, background_reconfig=False, auth_options=None):
         """Initialize the ContinuousStepdown.
 
         Args:
             hook_logger: the logger instance for this hook.
-            fixture: the target fixture (a replica set or sharded cluster).
+            fixture: the target fixture (replica sets or a sharded cluster).
             config_stepdown: whether to stepdown the CSRS.
             shard_stepdown: whether to stepdown the shard replica sets in a sharded cluster.
             stepdown_interval_ms: the number of milliseconds between stepdowns.
@@ -65,6 +66,7 @@ class ContinuousStepdown(interface.Hook):  # pylint: disable=too-many-instance-a
         self._kill = kill
 
         self._background_reconfig = background_reconfig
+        self._auth_options = auth_options
 
         # The stepdown file names need to match the same construction as found in
         # jstests/concurrency/fsm_libs/resmoke_runner.js.
@@ -89,7 +91,8 @@ class ContinuousStepdown(interface.Hook):  # pylint: disable=too-many-instance-a
         self._stepdown_thread = _StepdownThread(
             self.logger, self._mongos_fixtures, self._rs_fixtures, self._stepdown_interval_secs,
             self._terminate, self._kill, lifecycle, self._wait_for_mongos_retarget,
-            self._stepdown_via_heartbeats, self._background_reconfig)
+            self._stepdown_via_heartbeats, self._background_reconfig, self._fixture,
+            self._auth_options)
         self.logger.info("Starting the stepdown thread.")
         self._stepdown_thread.start()
 
@@ -127,6 +130,14 @@ class ContinuousStepdown(interface.Hook):  # pylint: disable=too-many-instance-a
             if self._wait_for_mongos_retarget:
                 for mongos_fixture in fixture.mongos:
                     self._mongos_fixtures.append(mongos_fixture)
+        elif isinstance(fixture, tenant_migration.TenantMigrationFixture):
+            if not fixture.all_nodes_electable:
+                raise ValueError(
+                    "The replica sets that are the target of the ContinuousStepdown hook must have"
+                    " the 'all_nodes_electable' option set.")
+
+            for rs_fixture in fixture.get_replsets():
+                self._rs_fixtures.append(rs_fixture)
 
 
 class FlagBasedStepdownLifecycle(object):
@@ -343,7 +354,7 @@ class _StepdownThread(threading.Thread):  # pylint: disable=too-many-instance-at
     def __init__(  # pylint: disable=too-many-arguments
             self, logger, mongos_fixtures, rs_fixtures, stepdown_interval_secs, terminate, kill,
             stepdown_lifecycle, wait_for_mongos_retarget, stepdown_via_heartbeats,
-            background_reconfig):
+            background_reconfig, fixture, auth_options=None):
         """Initialize _StepdownThread."""
         threading.Thread.__init__(self, name="StepdownThread")
         self.daemon = True
@@ -361,6 +372,8 @@ class _StepdownThread(threading.Thread):  # pylint: disable=too-many-instance-at
         self._should_wait_for_mongos_retarget = wait_for_mongos_retarget
         self._stepdown_via_heartbeats = stepdown_via_heartbeats
         self._background_reconfig = background_reconfig
+        self._fixture = fixture
+        self._auth_options = auth_options
 
         self._last_exec = time.time()
         # Event set when the thread has been stopped using the 'stop()' method.
@@ -470,11 +483,14 @@ class _StepdownThread(threading.Thread):  # pylint: disable=too-many-instance-at
         for fixture in self._rs_fixtures:
             fixture.get_primary()
 
+    def _create_client(self, node):
+        return fixture_interface.authenticate(node.mongo_client(), self._auth_options)
+
     def _step_down_all(self):
         for rs_fixture in self._rs_fixtures:
             self._step_down(rs_fixture)
 
-    # pylint: disable=R0912,R0915
+    # pylint: disable=R0912,R0914,R0915
     def _step_down(self, rs_fixture):
         try:
             primary = rs_fixture.get_primary(timeout_secs=self._stepdown_interval_secs)
@@ -523,7 +539,7 @@ class _StepdownThread(threading.Thread):  # pylint: disable=too-many-instance-at
             self.logger.info("Stepping down the primary on port %d of replica set '%s'.",
                              primary.port, rs_fixture.replset_name)
             try:
-                client = primary.mongo_client()
+                client = self._create_client(primary)
                 client.admin.command(
                     bson.SON([
                         ("replSetStepDown", self._stepdown_duration_secs),
@@ -555,7 +571,7 @@ class _StepdownThread(threading.Thread):  # pylint: disable=too-many-instance-at
                              chosen.port, rs_fixture.replset_name)
 
             try:
-                client = chosen.mongo_client()
+                client = self._create_client(chosen)
                 client.admin.command("replSetStepUp")
                 break
             except pymongo.errors.OperationFailure:
@@ -566,6 +582,10 @@ class _StepdownThread(threading.Thread):  # pylint: disable=too-many-instance-at
                 self.logger.info("Failed to step up the secondary on port %d of replica set '%s'.",
                                  chosen.port, rs_fixture.replset_name)
                 secondaries.remove(chosen)
+            except pymongo.errors.AutoReconnect:
+                # It is possible for a replSetStepUp to fail with AutoReconnect if that node goes
+                # into Rollback (which causes it to close any open connections).
+                pass
 
         if self._terminate:
             self.logger.info("Waiting for the old primary on port %d of replica set '%s' to exit.",
@@ -582,6 +602,7 @@ class _StepdownThread(threading.Thread):  # pylint: disable=too-many-instance-at
             primary.preserve_dbpath = True
             try:
                 primary.setup()
+                self.logger.info(fixture_interface.create_fixture_table(self._fixture))
                 primary.await_ready()
             finally:
                 primary.preserve_dbpath = original_preserve_dbpath
@@ -592,7 +613,7 @@ class _StepdownThread(threading.Thread):  # pylint: disable=too-many-instance-at
             # introduced a stepdown period on the former primary and so we have to run the
             # {replSetFreeze: 0} command to ensure the former primary is electable in the next
             # round of _step_down().
-            client = primary.mongo_client()
+            client = self._create_client(primary)
             client.admin.command({"replSetFreeze": 0})
         elif secondaries:
             # We successfully stepped up a secondary, wait for the former primary to step down via
@@ -604,7 +625,7 @@ class _StepdownThread(threading.Thread):  # pylint: disable=too-many-instance-at
                 chosen.port, rs_fixture.replset_name)
             while True:
                 try:
-                    client = primary.mongo_client()
+                    client = self._create_client(primary)
                     is_secondary = client.admin.command("isMaster")["secondary"]
                     if is_secondary:
                         break
@@ -626,7 +647,7 @@ class _StepdownThread(threading.Thread):  # pylint: disable=too-many-instance-at
             retry_start_time = time.time()
             while True:
                 try:
-                    client = primary.mongo_client()
+                    client = self._create_client(primary)
                     client.admin.command("replSetStepUp")
                     break
                 except pymongo.errors.OperationFailure:
@@ -655,7 +676,7 @@ class _StepdownThread(threading.Thread):  # pylint: disable=too-many-instance-at
         for mongos_fixture in self._mongos_fixtures:
             mongos_conn_str = mongos_fixture.get_internal_connection_string()
             try:
-                client = mongos_fixture.mongo_client()
+                client = self._create_client(mongos_fixture)
             except pymongo.errors.AutoReconnect:
                 pass
             for db in client.database_names():
