@@ -22,10 +22,21 @@ class WaitForReplication(interface.Hook):
         """Run mongo shell to call replSetTest.awaitReplication()."""
         start_time = time.time()
         client_conn = self.fixture.get_driver_connection_url()
-        js_cmds = "conn = '{}'; rst = new ReplSetTest(conn); rst.awaitReplication();".format(
-            client_conn)
-        shell_options = {"nodb": "", "eval": js_cmds}
-        shell_proc = core.programs.mongo_shell_program(self.hook_logger, **shell_options)
+        js_cmds = """
+            conn = '{}';
+            try {{
+                rst = new ReplSetTest(conn);
+                rst.awaitReplication();
+            }} catch (e) {{
+                jsTestLog("WaitForReplication got error: " + tojson(e));
+                if (!e.message.includes('The server is in quiesce mode and will shut down')) {{
+                    throw e;
+                }}
+                jsTestLog("Ignoring shutdown error in quiesce mode");
+            }}"""
+        shell_options = {"nodb": "", "eval": js_cmds.format(client_conn)}
+        shell_proc = core.programs.mongo_shell_program(self.hook_logger, self.fixture.job_num,
+                                                       **shell_options)
         shell_proc.start()
         return_code = shell_proc.wait()
         if return_code:
