@@ -8,21 +8,29 @@ import threading
 import time
 import unittest
 
-from .. import config as _config
-from .. import logging
+from buildscripts.resmokelib import config as _config
+from buildscripts.resmokelib import logging
 
 
 # pylint: disable=attribute-defined-outside-init
 class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attributes
     """Record test status and timing information."""
 
-    def __init__(self, job_logger, suite_options):
-        """Initialize the TestReport with the buildlogger configuration."""
+    def __init__(self, job_logger, suite_options, job_num=None):
+        """
+        Initialize the TestReport with the buildlogger configuration.
+
+        :param job_logger: The higher-level logger that will be used to print metadata about the test.
+        :param suite_options: Options for the suite being executed.
+        :param job_num: The number corresponding to the job this test runs in.
+        """
 
         unittest.TestResult.__init__(self)
 
         self.job_logger = job_logger
+        self.job_num = job_num
         self.suite_options = suite_options
+        self.logging_prefix = None
 
         self._lock = threading.Lock()
 
@@ -40,7 +48,7 @@ class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attr
 
         # TestReports that are used when running tests need a JobLogger but combined reports don't
         # use the logger.
-        combined_report = cls(logging.loggers.EXECUTOR_LOGGER,
+        combined_report = cls(logging.loggers.ROOT_EXECUTOR_LOGGER,
                               _config.SuiteOptions.ALL_INHERITED.resolve())
         combining_time = time.time()
 
@@ -75,6 +83,11 @@ class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attr
                         # Use the current time as the time that the test finished running.
                         test_info.end_time = combining_time
 
+                    # If we receive a SIGUSR1 then we may start combining reports before
+                    # their start time has been set.
+                    if test_info.start_time is None:
+                        test_info.start_time = combining_time
+
                     combined_report.test_infos.append(test_info)
 
                 combined_report.num_dynamic += report.num_dynamic
@@ -93,7 +106,11 @@ class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attr
         unittest.TestResult.startTest(self, test)
 
         test_info = _TestInfo(test.id(), test.test_name, test.dynamic)
-        test_info.start_time = time.time()
+        if _config.SPAWN_USING == "jasper":
+            # The group id represents the group of logs this test belongs to in
+            # cedar buildlogger. It must be sent to evergreen/cedar in order to
+            # create the correct log URL.
+            test_info.group_id = str(self.job_num)
 
         basename = test.basename()
         command = test.as_command()
@@ -108,11 +125,18 @@ class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attr
                 self.num_dynamic += 1
 
         # Set up the test-specific logger.
-        test_logger = self.job_logger.new_test_logger(test.short_name(), test.basename(), command,
-                                                      test.logger)
-        test_info.url_endpoint = test_logger.url_endpoint
+        (test_logger, url_endpoint) = logging.loggers.new_test_logger(test.short_name(),
+                                                                      test.basename(), command,
+                                                                      test.logger, self.job_num,
+                                                                      test.id(), self.job_logger)
+        test_info.url_endpoint = url_endpoint
+        if self.logging_prefix is not None:
+            test_logger.info(self.logging_prefix)
+        # Set job_num in test.
+        test.job_num = self.job_num
 
         test.override_logger(test_logger)
+        test_info.start_time = time.time()
 
     def stopTest(self, test):  # pylint: disable=invalid-name
         """Call after 'test' has run."""
@@ -274,6 +298,12 @@ class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attr
                     "elapsed": test_info.end_time - test_info.start_time,
                 }
 
+                if test_info.display_test_name is not None:
+                    result["display_test_name"] = test_info.display_test_name
+
+                if test_info.group_id is not None:
+                    result["group_id"] = test_info.group_id
+
                 if test_info.url_endpoint is not None:
                     result["url"] = test_info.url_endpoint
                     result["url_raw"] = test_info.url_endpoint + "?raw=1"
@@ -295,11 +325,13 @@ class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attr
         report = cls(logging.loggers.EXECUTOR_LOGGER, _config.SuiteOptions.ALL_INHERITED.resolve())
         for result in report_dict["results"]:
             # By convention, dynamic tests are named "<basename>:<hook name>".
-            is_dynamic = ":" in result["test_file"]
+            is_dynamic = ":" in result["test_file"] or ":" in result.get("display_test_name", "")
             test_file = result["test_file"]
             # Using test_file as the test id is ok here since the test id only needs to be unique
             # during suite execution.
             test_info = _TestInfo(test_file, test_file, is_dynamic)
+            test_info.display_test_name = result.get("display_test_name")
+            test_info.group_id = result.get("group_id")
             test_info.url_endpoint = result.get("url")
             test_info.status = result["status"]
             test_info.evergreen_status = test_info.status
@@ -352,12 +384,39 @@ class _TestInfo(object):  # pylint: disable=too-many-instance-attributes
         """Initialize the _TestInfo instance."""
 
         self.test_id = test_id
-        self.test_file = test_file
+        # If spawned using jasper, we need to set the display_test_name and the
+        # test_file since these are distinct in cedar buildlogger.
+        if _config.SPAWN_USING == "jasper":
+            self.test_file = str(test_id)
+            self.display_test_name = test_file
+        else:
+            self.test_file = test_file
+            self.display_test_name = None
         self.dynamic = dynamic
 
+        self.group_id = None
         self.start_time = None
         self.end_time = None
         self.status = None
         self.evergreen_status = None
         self.return_code = None
         self.url_endpoint = None
+
+
+def test_order(test_name):
+    """
+    A key function used for sorting _TestInfo objects by recommended order of investigation.
+
+    Investigate setup/teardown errors, then hooks, then test files.
+    """
+
+    if 'fixture_setup' in test_name:
+        return 1
+    elif 'fixture_teardown' in test_name:
+        return 2
+    elif 'fixture_abort' in test_name:
+        return 3
+    elif ':' in test_name:
+        return 4
+    else:
+        return 5
