@@ -1,4 +1,4 @@
-DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" >/dev/null 2>&1 && pwd)"
+DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" > /dev/null 2>&1 && pwd)"
 . "$DIR/prelude.sh"
 
 cd src
@@ -8,7 +8,7 @@ set -o verbose
 
 activate_venv
 
-rm -rf /data/install dist-test/bin
+rm -rf /data/install /data/multiversion
 
 edition="${multiversion_edition}"
 platform="${multiversion_platform}"
@@ -34,12 +34,18 @@ if [ ! -z "${multiversion_architecture_44_or_later}" ]; then
   architecture="${multiversion_architecture_44_or_later}"
 fi
 
+version=${project#mongodb-mongo-}
+version=${version#v}
+
 # This is primarily for tests for infrastructure which don't always need the latest
 # binaries.
 $python buildscripts/resmoke.py setup-multiversion \
   --installDir /data/install \
-  --linkDir dist-test/bin \
+  --linkDir /data/multiversion \
   --edition $edition \
   --platform $platform \
   --architecture $architecture \
-  --useLatest master
+  --useLatest $version
+
+version_dir=$(find /data/install -type d -iname "*$version*")
+mv $version_dir/dist-test $(pwd)
