@@ -70,4 +70,4 @@ $python buildscripts/resmoke.py setup-multiversion \
   --edition $edition \
   --platform $platform \
   --architecture $architecture \
-  --useLatest 4.4 4.9
+  --useLatest 4.4
