@@ -29,24 +29,24 @@ function assertFirstLast(expectedFirst, expectedLast, stages, expression) {
 }
 
 // One document.
-assert.writeOK(coll.insert({a: 1, b: 1}));
+assert.commandWorked(coll.insert({a: 1, b: 1}));
 assertFirstLast(1, 1);
 
 // Two documents.
-assert.writeOK(coll.insert({a: 1, b: 2}));
+assert.commandWorked(coll.insert({a: 1, b: 2}));
 assertFirstLast(1, 2);
 
 // Three documents.
-assert.writeOK(coll.insert({a: 1, b: 3}));
+assert.commandWorked(coll.insert({a: 1, b: 3}));
 assertFirstLast(1, 3);
 
 // Another 'a' key value does not affect outcome.
 assert(coll.drop());
-assert.writeOK(coll.insert({a: 3, b: 0}));
-assert.writeOK(coll.insert({a: 1, b: 1}));
-assert.writeOK(coll.insert({a: 1, b: 2}));
-assert.writeOK(coll.insert({a: 1, b: 3}));
-assert.writeOK(coll.insert({a: 2, b: 0}));
+assert.commandWorked(coll.insert({a: 3, b: 0}));
+assert.commandWorked(coll.insert({a: 1, b: 1}));
+assert.commandWorked(coll.insert({a: 1, b: 2}));
+assert.commandWorked(coll.insert({a: 1, b: 3}));
+assert.commandWorked(coll.insert({a: 2, b: 0}));
 assertFirstLast(1, 3);
 
 // Additional pipeline stages do not affect outcome if order is maintained.
@@ -57,64 +57,64 @@ assertFirstLast(3, 1, [{$sort: {b: -1}}]);
 
 // Skip and limit affect the results seen.
 assert(coll.drop());
-assert.writeOK(coll.insert({a: 1, b: 1}));
-assert.writeOK(coll.insert({a: 1, b: 2}));
-assert.writeOK(coll.insert({a: 1, b: 3}));
+assert.commandWorked(coll.insert({a: 1, b: 1}));
+assert.commandWorked(coll.insert({a: 1, b: 2}));
+assert.commandWorked(coll.insert({a: 1, b: 3}));
 assertFirstLast(1, 2, [{$limit: 2}]);
 assertFirstLast(2, 3, [{$skip: 1}, {$limit: 2}]);
 assertFirstLast(2, 2, [{$skip: 1}, {$limit: 1}]);
 
 // Mixed type values.
-assert.writeOK(coll.insert({a: 1, b: 'foo'}));
+assert.commandWorked(coll.insert({a: 1, b: 'foo'}));
 assertFirstLast(1, 'foo');
 
 assert(coll.drop());
-assert.writeOK(coll.insert({a: 1, b: 'bar'}));
-assert.writeOK(coll.insert({a: 1, b: true}));
+assert.commandWorked(coll.insert({a: 1, b: 'bar'}));
+assert.commandWorked(coll.insert({a: 1, b: true}));
 assertFirstLast('bar', true);
 
 // Value null.
 assert(coll.drop());
-assert.writeOK(coll.insert({a: 1, b: null}));
-assert.writeOK(coll.insert({a: 1, b: 2}));
+assert.commandWorked(coll.insert({a: 1, b: null}));
+assert.commandWorked(coll.insert({a: 1, b: 2}));
 assertFirstLast(null, 2);
 
 assert(coll.drop());
-assert.writeOK(coll.insert({a: 1, b: 2}));
-assert.writeOK(coll.insert({a: 1, b: null}));
+assert.commandWorked(coll.insert({a: 1, b: 2}));
+assert.commandWorked(coll.insert({a: 1, b: null}));
 assertFirstLast(2, null);
 
 assert(coll.drop());
-assert.writeOK(coll.insert({a: 1, b: null}));
-assert.writeOK(coll.insert({a: 1, b: null}));
+assert.commandWorked(coll.insert({a: 1, b: null}));
+assert.commandWorked(coll.insert({a: 1, b: null}));
 assertFirstLast(null, null);
 
 // Value missing.
 assert(coll.drop());
-assert.writeOK(coll.insert({a: 1}));
-assert.writeOK(coll.insert({a: 1, b: 2}));
+assert.commandWorked(coll.insert({a: 1}));
+assert.commandWorked(coll.insert({a: 1, b: 2}));
 assertFirstLast(undefined, 2);
 
 assert(coll.drop());
-assert.writeOK(coll.insert({a: 1, b: 2}));
-assert.writeOK(coll.insert({a: 1}));
+assert.commandWorked(coll.insert({a: 1, b: 2}));
+assert.commandWorked(coll.insert({a: 1}));
 assertFirstLast(2, undefined);
 
 assert(coll.drop());
-assert.writeOK(coll.insert({a: 1}));
-assert.writeOK(coll.insert({a: 1}));
+assert.commandWorked(coll.insert({a: 1}));
+assert.commandWorked(coll.insert({a: 1}));
 assertFirstLast(undefined, undefined);
 
 // Dotted field.
 assert(coll.drop());
-assert.writeOK(coll.insert({a: 1, b: [{c: 1}, {c: 2}]}));
-assert.writeOK(coll.insert({a: 1, b: [{c: 6}, {}]}));
+assert.commandWorked(coll.insert({a: 1, b: [{c: 1}, {c: 2}]}));
+assert.commandWorked(coll.insert({a: 1, b: [{c: 6}, {}]}));
 assertFirstLast([1, 2], [6], [], '$b.c');
 
 // Computed expressions.
 assert(coll.drop());
-assert.writeOK(coll.insert({a: 1, b: 1}));
-assert.writeOK(coll.insert({a: 1, b: 2}));
+assert.commandWorked(coll.insert({a: 1, b: 1}));
+assert.commandWorked(coll.insert({a: 1, b: 2}));
 assertFirstLast(1, 0, [], {$mod: ['$b', 2]});
 assertFirstLast(0, 1, [], {$mod: [{$add: ['$b', 1]}, 2]});
 }());
