@@ -2,6 +2,7 @@
 //   requires_getmore,
 //   requires_non_retryable_writes,
 //   uses_multiple_connections,
+//   uses_parallel_shell,
 // ]
 
 // SERVER-2009 Count odd numbered entries while updating and deleting even numbered entries.
@@ -11,13 +12,13 @@
 
 const t = db.jstests_remove9;
 t.drop();
-t.ensureIndex({i: 1});
+t.createIndex({i: 1});
 
 const bulk = t.initializeUnorderedBulkOp();
 for (let i = 0; i < 1000; ++i) {
     bulk.insert({i: i});
 }
-assert.writeOK(bulk.execute());
+assert.commandWorked(bulk.execute());
 
 const s = startParallelShell(function() {
     const t = db.jstests_remove9;
