@@ -5,13 +5,13 @@ var replTest =
     new ReplSetTest({name: name, oplogSize: 1, nodes: 3, settings: {chainingAllowed: false}});
 var nodes = replTest.startSet();
 replTest.initiate();
-var master = replTest.getPrimary();
-var mdb = master.getDB("test");
+var primary = replTest.getPrimary();
+var mdb = primary.getDB("test");
 
 // synchronize replication
 assert.commandWorked(mdb.foo.insert({_id: "1"}, {writeConcern: {w: 3, wtimeout: 5 * 60 * 1000}}));
 
-var gle = master.getDB("test").runCommand({getLastError: 1, j: true});
+var gle = primary.getDB("test").runCommand({getLastError: 1, j: true});
 print('Trying j=true');
 printjson(gle);
 if (gle.err === null) {
@@ -48,8 +48,8 @@ assert.eq(gle.wtimeout, null);
 
 // take a node down and GLE for more nodes than are up
 replTest.stop(2);
-master = replTest.getPrimary();
-mdb = master.getDB("test");
+primary = replTest.getPrimary();
+mdb = primary.getDB("test");
 // do w:2 write so secondary is caught up before calling {gle w:3}.
 assert.commandWorked(mdb.foo.insert({_id: "3"}, {writeConcern: {w: 2, wtimeout: 5 * 60 * 1000}}));
 gle = mdb.getLastErrorObj(3, 1000);
