@@ -227,6 +227,13 @@ if (typeof _threadInject != "undefined") {
 
             // Assumes that other tests are not creating API version 1 incompatible data.
             "validate_db_metadata_command.js",
+
+            // This test causes delete commands to hang, which may affect other tests running
+            // concurrently.
+            "timeseries/timeseries_delete_hint.js",
+            "timeseries/timeseries_update_hint.js",
+            "timeseries/timeseries_delete_concurrent.js",
+            "timeseries/timeseries_update_concurrent.js"
         ]);
 
         // Get files, including files in subdirectories.
@@ -249,8 +256,10 @@ if (typeof _threadInject != "undefined") {
                 "apply_ops_system_dot_views.js",
                 "command_let_variables.js",
                 "doc_validation_error.js",
+                "internal_apply_oplog_update.js",
                 "merge_sort_collation.js",
                 "explode_for_sort_fetch.js",
+                "or_to_in.js",
                 "update_pipeline_shell_helpers.js",
                 "update_with_pipeline.js",
                 "verify_update_mods.js",
@@ -271,7 +280,11 @@ if (typeof _threadInject != "undefined") {
         }
 
         // Transactions are not supported on standalone nodes so we do not run them here.
-        let txnsTestFiles = getFilesRecursive("jstests/core/txns").map(f => ("txns/" + f.baseName));
+        // NOTE: We need to take substring of the full test path to ensure that 'jstests/core/' is
+        // not included.
+        const txnsTestFiles =
+            getFilesRecursive("jstests/core/txns/")
+                .map(fullPathToTest => fullPathToTest.name.substring("jstests/core/".length));
         Object.assign(skipTests, makeKeys(txnsTestFiles));
 
         var parallelFilesDir = "jstests/core";
