@@ -5,8 +5,6 @@
 (function() {
 'use strict';
 
-load("jstests/libs/feature_compatibility_version.js");
-
 let dbpath = MongoRunner.dataPath + "setFCV_collmod_transaction_rollback";
 resetDbpath(dbpath);
 
@@ -38,15 +36,5 @@ const index = coll.getIndexes();
 var ttlAfterRollback = index[1].expireAfterSeconds;
 assert.eq(ttlAfterRollback, ttlBeforeRollback);
 
-// SERVER-37634 should remove this test post 4.2.
-// Test transaction rollback after unique index upgrade collMod.
-assert.commandWorked(adminDB.adminCommand({setFeatureCompatibilityVersion: lastStableFCV}));
-
-assert.commandWorked(coll.createIndex({a: 1}, {unique: true}));
-assert.writeOK(coll.insert({_id: 0, a: 1}));
-
-assert.commandFailedWithCode(adminDB.adminCommand({setFeatureCompatibilityVersion: latestFCV}),
-                             50971);
-
 MongoRunner.stopMongod(conn);
 })();
