@@ -7,6 +7,8 @@ load("jstests/ssl/libs/ssl_helpers.js");
 const OCSP_CA_PEM = "jstests/libs/ocsp/ca_ocsp.pem";
 const OCSP_CA_CERT = "jstests/libs/ocsp/ca_ocsp.crt";
 const OCSP_CA_KEY = "jstests/libs/ocsp/ca_ocsp.key";
+const CLUSTER_CA_CERT = "jstests/libs/ca.pem";
+const CLUSTER_KEY = "jstests/libs/server.pem";
 const OCSP_SERVER_CERT = "jstests/libs/ocsp/server_ocsp.pem";
 const OCSP_CLIENT_CERT = "jstests/libs/ocsp/client_ocsp.pem";
 const OCSP_SERVER_MUSTSTAPLE_CERT = "jstests/libs/ocsp/server_ocsp_mustStaple.pem";
@@ -59,6 +61,40 @@ var waitForServer = function(conn) {
     }
 };
 
+var clientConnect = function(conn) {
+    const exitCode = runMongoProgram("mongo",
+                                     "--host",
+                                     "localhost",
+                                     "--port",
+                                     conn.port,
+                                     "--tls",
+                                     "--tlsCAFile",
+                                     OCSP_CA_PEM,
+                                     "--tlsCertificateKeyFile",
+                                     OCSP_CLIENT_CERT,
+                                     "--tlsAllowInvalidHostnames",
+                                     "--verbose",
+                                     1,
+                                     "--eval",
+                                     ";");
+    return exitCode;
+};
+
+const OCSP_REVOKED = "OCSPCertificateStatusRevoked";
+
+var assertClientConnectFails = function(conn, reason) {
+    clearRawMongoProgramOutput();
+    assert.neq(clientConnect(conn), 0);
+    const errmsg = rawMongoProgramOutput();
+    if (typeof reason === 'string' || reason instanceof RegExp) {
+        assert.neq(errmsg.search(reason), -1);
+    }
+};
+
+var assertClientConnectSucceeds = function(conn) {
+    assert.eq(clientConnect(conn), 0);
+};
+
 var supportsStapling = function() {
     if (determineSSLProvider() !== "openssl") {
         return false;
