@@ -8,23 +8,23 @@ var replTest = new ReplSetTest({name: 'capped_insert_order', nodes: 2});
 replTest.startSet();
 replTest.initiate();
 
-var master = replTest.getPrimary();
-var slave = replTest._slaves[0];
+var primary = replTest.getPrimary();
+var secondary = replTest.getSecondary();
 
 var dbName = "db";
-var masterDb = master.getDB(dbName);
-var slaveDb = slave.getDB(dbName);
+var primaryDb = primary.getDB(dbName);
+var secondaryDb = secondary.getDB(dbName);
 
 var collectionName = "collection";
-var masterColl = masterDb[collectionName];
-var slaveColl = slaveDb[collectionName];
+var primaryColl = primaryDb[collectionName];
+var secondaryColl = secondaryDb[collectionName];
 
 // Making a large capped collection to ensure that every document fits.
-masterDb.createCollection(collectionName, {capped: true, size: 1024 * 1024});
+primaryDb.createCollection(collectionName, {capped: true, size: 1024 * 1024});
 
 // Insert 1000 docs with _id from 0 to 999 inclusive.
 const nDocuments = 1000;
-var batch = masterColl.initializeOrderedBulkOp();
+var batch = primaryColl.initializeOrderedBulkOp();
 for (var i = 0; i < nDocuments; i++) {
     batch.insert({_id: i});
 }
@@ -42,8 +42,8 @@ function checkCollection(coll) {
     assert.eq(i, nDocuments);
 }
 
-checkCollection(masterColl);
-checkCollection(slaveColl);
+checkCollection(primaryColl);
+checkCollection(secondaryColl);
 
 replTest.stopSet();
 })();
