@@ -2,14 +2,15 @@
  * Tests that setting the feature compatibility version during initial sync leads to initial sync
  * restarting. This test also ensures that even if initial sync takes two attempts to complete,
  * that the fCV is reset between attempts.
+ *
+ * This tests behavior centered around both upgrading and downgrading FCV.
+ * @tags: [multiversion_incompatible]
  */
 
 (function() {
 'use strict';
 
-load('jstests/libs/check_log.js');
 load("jstests/libs/fail_point_util.js");
-load("jstests/libs/feature_compatibility_version.js");
 
 const rst = new ReplSetTest({nodes: 2});
 rst.startSet();
@@ -24,19 +25,25 @@ rst.initiate(replSetConfig);
 const primary = rst.getPrimary();
 const dbName = 'foo';
 const collName = 'bar';
+// The default WC is majority and this test can't satisfy majority writes.
+assert.commandWorked(primary.adminCommand(
+    {setDefaultRWConcern: 1, defaultWriteConcern: {w: 1}, writeConcern: {w: "majority"}}));
 
-assert.writeOK(primary.getDB(dbName).getCollection(collName).insert({a: 1}));
+assert.commandWorked(primary.getDB(dbName).getCollection(collName).insert({a: 1}));
 
 function runInitialSync(cmd, initialFCV) {
     assert.commandWorked(primary.adminCommand({setFeatureCompatibilityVersion: initialFCV}));
 
     jsTestLog('Testing setting fCV with ' + tojson(cmd));
 
-    const failPointOptions = tojson({mode: 'alwaysOn', data: {database: dbName}});
+    const failPointOptions = tojson({
+        mode: 'alwaysOn',
+        data: {cloner: "DatabaseCloner", stage: "listCollections", database: dbName}
+    });
     rst.restart(1, {
         startClean: true,
         setParameter: {
-            'failpoint.initialSyncHangBeforeListCollections': failPointOptions,
+            'failpoint.hangBeforeClonerStage': failPointOptions,
             'failpoint.skipClearInitialSyncState': tojson({mode: 'alwaysOn'}),
             numInitialSyncAttempts: 2
         }
@@ -47,7 +54,7 @@ function runInitialSync(cmd, initialFCV) {
     // secondary to initialFCV. We then block the secondary before issuing 'listCollections' on
     // the test database.
     assert.commandWorked(secondary.adminCommand({
-        waitForFailPoint: "initialSyncHangBeforeListCollections",
+        waitForFailPoint: "hangBeforeClonerStage",
         timesEntered: 1,
         maxTimeMS: kDefaultWaitForFailPointTimeout
     }));
@@ -58,8 +65,8 @@ function runInitialSync(cmd, initialFCV) {
 
     // Let initial sync finish, making sure that it fails due to the feature compatibility
     // version change.
-    assert.commandWorked(secondary.adminCommand(
-        {configureFailPoint: 'initialSyncHangBeforeListCollections', mode: 'off'}));
+    assert.commandWorked(
+        secondary.adminCommand({configureFailPoint: 'hangBeforeClonerStage', mode: 'off'}));
     checkLog.contains(secondary, 'Applying operation on feature compatibility version document');
 
     jsTestLog('Wait for both nodes to be up-to-date');
@@ -76,16 +83,16 @@ function runInitialSync(cmd, initialFCV) {
 
 // Ensure that attempting to downgrade the featureCompatibilityVersion during initial sync
 // fails.
-runInitialSync({setFeatureCompatibilityVersion: lastStableFCV}, /*initialFCV*/ latestFCV);
+runInitialSync({setFeatureCompatibilityVersion: lastLTSFCV}, /*initialFCV*/ latestFCV);
 
 // Ensure that attempting to upgrade the featureCompatibilityVersion during initial sync fails.
-runInitialSync({setFeatureCompatibilityVersion: latestFCV}, /*initialFCV*/ lastStableFCV);
+runInitialSync({setFeatureCompatibilityVersion: latestFCV}, /*initialFCV*/ lastLTSFCV);
 
 // Modifications to the featureCompatibilityVersion document during initial sync should be
 // caught and cause initial sync to fail.
 runInitialSync({
     update: 'system.version',
-    updates: [{q: {_id: 'featureCompatibilityVersion'}, u: {'version': lastStableFCV}}]
+    updates: [{q: {_id: 'featureCompatibilityVersion'}, u: {'version': lastLTSFCV}}]
 },
                /*initialFCV*/ latestFCV);
 
