@@ -9,10 +9,12 @@ const commandsRemovedFromMongosSinceLastLTS = [];
 // defined without always existing on the mongos being used.
 const commandsAddedToMongosSinceLastLTS = [
     "abortReshardCollection",
+    "appendOplogNote",
     "cleanupReshardCollection",
     "commitReshardCollection",
     "reshardCollection",
     "rotateCertificates",
+    "setAllowMigrations",
     "testDeprecation",
     "testDeprecationInVersion2",
     "testRemoval",
