@@ -103,6 +103,7 @@ let testCases = {
     _configsvrRemoveTags: {skip: "internal command"},
     _configsvrRenameCollection: {skip: "internal command"},
     _configsvrRenameCollectionMetadata: {skip: "internal command"},
+    _configsvrRepairShardedCollectionChunksHistory: {skip: "internal command"},
     _configsvrReshardCollection: {skip: "internal command"},
     _configsvrSetAllowMigrations: {skip: "internal command"},
     _configsvrShardCollection: {skip: "internal command"},
@@ -131,6 +132,7 @@ let testCases = {
     _shardsvrCreateCollection: {skip: "internal command"},
     _shardsvrCreateCollectionParticipant: {skip: "internal command"},
     _shardsvrDropCollection: {skip: "internal command"},
+    _shardsvrDropCollectionIfUUIDNotMatching: {skip: "internal command"},
     _shardsvrDropCollectionParticipant: {skip: "internal command"},
     _shardsvrDropDatabase: {skip: "internal command"},
     _shardsvrDropDatabaseParticipant: {skip: "internal command"},
@@ -141,6 +143,7 @@ let testCases = {
     _shardsvrRenameCollectionParticipantUnblock: {skip: "internal command"},
     _shardsvrReshardCollection: {skip: "internal command"},
     _shardsvrReshardingOperationTime: {skip: "internal command"},
+    _shardsvrSetAllowMigrations: {skip: "internal command"},
     _shardsvrShardCollection: {skip: "internal command"},
     _transferMods: {skip: "internal command"},
     _vectorClockPersist: {skip: "internal command"},
@@ -191,6 +194,7 @@ let testCases = {
     },
     applyOps: {skip: "internal command"},
     authenticate: {skip: "does not accept read or write concern"},
+    autoSplitVector: {skip: "internal command"},
     availableQueryOptions: {skip: "internal command"},
     balancerCollectionStatus: {skip: "does not accept read or write concern"},
     balancerStart: {skip: "does not accept read or write concern"},
@@ -547,6 +551,7 @@ let testCases = {
         checkWriteConcern: true,
     },
     repairDatabase: {skip: "does not accept read or write concern"},
+    repairShardedCollectionChunksHistory: {skip: "does not accept read or write concern"},
     replSetAbortPrimaryCatchUp: {skip: "does not accept read or write concern"},
     replSetFreeze: {skip: "does not accept read or write concern"},
     replSetGetConfig: {skip: "does not accept read or write concern"},
@@ -624,6 +629,7 @@ let testCases = {
     saslStart: {skip: "does not accept read or write concern"},
     sbe: {skip: "internal command"},
     serverStatus: {skip: "does not accept read or write concern"},
+    setAllowMigrations: {skip: "does not accept read or write concern"},
     setAuditConfig: {skip: "does not accept read or write concern"},
     setCommittedSnapshot: {skip: "internal command"},
     setDefaultRWConcern: {skip: "special case (must run after all other commands)"},
