@@ -12,11 +12,17 @@
 
 load("jstests/sharding/libs/find_chunks_util.js");
 
-// The total size of the zones and chunks documents that the config server writes in a single
-// replica set transaction totals to around 60 MB. WWhen combined with the other operations and
-// transactions occurring in the config server, this large transaction causes WiredTiger to run out
-// of dirty cache space. Hence, we need to increase the wiredTigerCacheSizeGB to 1 GB.
-const st = new ShardingTest({mongos: 1, shards: 2, config: 1});
+const st = new ShardingTest({
+    mongos: 1,
+    shards: 2,
+    config: 1,
+    other: {
+        configOptions: {
+            setParameter:
+                {reshardingCriticalSectionTimeoutMillis: 24 * 60 * 60 * 1000 /* 1 day */}
+        }
+    }
+});
 
 const kDbName = 'db';
 const collName = 'foo';
@@ -26,7 +32,6 @@ const mongos = st.s;
 assert.commandWorked(mongos.adminCommand({enableSharding: kDbName}));
 assert.commandWorked(mongos.adminCommand({shardCollection: ns, key: {oldKey: 1}}));
 
-// TODO SERVER-SERVER-57095 increase the number of zones
 let nZones = 10000;
 let zones = [];
 let shard0Zones = [];
