@@ -21,6 +21,7 @@
 
 load('jstests/libs/profiler.js');
 load('jstests/sharding/libs/last_lts_mongos_commands.js');
+load('jstests/sharding/libs/shard_versioning_util.js');
 
 let db = "test";
 let coll = "foo";
@@ -269,6 +270,7 @@ let testCases = {
     removeShard: {skip: "primary only"},
     removeShardFromZone: {skip: "primary only"},
     renameCollection: {skip: "primary only"},
+    repairShardedCollectionChunksHistory: {skip: "does not return user data"},
     replSetAbortPrimaryCatchUp: {skip: "does not return user data"},
     replSetFreeze: {skip: "does not return user data"},
     replSetGetConfig: {skip: "does not return user data"},
@@ -296,6 +298,7 @@ let testCases = {
     saslStart: {skip: "primary only"},
     sbe: {skip: "internal command"},
     serverStatus: {skip: "does not return user data"},
+    setAllowMigrations: {skip: "primary only"},
     setAuditConfig: {skip: "does not return user data"},
     setCommittedSnapshot: {skip: "does not return user data"},
     setDefaultRWConcern: {skip: "primary only"},
@@ -468,8 +471,8 @@ for (let command of commands) {
             profileDB: recipientShardSecondary.getDB(db),
             filter: Object.extend({
                 "command.shardVersion": {"$exists": true},
-                "command.shardVersion.0": {$ne: Timestamp(0, 0)},
-                "command.shardVersion.1": {$ne: ObjectId("00000000ffffffffffffffff")},
+                "command.shardVersion.0": {$ne: ShardVersioningUtil.kIgnoredShardVersion[0]},
+                "command.shardVersion.1": {$ne: ShardVersioningUtil.kIgnoredShardVersion[1]},
                 "command.$readPreference": {"mode": "secondary"},
                 "command.readConcern": {"level": "local"},
                 "errCode": {"$ne": ErrorCodes.StaleConfig},
