@@ -64,14 +64,14 @@ ConnectionString RemoteCommandTargeterRS::connectionString() {
     return uassertStatusOK(ConnectionString::parse(_rsMonitor->getServerAddress()));
 }
 
-SemiFuture<HostAndPort> RemoteCommandTargeterRS::findHostWithMaxWait(
-    const ReadPreferenceSetting& readPref, Milliseconds maxWait) {
-    return _rsMonitor->getHostOrRefresh(readPref, maxWait);
+SemiFuture<HostAndPort> RemoteCommandTargeterRS::findHost(const ReadPreferenceSetting& readPref,
+                                                          const CancellationToken& cancelToken) {
+    return _rsMonitor->getHostOrRefresh(readPref, cancelToken);
 }
 
-SemiFuture<std::vector<HostAndPort>> RemoteCommandTargeterRS::findHostsWithMaxWait(
-    const ReadPreferenceSetting& readPref, Milliseconds maxWait) {
-    return _rsMonitor->getHostsOrRefresh(readPref, maxWait);
+SemiFuture<std::vector<HostAndPort>> RemoteCommandTargeterRS::findHosts(
+    const ReadPreferenceSetting& readPref, const CancellationToken& cancelToken) {
+    return _rsMonitor->getHostsOrRefresh(readPref, cancelToken);
 }
 
 StatusWith<HostAndPort> RemoteCommandTargeterRS::findHost(OperationContext* opCtx,
@@ -81,16 +81,19 @@ StatusWith<HostAndPort> RemoteCommandTargeterRS::findHost(OperationContext* opCt
         return interruptStatus;
     }
 
-    // Enforce a 20-second ceiling on the time spent looking for a host. This conforms with the
-    // behavior used throughout mongos prior to version 3.4, but is not fundamentally desirable.
-    // See comment in remote_command_targeter.h for details.
-    return _rsMonitor
-        ->getHostOrRefresh(readPref,
-                           std::min<Milliseconds>(opCtx->getRemainingMaxTimeMillis(), Seconds(20)))
-        .getNoThrow(opCtx);
+    bool maxTimeMsLesser =
+        (opCtx->getRemainingMaxTimeMillis() < ReplicaSetMonitorInterface::kDefaultFindHostTimeout);
+    auto swHostAndPort =
+        _rsMonitor->getHostOrRefresh(readPref, opCtx->getCancellationToken()).getNoThrow(opCtx);
+
+    if (maxTimeMsLesser && swHostAndPort.getStatus() == ErrorCodes::FailedToSatisfyReadPreference) {
+        return Status(ErrorCodes::MaxTimeMSExpired, "operation timed out");
+    }
+
+    return swHostAndPort;
 }
 
-void RemoteCommandTargeterRS::markHostNotMaster(const HostAndPort& host, const Status& status) {
+void RemoteCommandTargeterRS::markHostNotPrimary(const HostAndPort& host, const Status& status) {
     invariant(_rsMonitor);
 
     _rsMonitor->failedHost(host, status);
