@@ -30,11 +30,14 @@
 #include "mongo/platform/basic.h"
 
 #include <boost/optional/optional_io.hpp>
+#include <memory>
 #include <pcrecpp.h>
 
 #include "mongo/bson/util/builder.h"
 #include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/db/catalog/index_build_block.h"
 #include "mongo/db/catalog/index_catalog.h"
+#include "mongo/db/catalog/uncommitted_collections.h"
 #include "mongo/db/client.h"
 #include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/concurrency/write_conflict_exception.h"
@@ -43,6 +46,7 @@
 #include "mongo/db/jsobj.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/op_observer.h"
+#include "mongo/db/op_observer_impl.h"
 #include "mongo/db/op_observer_registry.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/repl/drop_pending_collection_reaper.h"
@@ -52,21 +56,13 @@
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
 #include "mongo/db/repl/storage_interface_mock.h"
-#include "mongo/db/s/op_observer_sharding_impl.h"
 #include "mongo/db/service_context_d_test_fixture.h"
-#include "mongo/db/storage/durable_catalog.h"
-#include "mongo/stdx/memory.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/scopeguard.h"
 
+namespace mongo {
 namespace {
 
-using namespace mongo;
-
-ServiceContext::UniqueOperationContext makeOpCtx() {
-    return cc().makeOperationContext();
-}
-
 class DatabaseTest : public ServiceContextMongoDTest {
 private:
     void setUp() override;
@@ -84,15 +80,14 @@ void DatabaseTest::setUp() {
     auto service = getServiceContext();
     _opCtx = cc().makeOperationContext();
 
-    repl::StorageInterface::set(service, stdx::make_unique<repl::StorageInterfaceMock>());
+    repl::StorageInterface::set(service, std::make_unique<repl::StorageInterfaceMock>());
     repl::DropPendingCollectionReaper::set(
         service,
-        stdx::make_unique<repl::DropPendingCollectionReaper>(repl::StorageInterface::get(service)));
+        std::make_unique<repl::DropPendingCollectionReaper>(repl::StorageInterface::get(service)));
 
     // Set up ReplicationCoordinator and create oplog.
     repl::ReplicationCoordinator::set(service,
-                                      stdx::make_unique<repl::ReplicationCoordinatorMock>(service));
-    repl::setOplogCollectionName(service);
+                                      std::make_unique<repl::ReplicationCoordinatorMock>(service));
     repl::createOplog(_opCtx.get());
 
     // Ensure that we are primary.
@@ -103,7 +98,7 @@ void DatabaseTest::setUp() {
     // repl::logOp(). repl::logOp() will also store the oplog entry's optime in ReplClientInfo.
     OpObserverRegistry* opObserverRegistry =
         dynamic_cast<OpObserverRegistry*>(service->getOpObserver());
-    opObserverRegistry->addObserver(stdx::make_unique<OpObserverShardingImpl>());
+    opObserverRegistry->addObserver(std::make_unique<OpObserverImpl>());
 
     _nss = NamespaceString("test.foo");
 }
@@ -121,8 +116,8 @@ void DatabaseTest::tearDown() {
 
 TEST_F(DatabaseTest, SetDropPendingThrowsExceptionIfDatabaseIsAlreadyInADropPendingState) {
     writeConflictRetry(_opCtx.get(), "testSetDropPending", _nss.ns(), [this] {
-        AutoGetOrCreateDb autoDb(_opCtx.get(), _nss.db(), MODE_X);
-        auto db = autoDb.getDb();
+        AutoGetDb autoDb(_opCtx.get(), _nss.db(), MODE_X);
+        auto db = autoDb.ensureDbExists();
         ASSERT_TRUE(db);
 
         ASSERT_FALSE(db->isDropPending(_opCtx.get()));
@@ -144,8 +139,8 @@ TEST_F(DatabaseTest, SetDropPendingThrowsExceptionIfDatabaseIsAlreadyInADropPend
 TEST_F(DatabaseTest, CreateCollectionThrowsExceptionWhenDatabaseIsInADropPendingState) {
     writeConflictRetry(
         _opCtx.get(), "testÇreateCollectionWhenDatabaseIsInADropPendingState", _nss.ns(), [this] {
-            AutoGetOrCreateDb autoDb(_opCtx.get(), _nss.db(), MODE_X);
-            auto db = autoDb.getDb();
+            AutoGetDb autoDb(_opCtx.get(), _nss.db(), MODE_X);
+            auto db = autoDb.ensureDbExists();
             ASSERT_TRUE(db);
 
             db->setDropPending(_opCtx.get(), true);
@@ -173,21 +168,29 @@ void _testDropCollection(OperationContext* opCtx,
                          bool createCollectionBeforeDrop,
                          const repl::OpTime& dropOpTime = {},
                          const CollectionOptions& collOpts = {}) {
+    if (createCollectionBeforeDrop) {
+        writeConflictRetry(opCtx, "testDropCollection", nss.ns(), [=] {
+            WriteUnitOfWork wuow(opCtx);
+            AutoGetDb autoDb(opCtx, nss.db(), MODE_X);
+            auto db = autoDb.ensureDbExists();
+            ASSERT_TRUE(db);
+            ASSERT_TRUE(db->createCollection(opCtx, nss, collOpts));
+            wuow.commit();
+        });
+    }
+
     writeConflictRetry(opCtx, "testDropCollection", nss.ns(), [=] {
-        AutoGetOrCreateDb autoDb(opCtx, nss.db(), MODE_X);
-        auto db = autoDb.getDb();
+        AutoGetDb autoDb(opCtx, nss.db(), MODE_X);
+        auto db = autoDb.ensureDbExists();
         ASSERT_TRUE(db);
 
         WriteUnitOfWork wuow(opCtx);
-        if (createCollectionBeforeDrop) {
-            ASSERT_TRUE(db->createCollection(opCtx, nss, collOpts));
-        } else {
-            ASSERT_FALSE(db->getCollection(opCtx, nss));
+        if (!createCollectionBeforeDrop) {
+            ASSERT_FALSE(CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss));
         }
 
         ASSERT_OK(db->dropCollection(opCtx, nss, dropOpTime));
-
-        ASSERT_FALSE(db->getCollection(opCtx, nss));
+        ASSERT_FALSE(CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss));
         wuow.commit();
     });
 }
@@ -242,17 +245,19 @@ TEST_F(DatabaseTest, DropCollectionRejectsProvidedDropOpTimeIfWritesAreReplicate
 
     auto opCtx = _opCtx.get();
     auto nss = _nss;
-    writeConflictRetry(opCtx, "testDropOpTimeWithReplicated", nss.ns(), [opCtx, nss] {
-        AutoGetOrCreateDb autoDb(opCtx, nss.db(), MODE_X);
-        auto db = autoDb.getDb();
+    AutoGetDb autoDb(opCtx, nss.db(), MODE_X);
+    auto db = autoDb.ensureDbExists();
+    writeConflictRetry(opCtx, "testDropOpTimeWithReplicated", nss.ns(), [&] {
         ASSERT_TRUE(db);
 
         WriteUnitOfWork wuow(opCtx);
         ASSERT_TRUE(db->createCollection(opCtx, nss));
-
-        repl::OpTime dropOpTime(Timestamp(Seconds(100), 0), 1LL);
-        ASSERT_EQUALS(ErrorCodes::BadValue, db->dropCollection(opCtx, nss, dropOpTime));
+        wuow.commit();
     });
+
+    WriteUnitOfWork wuow(opCtx);
+    repl::OpTime dropOpTime(Timestamp(Seconds(100), 0), 1LL);
+    ASSERT_EQUALS(ErrorCodes::BadValue, db->dropCollection(opCtx, nss, dropOpTime));
 }
 
 TEST_F(
@@ -284,14 +289,14 @@ TEST_F(
 void _testDropCollectionThrowsExceptionIfThereAreIndexesInProgress(OperationContext* opCtx,
                                                                    const NamespaceString& nss) {
     writeConflictRetry(opCtx, "testDropCollectionWithIndexesInProgress", nss.ns(), [opCtx, nss] {
-        AutoGetOrCreateDb autoDb(opCtx, nss.db(), MODE_X);
-        auto db = autoDb.getDb();
+        AutoGetDb autoDb(opCtx, nss.db(), MODE_X);
+        auto db = autoDb.ensureDbExists();
         ASSERT_TRUE(db);
 
         Collection* collection = nullptr;
         {
             WriteUnitOfWork wuow(opCtx);
-            ASSERT_TRUE(collection = db->createCollection(opCtx, nss));
+            ASSERT_TRUE((collection = db->createCollection(opCtx, nss)));
             wuow.commit();
         }
 
@@ -299,11 +304,10 @@ void _testDropCollectionThrowsExceptionIfThereAreIndexesInProgress(OperationCont
         ASSERT_EQUALS(indexCatalog->numIndexesInProgress(opCtx), 0);
         auto indexInfoObj = BSON("v" << int(IndexDescriptor::kLatestIndexVersion) << "key"
                                      << BSON("a" << 1) << "name"
-                                     << "a_1"
-                                     << "ns" << nss.ns());
+                                     << "a_1");
 
-        auto indexBuildBlock =
-            indexCatalog->createIndexBuildBlock(opCtx, indexInfoObj, IndexBuildMethod::kHybrid);
+        auto indexBuildBlock = std::make_unique<IndexBuildBlock>(
+            collection->ns(), indexInfoObj, IndexBuildMethod::kHybrid, UUID::gen());
         {
             WriteUnitOfWork wuow(opCtx);
             ASSERT_OK(indexBuildBlock->init(opCtx, collection));
@@ -313,7 +317,8 @@ void _testDropCollectionThrowsExceptionIfThereAreIndexesInProgress(OperationCont
             WriteUnitOfWork wuow(opCtx);
             indexBuildBlock->success(opCtx, collection);
             wuow.commit();
-            indexBuildBlock->deleteTemporaryTables(opCtx);
+            indexBuildBlock->finalizeTemporaryTables(
+                opCtx, TemporaryRecordStore::FinalizationAction::kDelete);
         });
 
         ASSERT_GREATER_THAN(indexCatalog->numIndexesInProgress(opCtx), 0);
@@ -344,37 +349,40 @@ TEST_F(DatabaseTest, RenameCollectionPreservesUuidOfSourceCollectionAndUpdatesUu
     auto toNss = NamespaceString(fromNss.getSisterNS("bar"));
     ASSERT_NOT_EQUALS(fromNss, toNss);
 
-    writeConflictRetry(opCtx, "testRenameCollection", fromNss.ns(), [=] {
-        AutoGetOrCreateDb autoDb(opCtx, fromNss.db(), MODE_X);
-        auto db = autoDb.getDb();
-        ASSERT_TRUE(db);
+    AutoGetDb autoDb(opCtx, fromNss.db(), MODE_X);
+    auto db = autoDb.ensureDbExists();
+    ASSERT_TRUE(db);
 
-        auto fromUuid = UUID::gen();
-
-        auto&& catalog = CollectionCatalog::get(opCtx);
-        ASSERT_EQUALS(boost::none, catalog.lookupNSSByUUID(fromUuid));
+    auto fromUuid = UUID::gen();
+    writeConflictRetry(opCtx, "create", fromNss.ns(), [&] {
+        auto catalog = CollectionCatalog::get(opCtx);
+        ASSERT_EQUALS(boost::none, catalog->lookupNSSByUUID(opCtx, fromUuid));
 
         WriteUnitOfWork wuow(opCtx);
         CollectionOptions fromCollectionOptions;
         fromCollectionOptions.uuid = fromUuid;
         ASSERT_TRUE(db->createCollection(opCtx, fromNss, fromCollectionOptions));
-        ASSERT_EQUALS(fromNss, *catalog.lookupNSSByUUID(fromUuid));
+        ASSERT_EQUALS(fromNss, *catalog->lookupNSSByUUID(opCtx, fromUuid));
+        wuow.commit();
+    });
 
+    writeConflictRetry(opCtx, "rename", fromNss.ns(), [&] {
+        WriteUnitOfWork wuow(opCtx);
         auto stayTemp = false;
         ASSERT_OK(db->renameCollection(opCtx, fromNss, toNss, stayTemp));
 
-        ASSERT_FALSE(db->getCollection(opCtx, fromNss));
-        auto toCollection = db->getCollection(opCtx, toNss);
+        auto catalog = CollectionCatalog::get(opCtx);
+        ASSERT_FALSE(catalog->lookupCollectionByNamespace(opCtx, fromNss));
+        auto toCollection = catalog->lookupCollectionByNamespace(opCtx, toNss);
         ASSERT_TRUE(toCollection);
 
-        auto toCollectionOptions =
-            DurableCatalog::get(opCtx)->getCollectionOptions(opCtx, toCollection->ns());
+        const auto& toCollectionOptions = toCollection->getCollectionOptions();
 
         auto toUuid = toCollectionOptions.uuid;
         ASSERT_TRUE(toUuid);
         ASSERT_EQUALS(fromUuid, *toUuid);
 
-        ASSERT_EQUALS(toNss, *catalog.lookupNSSByUUID(*toUuid));
+        ASSERT_EQUALS(toNss, *catalog->lookupNSSByUUID(opCtx, *toUuid));
 
         wuow.commit();
     });
@@ -383,32 +391,19 @@ TEST_F(DatabaseTest, RenameCollectionPreservesUuidOfSourceCollectionAndUpdatesUu
 TEST_F(DatabaseTest,
        MakeUniqueCollectionNamespaceReturnsFailedToParseIfModelDoesNotContainPercentSign) {
     writeConflictRetry(_opCtx.get(), "testMakeUniqueCollectionNamespace", _nss.ns(), [this] {
-        AutoGetOrCreateDb autoDb(_opCtx.get(), _nss.db(), MODE_X);
-        auto db = autoDb.getDb();
+        AutoGetDb autoDb(_opCtx.get(), _nss.db(), MODE_X);
+        auto db = autoDb.ensureDbExists();
         ASSERT_TRUE(db);
         ASSERT_EQUALS(
             ErrorCodes::FailedToParse,
             db->makeUniqueCollectionNamespace(_opCtx.get(), "CollectionModelWithoutPercentSign"));
-
-        // Generated namespace has to satisfy namespace length constraints so we will reject
-        // any collection model where the first substituted percent sign will not be in the
-        // generated namespace. See NamespaceString::MaxNsCollectionLen.
-        auto dbPrefix = _nss.db() + ".";
-        auto modelTooLong =
-            (StringBuilder() << dbPrefix
-                             << std::string('x',
-                                            NamespaceString::MaxNsCollectionLen - dbPrefix.size())
-                             << "%")
-                .str();
-        ASSERT_EQUALS(ErrorCodes::FailedToParse,
-                      db->makeUniqueCollectionNamespace(_opCtx.get(), modelTooLong));
     });
 }
 
 TEST_F(DatabaseTest, MakeUniqueCollectionNamespaceReplacesPercentSignsWithRandomCharacters) {
     writeConflictRetry(_opCtx.get(), "testMakeUniqueCollectionNamespace", _nss.ns(), [this] {
-        AutoGetOrCreateDb autoDb(_opCtx.get(), _nss.db(), MODE_X);
-        auto db = autoDb.getDb();
+        AutoGetDb autoDb(_opCtx.get(), _nss.db(), MODE_X);
+        auto db = autoDb.ensureDbExists();
         ASSERT_TRUE(db);
 
         auto model = "tmp%%%%"_sd;
@@ -449,8 +444,8 @@ TEST_F(
     DatabaseTest,
     MakeUniqueCollectionNamespaceReturnsNamespaceExistsIfGeneratedNamesMatchExistingCollections) {
     writeConflictRetry(_opCtx.get(), "testMakeUniqueCollectionNamespace", _nss.ns(), [this] {
-        AutoGetOrCreateDb autoDb(_opCtx.get(), _nss.db(), MODE_X);
-        auto db = autoDb.getDb();
+        AutoGetDb autoDb(_opCtx.get(), _nss.db(), MODE_X);
+        auto db = autoDb.ensureDbExists();
         ASSERT_TRUE(db);
 
         auto model = "tmp%"_sd;
@@ -506,7 +501,7 @@ TEST_F(DatabaseTest, AutoGetCollectionForReadCommandSucceedsWithDeadlineNow) {
     ASSERT(_opCtx.get()->lockState()->isCollectionLockedForMode(nss, MODE_X));
     try {
         AutoGetCollectionForReadCommand db(
-            _opCtx.get(), nss, AutoGetCollection::kViewsForbidden, Date_t::now());
+            _opCtx.get(), nss, AutoGetCollectionViewMode::kViewsForbidden, Date_t::now());
     } catch (const ExceptionFor<ErrorCodes::LockTimeout>&) {
         FAIL("Should get the db within the timeout");
     }
@@ -520,7 +515,7 @@ TEST_F(DatabaseTest, AutoGetCollectionForReadCommandSucceedsWithDeadlineMin) {
     ASSERT(_opCtx.get()->lockState()->isCollectionLockedForMode(nss, MODE_X));
     try {
         AutoGetCollectionForReadCommand db(
-            _opCtx.get(), nss, AutoGetCollection::kViewsForbidden, Date_t());
+            _opCtx.get(), nss, AutoGetCollectionViewMode::kViewsForbidden, Date_t());
     } catch (const ExceptionFor<ErrorCodes::LockTimeout>&) {
         FAIL("Should get the db within the timeout");
     }
@@ -531,8 +526,8 @@ TEST_F(DatabaseTest, CreateCollectionProhibitsReplicatedCollectionsWithoutIdInde
                        "testÇreateCollectionProhibitsReplicatedCollectionsWithoutIdIndex",
                        _nss.ns(),
                        [this] {
-                           AutoGetOrCreateDb autoDb(_opCtx.get(), _nss.db(), MODE_X);
-                           auto db = autoDb.getDb();
+                           AutoGetDb autoDb(_opCtx.get(), _nss.db(), MODE_X);
+                           auto db = autoDb.ensureDbExists();
                            ASSERT_TRUE(db);
 
                            WriteUnitOfWork wuow(_opCtx.get());
@@ -551,5 +546,5 @@ TEST_F(DatabaseTest, CreateCollectionProhibitsReplicatedCollectionsWithoutIdInde
                        });
 }
 
-
 }  // namespace
+}  // namespace mongo
