@@ -231,8 +231,11 @@ void IndexCatalogEntryImpl::setMultikey(OperationContext* opCtx,
     // it multikey, that write will be marked as "isTrackingMultikeyPathInfo" on the applier's
     // OperationContext and we can safely defer that write to the end of the batch.
     if (MultikeyPathTracker::get(opCtx).isTrackingMultikeyPathInfo()) {
-        MultikeyPathTracker::get(opCtx).addMultikeyPathInfo(
-            {collection->ns(), _descriptor->indexName(), multikeyMetadataKeys, std::move(paths)});
+        MultikeyPathTracker::get(opCtx).addMultikeyPathInfo({collection->ns(),
+                                                             collection->uuid(),
+                                                             _descriptor->indexName(),
+                                                             multikeyMetadataKeys,
+                                                             std::move(paths)});
         return;
     }
 
