@@ -91,6 +91,14 @@ MultiIndexBlock::~MultiIndexBlock() {
 
 MultiIndexBlock::OnCleanUpFn MultiIndexBlock::kNoopOnCleanUpFn = []() {};
 
+MultiIndexBlock::OnCleanUpFn MultiIndexBlock::makeTimestampedOnCleanUpFn(
+    OperationContext* opCtx, const CollectionPtr& coll) {
+    return [opCtx, ns = coll->ns()]() -> Status {
+        opCtx->getServiceContext()->getOpObserver()->onAbortIndexBuildSinglePhase(opCtx, ns);
+        return Status::OK();
+    };
+}
+
 void MultiIndexBlock::abortIndexBuild(OperationContext* opCtx,
                                       CollectionWriter& collection,
                                       OnCleanUpFn onCleanUp) noexcept {
@@ -296,9 +304,12 @@ StatusWith<std::vector<BSONObj>> MultiIndexBlock::init(
             collection->getIndexCatalog()->prepareInsertDeleteOptions(
                 opCtx, collection->ns(), descriptor, &index.options);
 
-            // Index builds always relax constraints and check for violations at commit-time.
+            // Foreground index builds have to check for duplicates. Other index builds can relax
+            // constraints and check for violations at commit-time.
             index.options.getKeysMode = IndexAccessMethod::GetKeysMode::kRelaxConstraints;
-            index.options.dupsAllowed = true;
+            index.options.dupsAllowed = _method == IndexBuildMethod::kForeground
+                ? !descriptor->unique() || _ignoreUnique
+                : true;
             index.options.fromIndexBuilder = true;
 
             LOGV2(20384,
@@ -604,7 +615,15 @@ void MultiIndexBlock::_doCollectionScan(OperationContext* opCtx,
             _insert(opCtx,
                     objToIndex,
                     loc,
-                    /*saveCursorBeforeWrite*/ [&exec] { exec->saveState(); },
+                    /*saveCursorBeforeWrite*/
+                    [&exec, &objToIndex] {
+                        // Update objToIndex so that it continues to point to valid data when the
+                        // cursor is closed. A WCE may occur during a write to index A, and
+                        // objToIndex must still be used when the write is retried or for a write to
+                        // another index (if creating multiple indexes at once)
+                        objToIndex = objToIndex.getOwned();
+                        exec->saveState();
+                    },
                     /*restoreCursorAfterWrite*/ [&] { exec->restoreState(&collection); }));
 
         _failPointHangDuringBuild(opCtx,
@@ -645,6 +664,7 @@ Status MultiIndexBlock::_insert(OperationContext* opCtx,
         // exception.
         try {
             idxStatus = _indexes[i].bulk->insert(opCtx,
+                                                 _indexes[i].block->getPooledBuilder(),
                                                  doc,
                                                  loc,
                                                  _indexes[i].options,
@@ -688,6 +708,10 @@ Status MultiIndexBlock::dumpInsertsFromBulk(
               IndexBuildPhase_serializer(_phase).toString());
     _phase = IndexBuildPhaseEnum::kBulkLoad;
 
+    // Doesn't allow yielding when in a foreground index build.
+    const int32_t kYieldIterations =
+        isBackgroundBuilding() ? internalIndexBuildBulkLoadYieldIterations.load() : 0;
+
     for (size_t i = 0; i < _indexes.size(); i++) {
         // When onDuplicateRecord is passed, 'dupsAllowed' should be passed to reflect whether or
         // not the index is unique.
@@ -706,8 +730,10 @@ Status MultiIndexBlock::dumpInsertsFromBulk(
         try {
             Status status = _indexes[i].real->commitBulk(
                 opCtx,
+                collection,
                 _indexes[i].bulk.get(),
                 dupsAllowed,
+                kYieldIterations,
                 [=](const KeyString::Value& duplicateKey) {
                     // Do not record duplicates when explicitly ignored. This may be the case on
                     // secondaries.
@@ -910,12 +936,6 @@ void MultiIndexBlock::abortWithoutCleanup(OperationContext* opCtx,
         invariant(_buildUUID);
         invariant(_method == IndexBuildMethod::kHybrid);
 
-        // Index builds do not yield locks during the bulk load phase so it is not possible for
-        // rollback to interrupt an index build during this phase.
-        if (!ErrorCodes::isShutdownError(opCtx->checkForInterruptNoAssert())) {
-            invariant(IndexBuildPhaseEnum::kBulkLoad != _phase, str::stream() << *_buildUUID);
-        }
-
         _writeStateToDisk(opCtx, collection);
         action = TemporaryRecordStore::FinalizationAction::kKeep;
     }
