@@ -43,9 +43,7 @@
 #include "mongo/db/auth/action_type.h"
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/auth/privilege.h"
-#include "mongo/db/auth/user_management_commands_parser.h"
 #include "mongo/db/auth/user_name.h"
-#include "mongo/db/background.h"
 #include "mongo/db/catalog/coll_mod.h"
 #include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/catalog/create_collection.h"
@@ -96,24 +94,18 @@
 #include "mongo/util/scopeguard.h"
 
 namespace mongo {
-
-using std::ostringstream;
-using std::string;
-using std::stringstream;
-using std::unique_ptr;
+namespace {
 
 // Failpoint for making filemd5 hang.
 MONGO_FAIL_POINT_DEFINE(waitInFilemd5DuringManualYield);
 
-namespace {
-
 Status _setProfileSettings(OperationContext* opCtx,
                            Database* db,
                            StringData dbName,
                            mongo::CollectionCatalog::ProfileSettings newSettings) {
     invariant(db);
 
-    auto currSettings = CollectionCatalog::get(opCtx).getDatabaseProfileSettings(dbName);
+    auto currSettings = CollectionCatalog::get(opCtx)->getDatabaseProfileSettings(dbName);
 
     if (currSettings == newSettings) {
         return Status::OK();
@@ -121,7 +113,9 @@ Status _setProfileSettings(OperationContext* opCtx,
 
     if (newSettings.level == 0) {
         // No need to create the profile collection.
-        CollectionCatalog::get(opCtx).setDatabaseProfileSettings(dbName, newSettings);
+        CollectionCatalog::write(opCtx, [&](CollectionCatalog& catalog) {
+            catalog.setDatabaseProfileSettings(dbName, newSettings);
+        });
         return Status::OK();
     }
 
@@ -136,7 +130,9 @@ Status _setProfileSettings(OperationContext* opCtx,
         return status;
     }
 
-    CollectionCatalog::get(opCtx).setDatabaseProfileSettings(dbName, newSettings);
+    CollectionCatalog::write(opCtx, [&](CollectionCatalog& catalog) {
+        catalog.setDatabaseProfileSettings(dbName, newSettings);
+    });
 
     return Status::OK();
 }
@@ -176,7 +172,7 @@ protected:
 
         // Fetches the database profiling level + filter or the server default if the db does not
         // exist.
-        auto oldSettings = CollectionCatalog::get(opCtx).getDatabaseProfileSettings(dbName);
+        auto oldSettings = CollectionCatalog::get(opCtx)->getDatabaseProfileSettings(dbName);
 
         if (!readOnly) {
             if (!db) {
@@ -224,6 +220,10 @@ public:
         return false;
     }
 
+    bool collectsResourceConsumptionMetrics() const override {
+        return true;
+    }
+
     virtual std::string parseNs(const std::string& dbname, const BSONObj& cmdObj) const {
         std::string collectionName;
         if (const auto rootElt = cmdObj["root"]) {
@@ -245,7 +245,7 @@ public:
     }
 
     bool run(OperationContext* opCtx,
-             const string& dbname,
+             const std::string& dbname,
              const BSONObj& jsobj,
              BSONObjBuilder& result) {
         const NamespaceString nss(parseNs(dbname, jsobj));
@@ -282,28 +282,28 @@ public:
         BSONObj sort = BSON("files_id" << 1 << "n" << 1);
 
         return writeConflictRetry(opCtx, "filemd5", dbname, [&] {
-            auto qr = std::make_unique<QueryRequest>(nss);
-            qr->setFilter(query);
-            qr->setSort(sort);
+            auto findCommand = std::make_unique<FindCommandRequest>(nss);
+            findCommand->setFilter(query.getOwned());
+            findCommand->setSort(sort.getOwned());
 
-            auto statusWithCQ = CanonicalQuery::canonicalize(opCtx, std::move(qr));
+            auto statusWithCQ = CanonicalQuery::canonicalize(opCtx, std::move(findCommand));
             if (!statusWithCQ.isOK()) {
                 uasserted(17240, "Can't canonicalize query " + query.toString());
                 return false;
             }
-            unique_ptr<CanonicalQuery> cq = std::move(statusWithCQ.getValue());
+            std::unique_ptr<CanonicalQuery> cq = std::move(statusWithCQ.getValue());
 
             // Check shard version at startup.
             // This will throw before we've done any work if shard version is outdated
             // We drop and re-acquire these locks every document because md5'ing is expensive
-            unique_ptr<AutoGetCollectionForReadCommand> ctx(
+            std::unique_ptr<AutoGetCollectionForReadCommand> ctx(
                 new AutoGetCollectionForReadCommand(opCtx, nss));
-            Collection* coll = ctx->getCollection();
+            const CollectionPtr& coll = ctx->getCollection();
 
             auto exec = uassertStatusOK(getExecutor(opCtx,
-                                                    coll,
+                                                    &coll,
                                                     std::move(cq),
-                                                    PlanExecutor::YIELD_MANUAL,
+                                                    PlanYieldPolicy::YieldPolicy::YIELD_MANUAL,
                                                     QueryPlannerParams::NO_TABLE_SCAN));
 
             // We need to hold a lock to clean up the PlanExecutor, so make sure we have one when we
@@ -329,69 +329,68 @@ public:
                 exec.reset();
             });
 
-            BSONObj obj;
-            PlanExecutor::ExecState state;
-            while (PlanExecutor::ADVANCED == (state = exec->getNext(&obj, nullptr))) {
-                BSONElement ne = obj["n"];
-                verify(ne.isNumber());
-                int myn = ne.numberInt();
-                if (n != myn) {
-                    if (partialOk) {
-                        break;  // skipped chunk is probably on another shard
+            try {
+                BSONObj obj;
+                while (PlanExecutor::ADVANCED == exec->getNext(&obj, nullptr)) {
+                    BSONElement ne = obj["n"];
+                    verify(ne.isNumber());
+                    int myn = ne.numberInt();
+                    if (n != myn) {
+                        if (partialOk) {
+                            break;  // skipped chunk is probably on another shard
+                        }
+                        LOGV2(20452,
+                              "Should have chunk: {expected} have: {observed}",
+                              "Unexpected chunk",
+                              "expected"_attr = n,
+                              "observed"_attr = myn);
+                        dumpChunks(opCtx, nss.ns(), query, sort);
+                        uassert(10040, "chunks out of order", n == myn);
                     }
-                    LOGV2(20452,
-                          "Should have chunk: {expected} have: {observed}",
-                          "Unexpected chunk",
-                          "expected"_attr = n,
-                          "observed"_attr = myn);
-                    dumpChunks(opCtx, nss.ns(), query, sort);
-                    uassert(10040, "chunks out of order", n == myn);
+
+                    // make a copy of obj since we access data in it while yielding locks
+                    BSONObj owned = obj.getOwned();
+                    uassert(50848,
+                            str::stream() << "The element that calls binDataClean() must be type "
+                                             "of BinData, but type of misisng found. Field name is "
+                                             "required",
+                            owned["data"]);
+                    uassert(50849,
+                            str::stream() << "The element that calls binDataClean() must be type "
+                                             "of BinData, but type of "
+                                          << owned["data"].type() << " found.",
+                            owned["data"].type() == BSONType::BinData);
+
+                    exec->saveState();
+                    // UNLOCKED
+                    ctx.reset();
+
+                    int len;
+                    const char* data = owned["data"].binDataClean(len);
+                    // This is potentially an expensive operation, so do it out of the lock
+                    md5_append(&st, (const md5_byte_t*)(data), len);
+                    n++;
+
+                    CurOpFailpointHelpers::waitWhileFailPointEnabled(
+                        &waitInFilemd5DuringManualYield, opCtx, "waitInFilemd5DuringManualYield");
+
+                    try {
+                        // RELOCKED
+                        ctx.reset(new AutoGetCollectionForReadCommand(opCtx, nss));
+                    } catch (const StaleConfigException&) {
+                        LOGV2_DEBUG(
+                            20453,
+                            1,
+                            "Chunk metadata changed during filemd5, will retarget and continue");
+                        break;
+                    }
+
+                    // Now that we have the lock again, we can restore the PlanExecutor.
+                    exec->restoreState(&ctx->getCollection());
                 }
-
-                // make a copy of obj since we access data in it while yielding locks
-                BSONObj owned = obj.getOwned();
-                uassert(50848,
-                        str::stream() << "The element that calls binDataClean() must be type "
-                                         "of BinData, but type of misisng found. Field name is "
-                                         "required",
-                        owned["data"]);
-                uassert(50849,
-                        str::stream() << "The element that calls binDataClean() must be type "
-                                         "of BinData, but type of "
-                                      << owned["data"].type() << " found.",
-                        owned["data"].type() == BSONType::BinData);
-
-                exec->saveState();
-                // UNLOCKED
-                ctx.reset();
-
-                int len;
-                const char* data = owned["data"].binDataClean(len);
-                // This is potentially an expensive operation, so do it out of the lock
-                md5_append(&st, (const md5_byte_t*)(data), len);
-                n++;
-
-                CurOpFailpointHelpers::waitWhileFailPointEnabled(
-                    &waitInFilemd5DuringManualYield, opCtx, "waitInFilemd5DuringManualYield");
-
-                try {
-                    // RELOCKED
-                    ctx.reset(new AutoGetCollectionForReadCommand(opCtx, nss));
-                } catch (const StaleConfigException&) {
-                    LOGV2_DEBUG(
-                        20453,
-                        1,
-                        "Chunk metadata changed during filemd5, will retarget and continue");
-                    break;
-                }
-
-                // Now that we have the lock again, we can restore the PlanExecutor.
-                exec->restoreState();
-            }
-
-            if (PlanExecutor::FAILURE == state) {
-                uassertStatusOK(WorkingSetCommon::getMemberObjectStatus(obj).withContext(
-                    "Executor error during filemd5 command"));
+            } catch (DBException& exception) {
+                exception.addContext("Executor error during filemd5 command");
+                throw;
             }
 
             if (partialOk)
@@ -408,13 +407,13 @@ public:
     }
 
     void dumpChunks(OperationContext* opCtx,
-                    const string& ns,
+                    const std::string& ns,
                     const BSONObj& query,
                     const BSONObj& sort) {
         DBDirectClient client(opCtx);
         Query q(query);
         q.sort(sort);
-        unique_ptr<DBClientCursor> c = client.query(NamespaceString(ns), q);
+        std::unique_ptr<DBClientCursor> c = client.query(NamespaceString(ns), q);
         while (c->more()) {
             LOGV2(20454, "Chunk: {chunk}", "Dumping chunks", "chunk"_attr = c->nextSafe());
         }
@@ -439,7 +438,7 @@ public:
     }
 
     virtual bool run(OperationContext* opCtx,
-                     const string& dbname,
+                     const std::string& dbname,
                      const BSONObj& cmdObj,
                      BSONObjBuilder& result) {
         result << "options" << QueryOption_AllSupported;
