@@ -29,6 +29,7 @@
 #include "mongo/platform/basic.h"
 
 #include "mongo/bson/util/bson_check.h"
+#include "mongo/db/auth/authorization_checks.h"
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/catalog/document_validation.h"
@@ -65,9 +66,9 @@ Status OplogApplicationChecks::checkOperationAuthorization(OperationContext* opC
 
     if (oplogEntry.hasField("ui"_sd)) {
         // ns by UUID overrides the ns specified if they are different.
-        auto& catalog = CollectionCatalog::get(opCtx);
+        auto catalog = CollectionCatalog::get(opCtx);
         boost::optional<NamespaceString> uuidCollNS =
-            catalog.lookupNSSByUUID(opCtx, getUUIDFromOplogEntry(oplogEntry));
+            catalog->lookupNSSByUUID(opCtx, getUUIDFromOplogEntry(oplogEntry));
         if (uuidCollNS && *uuidCollNS != ns)
             ns = *uuidCollNS;
     }
@@ -105,7 +106,7 @@ Status OplogApplicationChecks::checkOperationAuthorization(OperationContext* opC
     }
 
     if (opType == "i"_sd) {
-        return authSession->checkAuthForInsert(opCtx, ns);
+        return auth::checkAuthForInsert(authSession, opCtx, ns);
     } else if (opType == "u"_sd) {
         BSONElement o2Elem = oplogEntry["o2"];
         checkBSONType(BSONType::Object, o2Elem);
@@ -119,10 +120,16 @@ Status OplogApplicationChecks::checkOperationAuthorization(OperationContext* opC
 
         const bool upsert = b || alwaysUpsert;
 
-        return authSession->checkAuthForUpdate(opCtx, ns, o, o2, upsert);
+        return auth::checkAuthForUpdate(authSession,
+                                        opCtx,
+                                        ns,
+                                        o2,
+                                        write_ops::UpdateModification::parseFromOplogEntry(
+                                            o, write_ops::UpdateModification::DiffOptions{}),
+                                        upsert);
     } else if (opType == "d"_sd) {
 
-        return authSession->checkAuthForDelete(opCtx, ns, o);
+        return auth::checkAuthForDelete(authSession, opCtx, ns, o);
     } else if (opType == "db"_sd) {
         // It seems that 'db' isn't used anymore. Require all actions to prevent casual use.
         ActionSet allActions;
@@ -204,9 +211,14 @@ Status OplogApplicationChecks::checkAuthForCommand(OperationContext* opCtx,
                                                    const BSONObj& cmdObj,
                                                    OplogApplicationValidity validity) {
     AuthorizationSession* authSession = AuthorizationSession::get(opCtx->getClient());
+    if (!authSession->isAuthorizedForActionsOnResource(ResourcePattern::forClusterResource(),
+                                                       ActionType::applyOps)) {
+        return Status(ErrorCodes::Unauthorized, "Unauthorized");
+    }
+
     if (validity == OplogApplicationValidity::kNeedsSuperuser) {
         std::vector<Privilege> universalPrivileges;
-        RoleGraph::generateUniversalPrivileges(&universalPrivileges);
+        auth::generateUniversalPrivileges(&universalPrivileges);
         if (!authSession->isAuthorizedForPrivileges(universalPrivileges)) {
             return Status(ErrorCodes::Unauthorized, "Unauthorized");
         }
