@@ -35,6 +35,7 @@
 
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/catalog/coll_mod.h"
+#include "mongo/db/catalog/collection_catalog_helper.h"
 #include "mongo/db/catalog/database.h"
 #include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/catalog/drop_collection.h"
@@ -60,6 +61,7 @@
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/tenant_migration_donor_service.h"
 #include "mongo/db/repl/tenant_migration_recipient_service.h"
+#include "mongo/db/s/balancer/balancer.h"
 #include "mongo/db/s/config/sharding_catalog_manager.h"
 #include "mongo/db/s/resharding/coordinator_document_gen.h"
 #include "mongo/db/s/resharding/resharding_coordinator_service.h"
@@ -399,10 +401,13 @@ public:
 
         boost::optional<Timestamp> changeTimestamp;
         if (serverGlobalParams.clusterRole == ClusterRole::ConfigServer) {
+            // TODO(SERVER-53283): Remove the call to requestPause()
+            // to allow the execution of balancer rounds during setFCV().
+            auto scopedBalancerPauseRequest = Balancer::get(opCtx)->requestPause();
+
             // The Config Server creates a new ID (i.e., timestamp) when it receives an upgrade or
             // downgrade request. Alternatively, the request refers to a previously aborted
             // operation for which the local FCV document must contain the ID to be reused.
-
             if (!serverGlobalParams.featureCompatibility.isUpgradingOrDowngrading()) {
                 const auto now = VectorClock::get(opCtx)->getTime();
                 changeTimestamp = now.clusterTime().asTimestamp();
@@ -587,13 +592,15 @@ private:
         }
 
         {
-            // Take the global lock in S mode to create a barrier for operations taking the global
-            // IX or X locks. This ensures that either:
+            // Take the FCV full transition lock in S mode to create a barrier for operations taking
+            // the global IX or X locks, which implicitly take the FCV full transition lock in IX
+            // mode (aside from those which explicitly opt out). This ensures that either:
             //   - The global IX/X locked operation will start after the FCV change, see the
             //     upgrading to the latest FCV and act accordingly.
             //   - The global IX/X locked operation began prior to the FCV change, is acting on that
             //     assumption and will finish before upgrade procedures begin right after this.
-            Lock::GlobalLock lk(opCtx, MODE_S);
+            Lock::ResourceLock lk(
+                opCtx, opCtx->lockState(), resourceIdFeatureCompatibilityVersion, MODE_S);
         }
 
         uassert(ErrorCodes::Error(549180),
@@ -671,6 +678,80 @@ private:
         }
     }
 
+    /**
+     * TTL indexes with NaN 'expireAfterSeconds' are only supported in 5.0. If the user tries to
+     * downgrade the cluster to an earlier version, they must first remove all TTL indexes with
+     * NaN 'expireAfterSeconds'.
+     */
+    void _disallowTTLIndexesWithNaNExpireAfterSecondsOnDowngrade(OperationContext* opCtx) {
+        auto collCatalog = CollectionCatalog::get(opCtx);
+        for (const auto& db : collCatalog->getAllDbNames()) {
+            for (auto collIt = collCatalog->begin(opCtx, db); collIt != collCatalog->end(opCtx);
+                 ++collIt) {
+                NamespaceStringOrUUID collName(
+                    collCatalog->lookupNSSByUUID(opCtx, collIt.uuid().get()).get());
+                AutoGetCollectionForRead coll(opCtx, collName);
+                if (!coll) {
+                    continue;
+                }
+
+                auto idxCatalog = coll->getIndexCatalog();
+                auto iter = idxCatalog->getIndexIterator(opCtx, /*includeUnfinished=*/true);
+                while (iter->more()) {
+                    opCtx->checkForInterrupt();
+                    auto entry = iter->next();
+                    auto desc = entry->descriptor();
+                    const auto& spec = desc->infoObj();
+                    if (!spec.hasField(IndexDescriptor::kExpireAfterSecondsFieldName)) {
+                        continue;
+                    }
+                    uassert(
+                        ErrorCodes::CannotDowngrade,
+                        fmt::format("Cannot downgrade the cluster when there are TTL indexes with "
+                                    "NaN 'expireAfterSeconds' in the catalog; drop all TTL indexes "
+                                    "with NaN 'expireAfterSeconds' before downgrading. "
+                                    "First detected index: namespace: {}, UUID: {}, index: {}",
+                                    coll->ns().toString(),
+                                    coll->uuid().toString(),
+                                    spec.toString()),
+                        !spec[IndexDescriptor::kExpireAfterSecondsFieldName].isNaN());
+                }
+            }
+        }
+    }
+
+    /**
+     * Partial index filters are only supported in 4.7.0 and above. If the user tries to
+     * downgrade the cluster to an earlier version, they must first ensure that there are no
+     * indexes with conflicting options.
+     */
+    void _disallowIndexesWithConflictingOptionsOnDowngrade(OperationContext* opCtx) {
+        auto collCatalog = CollectionCatalog::get(opCtx);
+        for (const auto& db : collCatalog->getAllDbNames()) {
+            for (auto collIt = collCatalog->begin(opCtx, db); collIt != collCatalog->end(opCtx);
+                 ++collIt) {
+                NamespaceStringOrUUID collName(
+                    collCatalog->lookupNSSByUUID(opCtx, collIt.uuid().get()).get());
+                AutoGetCollectionForRead coll(opCtx, collName);
+                if (!coll) {
+                    continue;
+                }
+
+                SimpleBSONObjUnorderedSet indexKeyPatterns;
+                auto indexIterator = coll->getIndexCatalog()->getIndexIterator(
+                    opCtx, /* includeUnfinishedIndexes  */ true);
+                while (indexIterator->more()) {
+                    auto indexKeyPattern = indexIterator->next()->descriptor()->keyPattern();
+                    uassert(ErrorCodes::CannotDowngrade,
+                            str::stream() << "Cannot downgrade the cluster as collection "
+                                          << coll->ns() << " has indexes with conflicting options",
+                            indexKeyPatterns.find(indexKeyPattern) == indexKeyPatterns.end());
+                    indexKeyPatterns.emplace(indexKeyPattern.getOwned());
+                }
+            }
+        }
+    }
+
     void _runDowngrade(OperationContext* opCtx,
                        const SetFeatureCompatibilityVersion& request,
                        boost::optional<Timestamp> changeTimestamp) {
@@ -743,19 +824,24 @@ private:
         }
 
         {
-            // Take the global lock in S mode to create a barrier for operations taking the global
-            // IX or X locks. This ensures that either
+            // Take the FCV full transition lock in S mode to create a barrier for operations taking
+            // the global IX or X locks, which implicitly take the FCV full transition lock in IX
+            // mode (aside from those which explicitly opt out). This ensures that either:
             //   - The global IX/X locked operation will start after the FCV change, see the
-            //     downgrading to the last-lts or last-continuous FCV and act accordingly.
+            //     upgrading to the latest FCV and act accordingly.
             //   - The global IX/X locked operation began prior to the FCV change, is acting on that
-            //     assumption and will finish before downgrade procedures begin right after this.
-            Lock::GlobalLock lk(opCtx, MODE_S);
+            //     assumption and will finish before upgrade procedures begin right after this.
+            Lock::ResourceLock lk(
+                opCtx, opCtx->lockState(), resourceIdFeatureCompatibilityVersion, MODE_S);
         }
 
         uassert(ErrorCodes::Error(549181),
                 "Failing upgrade due to 'failDowngrading' failpoint set",
                 !failDowngrading.shouldFail());
 
+        _disallowTTLIndexesWithNaNExpireAfterSecondsOnDowngrade(opCtx);
+        _disallowIndexesWithConflictingOptionsOnDowngrade(opCtx);
+
         if (serverGlobalParams.clusterRole == ClusterRole::ConfigServer) {
             // Always abort the reshardCollection regardless of version to ensure that it will run
             // on a consistent version from start to finish. This will ensure that it will be able
