@@ -58,6 +58,15 @@ MONGO_FAIL_POINT_DEFINE(enableTestOnlyFlagforRSTL);
 
 namespace {
 
+// Ignore data races in certain functions when running with TSAN. For performance reasons,
+// diagnostic commands are expected to race with concurrent lock acquisitions while gathering
+// statistics.
+#if defined(__has_feature) && __has_feature(thread_sanitizer)
+#define MONGO_TSAN_IGNORE __attribute__((no_sanitize("thread")))
+#else
+#define MONGO_TSAN_IGNORE
+#endif
+
 /**
  * Tracks global (across all clients) lock acquisition statistics, partitioned into multiple
  * buckets to minimize concurrent access conflicts.
@@ -139,8 +148,6 @@ bool LockerImpl::_shouldDelayUnlock(ResourceId resId, LockMode mode) const {
             return false;
 
         case RESOURCE_GLOBAL:
-        case RESOURCE_PBWM:
-        case RESOURCE_RSTL:
         case RESOURCE_DATABASE:
         case RESOURCE_COLLECTION:
         case RESOURCE_METADATA:
@@ -222,6 +229,21 @@ void LockerImpl::dump() const {
           "requests"_attr = entries);
 }
 
+void LockerImpl::_dumpLockerAndLockManagerRequests() {
+    // Log the _requests that this locker holds. This will provide identifying information to cross
+    // reference with the LockManager dump below for extra information.
+    dump();
+
+    // Log the LockManager's lock information. Given the locker 'dump()' above, we should be able to
+    // easily cross reference to find the lock info matching this operation. The LockManager can
+    // safely access (under internal locks) the LockRequest data that the locker cannot.
+    BSONObjBuilder builder;
+    auto lockToClientMap = LockManager::getLockToClientMap(getGlobalServiceContext());
+    getGlobalLockManager()->getLockInfoBSON(lockToClientMap, &builder);
+    auto lockInfo = builder.done();
+    LOGV2_ERROR(5736000, "Operation ending while holding locks.", "LockInfo"_attr = lockInfo);
+}
+
 
 //
 // CondVarLockGrantNotification
@@ -303,7 +325,12 @@ LockerImpl::~LockerImpl() {
     // to delete with unaccounted locks anyways.
     invariant(!inAWriteUnitOfWork());
     invariant(_numResourcesToUnlockAtEndUnitOfWork == 0);
+
+    if (!_requests.empty()) {
+        _dumpLockerAndLockManagerRequests();
+    }
     invariant(_requests.empty());
+
     invariant(_modeForTicket == MODE_NONE);
 
     // Reset the locking statistics so the object can be reused
@@ -412,8 +439,7 @@ bool LockerImpl::unlockGlobal() {
         // error for any lock used with multi-granularity locking to have more references than
         // the global lock, because every scope starts by calling lockGlobal.
         const auto resType = it.key().getType();
-        if (resType == RESOURCE_GLOBAL || resType == RESOURCE_PBWM || resType == RESOURCE_RSTL ||
-            resType == RESOURCE_MUTEX) {
+        if (resType == RESOURCE_GLOBAL || resType == RESOURCE_MUTEX) {
             it.next();
         } else {
             invariant(_unlockImpl(&it));
@@ -676,13 +702,7 @@ ResourceId LockerImpl::getWaitingResource() const {
     return _waitingResource;
 }
 
-// Ignore data races in this function when running with TSAN. For performance reasons, diagnostic
-// commands are expected to race with concurrent lock acquisitions while gathering statistics.
-#if defined(__has_feature)
-#if __has_feature(thread_sanitizer)
-__attribute__((no_sanitize("thread")))
-#endif
-#endif
+MONGO_TSAN_IGNORE
 void LockerImpl::getLockerInfo(LockerInfo* lockerInfo,
                                const boost::optional<SingleThreadedLockStats> lockStatsBase) const {
     invariant(lockerInfo);
@@ -770,8 +790,9 @@ bool LockerImpl::saveLockStateAndUnlock(Locker::LockSnapshot* stateOut) {
 
         // We should never have to save and restore metadata locks.
         invariant(RESOURCE_DATABASE == resType || RESOURCE_COLLECTION == resType ||
-                  (RESOURCE_PBWM == resType && isSharedLockMode(it->mode)) ||
-                  (RESOURCE_RSTL == resType && it->mode == MODE_IX));
+                  (resId == resourceIdParallelBatchWriterMode && isSharedLockMode(it->mode)) ||
+                  resId == resourceIdFeatureCompatibilityVersion ||
+                  (resId == resourceIdReplicationStateTransitionLock && it->mode == MODE_IX));
 
         // And, stuff the info into the out parameter.
         OneLock info;
@@ -821,6 +842,12 @@ void LockerImpl::restoreLockState(OperationContext* opCtx, const Locker::LockSna
     invariant(_modeForTicket != MODE_NONE);
 }
 
+MONGO_TSAN_IGNORE
+FlowControlTicketholder::CurOp LockerImpl::getFlowControlStats() const {
+    return _flowControlStats;
+}
+
+MONGO_TSAN_IGNORE
 LockResult LockerImpl::_lockBegin(OperationContext* opCtx, ResourceId resId, LockMode mode) {
     dassert(!getWaitingResource().isValid());
 
@@ -857,7 +884,7 @@ LockResult LockerImpl::_lockBegin(OperationContext* opCtx, ResourceId resId, Loc
     // Give priority to the full modes for Global, PBWM, and RSTL resources so we don't stall global
     // operations such as shutdown or stepdown.
     const ResourceType resType = resId.getType();
-    if (resType == RESOURCE_GLOBAL || resType == RESOURCE_PBWM || resType == RESOURCE_RSTL) {
+    if (resType == RESOURCE_GLOBAL) {
         if (mode == MODE_S || mode == MODE_X) {
             request->enqueueAtFront = true;
             request->compatibleFirst = true;
