@@ -57,7 +57,7 @@ class PlanYieldPolicy;
 class CachedPlanStage final : public RequiresAllIndicesStage {
 public:
     CachedPlanStage(ExpressionContext* expCtx,
-                    Collection* collection,
+                    const CollectionPtr& collection,
                     WorkingSet* ws,
                     CanonicalQuery* cq,
                     const QueryPlannerParams& params,
@@ -89,14 +89,6 @@ public:
     Status pickBestPlan(PlanYieldPolicy* yieldPolicy);
 
 private:
-    /**
-     * Passes stats from the trial period run of the cached plan to the plan cache.
-     *
-     * If the plan cache entry is deleted before we get a chance to update it, then this
-     * is a no-op.
-     */
-    void updatePlanCache();
-
     /**
      * Uses the QueryPlanner and the MultiPlanStage to re-generate candidate plans for this
      * query and select a new winner.
