@@ -27,6 +27,7 @@
  *    it in the license file.
  */
 
+#include "mongo/util/assert_util.h"
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/db/exec/collection_scan.h"
@@ -42,12 +43,9 @@
 #include "mongo/db/exec/working_set.h"
 #include "mongo/db/exec/working_set_common.h"
 #include "mongo/db/repl/optime.h"
-#include "mongo/db/storage/oplog_hack.h"
 #include "mongo/logv2/log.h"
 #include "mongo/util/fail_point.h"
 
-#include "mongo/db/client.h"  // XXX-ERH
-
 namespace mongo {
 
 using std::unique_ptr;
@@ -57,7 +55,7 @@ using std::vector;
 const char* CollectionScan::kStageType = "COLLSCAN";
 
 CollectionScan::CollectionScan(ExpressionContext* expCtx,
-                               const Collection* collection,
+                               const CollectionPtr& collection,
                                const CollectionScanParams& params,
                                WorkingSet* workingSet,
                                const MatchExpression* filter)
@@ -67,22 +65,29 @@ CollectionScan::CollectionScan(ExpressionContext* expCtx,
       _params(params) {
     // Explain reports the direction of the collection scan.
     _specificStats.direction = params.direction;
-    _specificStats.minTs = params.minTs;
-    _specificStats.maxTs = params.maxTs;
+    _specificStats.minRecord = params.minRecord;
+    _specificStats.maxRecord = params.maxRecord;
     _specificStats.tailable = params.tailable;
-    if (params.minTs || params.maxTs) {
-        // The 'minTs' and 'maxTs' parameters are used for a special optimization that
-        // applies only to forwards scans of the oplog.
-        invariant(params.direction == CollectionScanParams::FORWARD);
-        invariant(collection->ns().isOplog());
+    if (params.minRecord || params.maxRecord) {
+        // The 'minRecord' and 'maxRecord' parameters are used for a special optimization that
+        // applies only to forwards scans of the oplog and scans on collections clustered by _id.
         invariant(!params.resumeAfterRecordId);
+        if (collection->ns().isOplog()) {
+            invariant(params.direction == CollectionScanParams::FORWARD);
+        } else {
+            invariant(collection->isClustered());
+        }
     }
+    LOGV2_DEBUG(5400802,
+                5,
+                "collection scan bounds",
+                "min"_attr = (!_params.minRecord) ? "none" : _params.minRecord->toString(),
+                "max"_attr = (!_params.maxRecord) ? "none" : _params.maxRecord->toString());
     invariant(!_params.shouldTrackLatestOplogTimestamp || collection->ns().isOplog());
 
-    // We should never see 'assertMinTsHasNotFallenOffOplog' if 'minTS' is not present.
-    if (params.assertMinTsHasNotFallenOffOplog) {
+    if (params.assertTsHasNotFallenOffOplog) {
         invariant(params.shouldTrackLatestOplogTimestamp);
-        invariant(params.minTs);
+        invariant(params.direction == CollectionScanParams::FORWARD);
     }
 
     if (params.resumeAfterRecordId) {
@@ -90,13 +95,6 @@ CollectionScan::CollectionScan(ExpressionContext* expCtx,
         // only support in the forward direction.
         invariant(params.direction == CollectionScanParams::FORWARD);
     }
-
-    // Set early stop condition.
-    if (params.maxTs) {
-        _endConditionBSON = BSON("$gte"_sd << *(params.maxTs));
-        _endCondition = std::make_unique<GTEMatchExpression>(repl::OpTime::kTimestampFieldName,
-                                                             _endConditionBSON.firstElement());
-    }
 }
 
 PlanStage::StageState CollectionScan::doWork(WorkingSetID* out) {
@@ -139,16 +137,14 @@ PlanStage::StageState CollectionScan::doWork(WorkingSetID* out) {
                 // only time we'd need to create a cursor after already getting a record out of it
                 // and updating our _lastSeenId.
                 if (!_cursor->seekExact(_lastSeenId)) {
-                    Status status(ErrorCodes::CappedPositionLost,
-                                  str::stream() << "CollectionScan died due to failure to restore "
-                                                << "tailable cursor position. "
-                                                << "Last seen record id: " << _lastSeenId);
-                    *out = WorkingSetCommon::allocateStatusMember(_workingSet, status);
-                    return PlanStage::FAILURE;
+                    uasserted(ErrorCodes::CappedPositionLost,
+                              str::stream() << "CollectionScan died due to failure to restore "
+                                            << "tailable cursor position. "
+                                            << "Last seen record id: " << _lastSeenId);
                 }
             }
 
-            if (_params.resumeAfterRecordId) {
+            if (_params.resumeAfterRecordId && !_params.resumeAfterRecordId->isNull()) {
                 invariant(!_params.tailable);
                 invariant(_lastSeenId.isNull());
                 // Seek to where we are trying to resume the scan from. Signal a KeyNotFound error
@@ -158,31 +154,28 @@ PlanStage::StageState CollectionScan::doWork(WorkingSetID* out) {
                 // returned this one prior to the resume.
                 auto recordIdToSeek = *_params.resumeAfterRecordId;
                 if (!_cursor->seekExact(recordIdToSeek)) {
-                    Status status(
+                    uasserted(
                         ErrorCodes::KeyNotFound,
                         str::stream()
                             << "Failed to resume collection scan: the recordId from which we are "
                             << "attempting to resume no longer exists in the collection. "
                             << "recordId: " << recordIdToSeek);
-                    *out = WorkingSetCommon::allocateStatusMember(_workingSet, status);
-                    return PlanStage::FAILURE;
                 }
             }
 
             return PlanStage::NEED_TIME;
         }
 
-        if (_lastSeenId.isNull() && _params.minTs) {
-            // See if the RecordStore supports the oplogStartHack.
-            StatusWith<RecordId> goal = oploghack::keyForOptime(*_params.minTs);
-            if (goal.isOK()) {
-                boost::optional<RecordId> startLoc =
-                    collection()->getRecordStore()->oplogStartHack(opCtx(), goal.getValue());
-                if (startLoc && !startLoc->isNull()) {
-                    LOGV2_DEBUG(20584, 3, "Using direct oplog seek");
-                    record = _cursor->seekExact(*startLoc);
-                }
-            }
+        if (_lastSeenId.isNull() && _params.direction == CollectionScanParams::FORWARD &&
+            _params.minRecord) {
+            // Seek to the approximate start location.
+            record = _cursor->seekNear(*_params.minRecord);
+        }
+
+        if (_lastSeenId.isNull() && _params.direction == CollectionScanParams::BACKWARD &&
+            _params.maxRecord) {
+            // Seek to the approximate start location (at the end).
+            record = _cursor->seekNear(*_params.maxRecord);
         }
 
         if (!record) {
@@ -208,15 +201,11 @@ PlanStage::StageState CollectionScan::doWork(WorkingSetID* out) {
     }
 
     _lastSeenId = record->id;
-    if (_params.assertMinTsHasNotFallenOffOplog) {
-        assertMinTsHasNotFallenOffOplog(*record);
+    if (_params.assertTsHasNotFallenOffOplog) {
+        assertTsHasNotFallenOffOplog(*record);
     }
     if (_params.shouldTrackLatestOplogTimestamp) {
-        auto status = setLatestOplogEntryTimestamp(*record);
-        if (!status.isOK()) {
-            *out = WorkingSetCommon::allocateStatusMember(_workingSet, status);
-            return PlanStage::FAILURE;
-        }
+        setLatestOplogEntryTimestamp(*record);
     }
 
     WorkingSetID id = _workingSet->allocate();
@@ -228,50 +217,70 @@ PlanStage::StageState CollectionScan::doWork(WorkingSetID* out) {
     return returnIfMatches(member, id, out);
 }
 
-Status CollectionScan::setLatestOplogEntryTimestamp(const Record& record) {
+void CollectionScan::setLatestOplogEntryTimestamp(const Record& record) {
     auto tsElem = record.data.toBson()[repl::OpTime::kTimestampFieldName];
-    if (tsElem.type() != BSONType::bsonTimestamp) {
-        Status status(ErrorCodes::InternalError,
-                      str::stream() << "CollectionScan was asked to track latest operation time, "
-                                       "but found a result without a valid 'ts' field: "
-                                    << record.data.toBson().toString());
-        return status;
-    }
+    uassert(ErrorCodes::Error(4382100),
+            str::stream() << "CollectionScan was asked to track latest operation time, "
+                             "but found a result without a valid 'ts' field: "
+                          << record.data.toBson().toString(),
+            tsElem.type() == BSONType::bsonTimestamp);
+    LOGV2_DEBUG(550450,
+                5,
+                "Setting _latestOplogEntryTimestamp to the max of the timestamp of the current "
+                "latest oplog entry and the timestamp of the current record",
+                "latestOplogEntryTimestamp"_attr = _latestOplogEntryTimestamp,
+                "currentRecordTimestamp"_attr = tsElem.timestamp());
     _latestOplogEntryTimestamp = std::max(_latestOplogEntryTimestamp, tsElem.timestamp());
-    return Status::OK();
 }
 
-void CollectionScan::assertMinTsHasNotFallenOffOplog(const Record& record) {
+void CollectionScan::assertTsHasNotFallenOffOplog(const Record& record) {
     // If the first entry we see in the oplog is the replset initialization, then it doesn't matter
-    // if its timestamp is later than the specified minTs; no events earlier than the minTs can have
-    // fallen off this oplog. Otherwise, verify that the timestamp of the first observed oplog entry
-    // is earlier than or equal to the minTs time.
-    auto oplogEntry = invariantStatusOK(repl::OplogEntry::parse(record.data.toBson()));
+    // if its timestamp is later than the timestamp that should not have fallen off the oplog; no
+    // events earlier can have fallen off this oplog. Otherwise, verify that the timestamp of the
+    // first observed oplog entry is earlier than or equal to timestamp that should not have fallen
+    // off the oplog.
+    auto oplogEntry = uassertStatusOK(repl::OplogEntry::parse(record.data.toBson()));
     invariant(_specificStats.docsTested == 0);
     const bool isNewRS =
         oplogEntry.getObject().binaryEqual(BSON("msg" << repl::kInitiatingSetMsg)) &&
         oplogEntry.getOpType() == repl::OpTypeEnum::kNoop;
     uassert(ErrorCodes::OplogQueryMinTsMissing,
-            "Specified minTs has already fallen off the oplog",
-            isNewRS || oplogEntry.getTimestamp() <= *_params.minTs);
+            "Specified timestamp has already fallen off the oplog",
+            isNewRS || oplogEntry.getTimestamp() <= *_params.assertTsHasNotFallenOffOplog);
     // We don't need to check this assertion again after we've confirmed the first oplog event.
-    _params.assertMinTsHasNotFallenOffOplog = false;
+    _params.assertTsHasNotFallenOffOplog = boost::none;
 }
 
+namespace {
+bool atEndOfRangeInclusive(const CollectionScanParams& params, const WorkingSetMember& member) {
+    if (params.direction == CollectionScanParams::FORWARD) {
+        return params.maxRecord && member.recordId > *params.maxRecord;
+    } else {
+        return params.minRecord && member.recordId < *params.minRecord;
+    }
+}
+}  // namespace
+
 PlanStage::StageState CollectionScan::returnIfMatches(WorkingSetMember* member,
                                                       WorkingSetID memberID,
                                                       WorkingSetID* out) {
     ++_specificStats.docsTested;
+
+    // The 'minRecord' and 'maxRecord' bounds are always inclusive, even if the query predicate is
+    // an exclusive inequality like $gt or $lt. In such cases, we rely on '_filter' to either
+    // exclude or include the endpoints as required by the user's query.
+    if (atEndOfRangeInclusive(_params, *member)) {
+        _workingSet->free(memberID);
+        _commonStats.isEOF = true;
+        return PlanStage::IS_EOF;
+    }
+
     if (Filter::passes(member, _filter)) {
         if (_params.stopApplyingFilterAfterFirstMatch) {
             _filter = nullptr;
         }
         *out = memberID;
         return PlanStage::ADVANCED;
-    } else if (_endCondition && Filter::passes(member, _endCondition.get())) {
-        _workingSet->free(memberID);
-        _commonStats.isEOF = true;
-        return PlanStage::IS_EOF;
     } else {
         _workingSet->free(memberID);
         return PlanStage::NEED_TIME;
@@ -290,7 +299,8 @@ void CollectionScan::doSaveStateRequiresCollection() {
 
 void CollectionScan::doRestoreStateRequiresCollection() {
     if (_cursor) {
-        const bool couldRestore = _cursor->restore();
+        const auto tolerateCappedCursorRepositioning = expCtx()->getIsCappedDelete();
+        const bool couldRestore = _cursor->restore(tolerateCappedCursorRepositioning);
         uassert(ErrorCodes::CappedPositionLost,
                 str::stream()
                     << "CollectionScan died due to position in capped collection being deleted. "
