@@ -34,7 +34,6 @@
 #include "mongo/db/index/expression_keys_private.h"
 
 #include <algorithm>
-#include <s2.h>
 
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/simple_bsonobj_comparator.h"
@@ -66,9 +65,9 @@ std::string dumpMultikeyPaths(const MultikeyPaths& multikeyPaths) {
     std::stringstream ss;
 
     ss << "[ ";
-    for (const auto multikeyComponents : multikeyPaths) {
+    for (const auto& multikeyComponents : multikeyPaths) {
         ss << "[ ";
-        for (const auto multikeyComponent : multikeyComponents) {
+        for (const auto& multikeyComponent : multikeyComponents) {
             ss << multikeyComponent << " ";
         }
         ss << "] ";
@@ -106,41 +105,46 @@ void assertMultikeyPathsEqual(const MultikeyPaths& expectedMultikeyPaths,
     }
 }
 
-long long getCellID(int x, int y, bool multiPoint = false) {
-    BSONObj obj;
-    if (multiPoint) {
-        obj = BSON("a" << BSON("type"
-                               << "MultiPoint"
-                               << "coordinates" << BSON_ARRAY(BSON_ARRAY(x << y))));
-    } else {
-        obj = BSON("a" << BSON("type"
-                               << "Point"
-                               << "coordinates" << BSON_ARRAY(x << y)));
+struct S2KeyGeneratorTest : public unittest::Test {
+    SharedBufferFragmentBuilder allocator{KeyString::HeapBuilder::kHeapAllocatorDefaultBytes};
+
+    long long getCellID(int x, int y, bool multiPoint = false) {
+        BSONObj obj;
+        if (multiPoint) {
+            obj = BSON("a" << BSON("type"
+                                   << "MultiPoint"
+                                   << "coordinates" << BSON_ARRAY(BSON_ARRAY(x << y))));
+        } else {
+            obj = BSON("a" << BSON("type"
+                                   << "Point"
+                                   << "coordinates" << BSON_ARRAY(x << y)));
+        }
+        BSONObj keyPattern = fromjson("{a: '2dsphere'}");
+        BSONObj infoObj = fromjson("{key: {a: '2dsphere'}, '2dsphereIndexVersion': 3}");
+        S2IndexingParams params;
+        const CollatorInterface* collator = nullptr;
+        ExpressionParams::initialize2dsphereParams(infoObj, collator, &params);
+
+        KeyStringSet keys;
+        // There's no need to compute the prefixes of the indexed fields that cause the index to be
+        // multikey when computing the cell id of the geo field.
+        MultikeyPaths* multikeyPaths = nullptr;
+        ExpressionKeysPrivate::getS2Keys(allocator,
+                                         obj,
+                                         keyPattern,
+                                         params,
+                                         &keys,
+                                         multikeyPaths,
+                                         KeyString::Version::kLatestVersion,
+                                         Ordering::make(BSONObj()));
+
+        ASSERT_EQUALS(1U, keys.size());
+        auto key = KeyString::toBson(*keys.begin(), Ordering::make(BSONObj()));
+        return key.firstElement().Long();
     }
-    BSONObj keyPattern = fromjson("{a: '2dsphere'}");
-    BSONObj infoObj = fromjson("{key: {a: '2dsphere'}, '2dsphereIndexVersion': 3}");
-    S2IndexingParams params;
-    const CollatorInterface* collator = nullptr;
-    ExpressionParams::initialize2dsphereParams(infoObj, collator, &params);
+};
 
-    KeyStringSet keys;
-    // There's no need to compute the prefixes of the indexed fields that cause the index to be
-    // multikey when computing the cell id of the geo field.
-    MultikeyPaths* multikeyPaths = nullptr;
-    ExpressionKeysPrivate::getS2Keys(obj,
-                                     keyPattern,
-                                     params,
-                                     &keys,
-                                     multikeyPaths,
-                                     KeyString::Version::kLatestVersion,
-                                     Ordering::make(BSONObj()));
-
-    ASSERT_EQUALS(1U, keys.size());
-    auto key = KeyString::toBson(*keys.begin(), Ordering::make(BSONObj()));
-    return key.firstElement().Long();
-}
-
-TEST(S2KeyGeneratorTest, GetS2KeysFromSubobjectWithArrayOfGeoAndNonGeoSubobjects) {
+TEST_F(S2KeyGeneratorTest, GetS2KeysFromSubobjectWithArrayOfGeoAndNonGeoSubobjects) {
     BSONObj keyPattern = fromjson("{'a.b.nongeo': 1, 'a.b.geo': '2dsphere'}");
     BSONObj genKeysFrom = fromjson(
         "{a: {b: [{nongeo: 1, geo: {type: 'Point', coordinates: [0, 0]}}, "
@@ -153,7 +157,8 @@ TEST(S2KeyGeneratorTest, GetS2KeysFromSubobjectWithArrayOfGeoAndNonGeoSubobjects
 
     KeyStringSet actualKeys;
     MultikeyPaths actualMultikeyPaths;
-    ExpressionKeysPrivate::getS2Keys(genKeysFrom,
+    ExpressionKeysPrivate::getS2Keys(allocator,
+                                     genKeysFrom,
                                      keyPattern,
                                      params,
                                      &actualKeys,
@@ -180,7 +185,7 @@ TEST(S2KeyGeneratorTest, GetS2KeysFromSubobjectWithArrayOfGeoAndNonGeoSubobjects
     assertMultikeyPathsEqual(MultikeyPaths{{1U}, {1U}}, actualMultikeyPaths);
 }
 
-TEST(S2KeyGeneratorTest, GetS2KeysFromArrayOfNonGeoSubobjectsWithArrayValues) {
+TEST_F(S2KeyGeneratorTest, GetS2KeysFromArrayOfNonGeoSubobjectsWithArrayValues) {
     BSONObj keyPattern = fromjson("{'a.nongeo': 1, geo: '2dsphere'}");
     BSONObj genKeysFrom = fromjson(
         "{a: [{nongeo: [1, 2]}, {nongeo: [2, 3]}], "
@@ -193,7 +198,8 @@ TEST(S2KeyGeneratorTest, GetS2KeysFromArrayOfNonGeoSubobjectsWithArrayValues) {
 
     KeyStringSet actualKeys;
     MultikeyPaths actualMultikeyPaths;
-    ExpressionKeysPrivate::getS2Keys(genKeysFrom,
+    ExpressionKeysPrivate::getS2Keys(allocator,
+                                     genKeysFrom,
                                      keyPattern,
                                      params,
                                      &actualKeys,
@@ -213,10 +219,10 @@ TEST(S2KeyGeneratorTest, GetS2KeysFromArrayOfNonGeoSubobjectsWithArrayValues) {
     KeyStringSet expectedKeys{keyString1.release(), keyString2.release(), keyString3.release()};
 
     ASSERT_TRUE(areKeysetsEqual(expectedKeys, actualKeys));
-    assertMultikeyPathsEqual(MultikeyPaths{{0U, 1U}, std::set<size_t>{}}, actualMultikeyPaths);
+    assertMultikeyPathsEqual(MultikeyPaths{{0U, 1U}, MultikeyComponents{}}, actualMultikeyPaths);
 }
 
-TEST(S2KeyGeneratorTest, GetS2KeysFromMultiPointInGeoField) {
+TEST_F(S2KeyGeneratorTest, GetS2KeysFromMultiPointInGeoField) {
     BSONObj keyPattern = fromjson("{nongeo: 1, geo: '2dsphere'}");
     BSONObj genKeysFrom =
         fromjson("{nongeo: 1, geo: {type: 'MultiPoint', coordinates: [[0, 0], [1, 0], [1, 1]]}}");
@@ -227,7 +233,8 @@ TEST(S2KeyGeneratorTest, GetS2KeysFromMultiPointInGeoField) {
 
     KeyStringSet actualKeys;
     MultikeyPaths actualMultikeyPaths;
-    ExpressionKeysPrivate::getS2Keys(genKeysFrom,
+    ExpressionKeysPrivate::getS2Keys(allocator,
+                                     genKeysFrom,
                                      keyPattern,
                                      params,
                                      &actualKeys,
@@ -248,10 +255,10 @@ TEST(S2KeyGeneratorTest, GetS2KeysFromMultiPointInGeoField) {
     KeyStringSet expectedKeys{keyString1.release(), keyString2.release(), keyString3.release()};
 
     ASSERT_TRUE(areKeysetsEqual(expectedKeys, actualKeys));
-    assertMultikeyPathsEqual(MultikeyPaths{std::set<size_t>{}, {0U}}, actualMultikeyPaths);
+    assertMultikeyPathsEqual(MultikeyPaths{MultikeyComponents{}, {0U}}, actualMultikeyPaths);
 }
 
-TEST(S2KeyGeneratorTest, CollationAppliedToNonGeoStringFieldAfterGeoField) {
+TEST_F(S2KeyGeneratorTest, CollationAppliedToNonGeoStringFieldAfterGeoField) {
     BSONObj obj = fromjson("{a: {type: 'Point', coordinates: [0, 0]}, b: 'string'}");
     BSONObj keyPattern = fromjson("{a: '2dsphere', b: 1}");
     BSONObj infoObj = fromjson("{key: {a: '2dsphere', b: 1}, '2dsphereIndexVersion': 3}");
@@ -261,7 +268,8 @@ TEST(S2KeyGeneratorTest, CollationAppliedToNonGeoStringFieldAfterGeoField) {
 
     KeyStringSet actualKeys;
     MultikeyPaths actualMultikeyPaths;
-    ExpressionKeysPrivate::getS2Keys(obj,
+    ExpressionKeysPrivate::getS2Keys(allocator,
+                                     obj,
                                      keyPattern,
                                      params,
                                      &actualKeys,
@@ -276,11 +284,11 @@ TEST(S2KeyGeneratorTest, CollationAppliedToNonGeoStringFieldAfterGeoField) {
     KeyStringSet expectedKeys{keyString.release()};
 
     ASSERT_TRUE(areKeysetsEqual(expectedKeys, actualKeys));
-    assertMultikeyPathsEqual(MultikeyPaths{std::set<size_t>{}, std::set<size_t>{}},
+    assertMultikeyPathsEqual(MultikeyPaths{MultikeyComponents{}, MultikeyComponents{}},
                              actualMultikeyPaths);
 }
 
-TEST(S2KeyGeneratorTest, CollationAppliedToNonGeoStringFieldBeforeGeoField) {
+TEST_F(S2KeyGeneratorTest, CollationAppliedToNonGeoStringFieldBeforeGeoField) {
     BSONObj obj = fromjson("{a: 'string', b: {type: 'Point', coordinates: [0, 0]}}");
     BSONObj keyPattern = fromjson("{a: 1, b: '2dsphere'}");
     BSONObj infoObj = fromjson("{key: {a: 1, b: '2dsphere'}, '2dsphereIndexVersion': 3}");
@@ -290,7 +298,8 @@ TEST(S2KeyGeneratorTest, CollationAppliedToNonGeoStringFieldBeforeGeoField) {
 
     KeyStringSet actualKeys;
     MultikeyPaths actualMultikeyPaths;
-    ExpressionKeysPrivate::getS2Keys(obj,
+    ExpressionKeysPrivate::getS2Keys(allocator,
+                                     obj,
                                      keyPattern,
                                      params,
                                      &actualKeys,
@@ -306,11 +315,11 @@ TEST(S2KeyGeneratorTest, CollationAppliedToNonGeoStringFieldBeforeGeoField) {
     KeyStringSet expectedKeys{keyString.release()};
 
     ASSERT_TRUE(areKeysetsEqual(expectedKeys, actualKeys));
-    assertMultikeyPathsEqual(MultikeyPaths{std::set<size_t>{}, std::set<size_t>{}},
+    assertMultikeyPathsEqual(MultikeyPaths{MultikeyComponents{}, MultikeyComponents{}},
                              actualMultikeyPaths);
 }
 
-TEST(S2KeyGeneratorTest, CollationAppliedToAllNonGeoStringFields) {
+TEST_F(S2KeyGeneratorTest, CollationAppliedToAllNonGeoStringFields) {
     BSONObj obj = fromjson("{a: 'string', b: {type: 'Point', coordinates: [0, 0]}, c: 'string2'}");
     BSONObj keyPattern = fromjson("{a: 1, b: '2dsphere', c: 1}");
     BSONObj infoObj = fromjson("{key: {a: 1, b: '2dsphere', c: 1}, '2dsphereIndexVersion': 3}");
@@ -320,7 +329,8 @@ TEST(S2KeyGeneratorTest, CollationAppliedToAllNonGeoStringFields) {
 
     KeyStringSet actualKeys;
     MultikeyPaths actualMultikeyPaths;
-    ExpressionKeysPrivate::getS2Keys(obj,
+    ExpressionKeysPrivate::getS2Keys(allocator,
+                                     obj,
                                      keyPattern,
                                      params,
                                      &actualKeys,
@@ -338,11 +348,11 @@ TEST(S2KeyGeneratorTest, CollationAppliedToAllNonGeoStringFields) {
 
     ASSERT_TRUE(areKeysetsEqual(expectedKeys, actualKeys));
     assertMultikeyPathsEqual(
-        MultikeyPaths{std::set<size_t>{}, std::set<size_t>{}, std::set<size_t>{}},
+        MultikeyPaths{MultikeyComponents{}, MultikeyComponents{}, MultikeyComponents{}},
         actualMultikeyPaths);
 }
 
-TEST(S2KeyGeneratorTest, CollationAppliedToNonGeoStringFieldWithMultiplePathComponents) {
+TEST_F(S2KeyGeneratorTest, CollationAppliedToNonGeoStringFieldWithMultiplePathComponents) {
     BSONObj obj = fromjson("{a: {type: 'Point', coordinates: [0, 0]}, b: {c: {d: 'string'}}}");
     BSONObj keyPattern = fromjson("{a: '2dsphere', 'b.c.d': 1}");
     BSONObj infoObj = fromjson("{key: {a: '2dsphere', 'b.c.d': 1}, '2dsphereIndexVersion': 3}");
@@ -352,7 +362,8 @@ TEST(S2KeyGeneratorTest, CollationAppliedToNonGeoStringFieldWithMultiplePathComp
 
     KeyStringSet actualKeys;
     MultikeyPaths actualMultikeyPaths;
-    ExpressionKeysPrivate::getS2Keys(obj,
+    ExpressionKeysPrivate::getS2Keys(allocator,
+                                     obj,
                                      keyPattern,
                                      params,
                                      &actualKeys,
@@ -367,11 +378,11 @@ TEST(S2KeyGeneratorTest, CollationAppliedToNonGeoStringFieldWithMultiplePathComp
     KeyStringSet expectedKeys{keyString.release()};
 
     ASSERT_TRUE(areKeysetsEqual(expectedKeys, actualKeys));
-    assertMultikeyPathsEqual(MultikeyPaths{std::set<size_t>{}, std::set<size_t>{}},
+    assertMultikeyPathsEqual(MultikeyPaths{MultikeyComponents{}, MultikeyComponents{}},
                              actualMultikeyPaths);
 }
 
-TEST(S2KeyGeneratorTest, CollationAppliedToStringsInArray) {
+TEST_F(S2KeyGeneratorTest, CollationAppliedToStringsInArray) {
     BSONObj obj = fromjson("{a: {type: 'Point', coordinates: [0, 0]}, b: ['string', 'string2']}");
     BSONObj keyPattern = fromjson("{a: '2dsphere', b: 1}");
     BSONObj infoObj = fromjson("{key: {a: '2dsphere', b: 1}, '2dsphereIndexVersion': 3}");
@@ -381,7 +392,8 @@ TEST(S2KeyGeneratorTest, CollationAppliedToStringsInArray) {
 
     KeyStringSet actualKeys;
     MultikeyPaths actualMultikeyPaths;
-    ExpressionKeysPrivate::getS2Keys(obj,
+    ExpressionKeysPrivate::getS2Keys(allocator,
+                                     obj,
                                      keyPattern,
                                      params,
                                      &actualKeys,
@@ -400,10 +412,10 @@ TEST(S2KeyGeneratorTest, CollationAppliedToStringsInArray) {
     KeyStringSet expectedKeys{keyString1.release(), keyString2.release()};
 
     ASSERT_TRUE(areKeysetsEqual(expectedKeys, actualKeys));
-    assertMultikeyPathsEqual(MultikeyPaths{std::set<size_t>{}, {0U}}, actualMultikeyPaths);
+    assertMultikeyPathsEqual(MultikeyPaths{MultikeyComponents{}, {0U}}, actualMultikeyPaths);
 }
 
-TEST(S2KeyGeneratorTest, CollationAppliedToStringsInAllArrays) {
+TEST_F(S2KeyGeneratorTest, CollationAppliedToStringsInAllArrays) {
     BSONObj obj = fromjson(
         "{a: {type: 'Point', coordinates: [0, 0]}, b: ['string', 'string2'], c: ['abc', 'def']}");
     BSONObj keyPattern = fromjson("{a: '2dsphere', b: 1, c: 1}");
@@ -414,7 +426,8 @@ TEST(S2KeyGeneratorTest, CollationAppliedToStringsInAllArrays) {
 
     KeyStringSet actualKeys;
     MultikeyPaths actualMultikeyPaths;
-    ExpressionKeysPrivate::getS2Keys(obj,
+    ExpressionKeysPrivate::getS2Keys(allocator,
+                                     obj,
                                      keyPattern,
                                      params,
                                      &actualKeys,
@@ -450,10 +463,10 @@ TEST(S2KeyGeneratorTest, CollationAppliedToStringsInAllArrays) {
         keyString1.release(), keyString2.release(), keyString3.release(), keyString4.release()};
 
     ASSERT_TRUE(areKeysetsEqual(expectedKeys, actualKeys));
-    assertMultikeyPathsEqual(MultikeyPaths{std::set<size_t>{}, {0U}, {0U}}, actualMultikeyPaths);
+    assertMultikeyPathsEqual(MultikeyPaths{MultikeyComponents{}, {0U}, {0U}}, actualMultikeyPaths);
 }
 
-TEST(S2KeyGeneratorTest, CollationDoesNotAffectNonStringFields) {
+TEST_F(S2KeyGeneratorTest, CollationDoesNotAffectNonStringFields) {
     BSONObj obj = fromjson("{a: {type: 'Point', coordinates: [0, 0]}, b: 5}");
     BSONObj keyPattern = fromjson("{a: '2dsphere', b: 1}");
     BSONObj infoObj = fromjson("{key: {a: '2dsphere', b: 1}, '2dsphereIndexVersion': 3}");
@@ -463,7 +476,8 @@ TEST(S2KeyGeneratorTest, CollationDoesNotAffectNonStringFields) {
 
     KeyStringSet actualKeys;
     MultikeyPaths actualMultikeyPaths;
-    ExpressionKeysPrivate::getS2Keys(obj,
+    ExpressionKeysPrivate::getS2Keys(allocator,
+                                     obj,
                                      keyPattern,
                                      params,
                                      &actualKeys,
@@ -477,11 +491,11 @@ TEST(S2KeyGeneratorTest, CollationDoesNotAffectNonStringFields) {
     KeyStringSet expectedKeys{keyString.release()};
 
     ASSERT_TRUE(areKeysetsEqual(expectedKeys, actualKeys));
-    assertMultikeyPathsEqual(MultikeyPaths{std::set<size_t>{}, std::set<size_t>{}},
+    assertMultikeyPathsEqual(MultikeyPaths{MultikeyComponents{}, MultikeyComponents{}},
                              actualMultikeyPaths);
 }
 
-TEST(S2KeyGeneratorTest, CollationAppliedToStringsInNestedObjects) {
+TEST_F(S2KeyGeneratorTest, CollationAppliedToStringsInNestedObjects) {
     BSONObj obj = fromjson("{a: {type: 'Point', coordinates: [0, 0]}, b: {c: 'string'}}");
     BSONObj keyPattern = fromjson("{a: '2dsphere', b: 1}");
     BSONObj infoObj = fromjson("{key: {a: '2dsphere', b: 1}, '2dsphereIndexVersion': 3}");
@@ -491,7 +505,8 @@ TEST(S2KeyGeneratorTest, CollationAppliedToStringsInNestedObjects) {
 
     KeyStringSet actualKeys;
     MultikeyPaths actualMultikeyPaths;
-    ExpressionKeysPrivate::getS2Keys(obj,
+    ExpressionKeysPrivate::getS2Keys(allocator,
+                                     obj,
                                      keyPattern,
                                      params,
                                      &actualKeys,
@@ -507,11 +522,11 @@ TEST(S2KeyGeneratorTest, CollationAppliedToStringsInNestedObjects) {
     KeyStringSet expectedKeys{keyString.release()};
 
     ASSERT_TRUE(areKeysetsEqual(expectedKeys, actualKeys));
-    assertMultikeyPathsEqual(MultikeyPaths{std::set<size_t>{}, std::set<size_t>{}},
+    assertMultikeyPathsEqual(MultikeyPaths{MultikeyComponents{}, MultikeyComponents{}},
                              actualMultikeyPaths);
 }
 
-TEST(S2KeyGeneratorTest, NoCollation) {
+TEST_F(S2KeyGeneratorTest, NoCollation) {
     BSONObj obj = fromjson("{a: {type: 'Point', coordinates: [0, 0]}, b: 'string'}");
     BSONObj keyPattern = fromjson("{a: '2dsphere', b: 1}");
     BSONObj infoObj = fromjson("{key: {a: '2dsphere', b: 1}, '2dsphereIndexVersion': 3}");
@@ -521,7 +536,8 @@ TEST(S2KeyGeneratorTest, NoCollation) {
 
     KeyStringSet actualKeys;
     MultikeyPaths actualMultikeyPaths;
-    ExpressionKeysPrivate::getS2Keys(obj,
+    ExpressionKeysPrivate::getS2Keys(allocator,
+                                     obj,
                                      keyPattern,
                                      params,
                                      &actualKeys,
@@ -536,11 +552,11 @@ TEST(S2KeyGeneratorTest, NoCollation) {
     KeyStringSet expectedKeys{keyString.release()};
 
     ASSERT_TRUE(areKeysetsEqual(expectedKeys, actualKeys));
-    assertMultikeyPathsEqual(MultikeyPaths{std::set<size_t>{}, std::set<size_t>{}},
+    assertMultikeyPathsEqual(MultikeyPaths{MultikeyComponents{}, MultikeyComponents{}},
                              actualMultikeyPaths);
 }
 
-TEST(S2KeyGeneratorTest, EmptyArrayForLeadingFieldIsConsideredMultikey) {
+TEST_F(S2KeyGeneratorTest, EmptyArrayForLeadingFieldIsConsideredMultikey) {
     BSONObj obj = fromjson("{a: [], b: {type: 'Point', coordinates: [0, 0]}}");
     BSONObj keyPattern = fromjson("{a: 1, b: '2dsphere'}");
     BSONObj infoObj = fromjson("{key: {a: 1, b: '2dsphere'}, '2dsphereIndexVersion': 3}");
@@ -550,7 +566,8 @@ TEST(S2KeyGeneratorTest, EmptyArrayForLeadingFieldIsConsideredMultikey) {
 
     KeyStringSet actualKeys;
     MultikeyPaths actualMultikeyPaths;
-    ExpressionKeysPrivate::getS2Keys(obj,
+    ExpressionKeysPrivate::getS2Keys(allocator,
+                                     obj,
                                      keyPattern,
                                      params,
                                      &actualKeys,
@@ -564,10 +581,10 @@ TEST(S2KeyGeneratorTest, EmptyArrayForLeadingFieldIsConsideredMultikey) {
     KeyStringSet expectedKeys{keyString.release()};
 
     ASSERT_TRUE(areKeysetsEqual(expectedKeys, actualKeys));
-    assertMultikeyPathsEqual(MultikeyPaths{{0U}, std::set<size_t>{}}, actualMultikeyPaths);
+    assertMultikeyPathsEqual(MultikeyPaths{{0U}, MultikeyComponents{}}, actualMultikeyPaths);
 }
 
-TEST(S2KeyGeneratorTest, EmptyArrayForTrailingFieldIsConsideredMultikey) {
+TEST_F(S2KeyGeneratorTest, EmptyArrayForTrailingFieldIsConsideredMultikey) {
     BSONObj obj = fromjson("{a: {type: 'Point', coordinates: [0, 0]}, b: []}");
     BSONObj keyPattern = fromjson("{a: '2dsphere', b: 1}");
     BSONObj infoObj = fromjson("{key: {a: '2dsphere', a: 1}, '2dsphereIndexVersion': 3}");
@@ -577,7 +594,8 @@ TEST(S2KeyGeneratorTest, EmptyArrayForTrailingFieldIsConsideredMultikey) {
 
     KeyStringSet actualKeys;
     MultikeyPaths actualMultikeyPaths;
-    ExpressionKeysPrivate::getS2Keys(obj,
+    ExpressionKeysPrivate::getS2Keys(allocator,
+                                     obj,
                                      keyPattern,
                                      params,
                                      &actualKeys,
@@ -591,10 +609,10 @@ TEST(S2KeyGeneratorTest, EmptyArrayForTrailingFieldIsConsideredMultikey) {
     KeyStringSet expectedKeys{keyString.release()};
 
     ASSERT_TRUE(areKeysetsEqual(expectedKeys, actualKeys));
-    assertMultikeyPathsEqual(MultikeyPaths{std::set<size_t>{}, {0U}}, actualMultikeyPaths);
+    assertMultikeyPathsEqual(MultikeyPaths{MultikeyComponents{}, {0U}}, actualMultikeyPaths);
 }
 
-TEST(S2KeyGeneratorTest, SingleElementTrailingArrayIsConsideredMultikey) {
+TEST_F(S2KeyGeneratorTest, SingleElementTrailingArrayIsConsideredMultikey) {
     BSONObj obj = fromjson("{a: {c: [99]}, b: {type: 'Point', coordinates: [0, 0]}}");
     BSONObj keyPattern = fromjson("{'a.c': 1, b: '2dsphere'}");
     BSONObj infoObj = fromjson("{key: {'a.c': 1, b: '2dsphere'}, '2dsphereIndexVersion': 3}");
@@ -604,7 +622,8 @@ TEST(S2KeyGeneratorTest, SingleElementTrailingArrayIsConsideredMultikey) {
 
     KeyStringSet actualKeys;
     MultikeyPaths actualMultikeyPaths;
-    ExpressionKeysPrivate::getS2Keys(obj,
+    ExpressionKeysPrivate::getS2Keys(allocator,
+                                     obj,
                                      keyPattern,
                                      params,
                                      &actualKeys,
@@ -618,10 +637,10 @@ TEST(S2KeyGeneratorTest, SingleElementTrailingArrayIsConsideredMultikey) {
     KeyStringSet expectedKeys{keyString.release()};
 
     ASSERT_TRUE(areKeysetsEqual(expectedKeys, actualKeys));
-    assertMultikeyPathsEqual(MultikeyPaths{{1U}, std::set<size_t>{}}, actualMultikeyPaths);
+    assertMultikeyPathsEqual(MultikeyPaths{{1U}, MultikeyComponents{}}, actualMultikeyPaths);
 }
 
-TEST(S2KeyGeneratorTest, MidPathSingleElementArrayIsConsideredMultikey) {
+TEST_F(S2KeyGeneratorTest, MidPathSingleElementArrayIsConsideredMultikey) {
     BSONObj obj = fromjson("{a: [{c: 99}], b: {type: 'Point', coordinates: [0, 0]}}");
     BSONObj keyPattern = fromjson("{'a.c': 1, b: '2dsphere'}");
     BSONObj infoObj = fromjson("{key: {'a.c': 1, b: '2dsphere'}, '2dsphereIndexVersion': 3}");
@@ -631,7 +650,8 @@ TEST(S2KeyGeneratorTest, MidPathSingleElementArrayIsConsideredMultikey) {
 
     KeyStringSet actualKeys;
     MultikeyPaths actualMultikeyPaths;
-    ExpressionKeysPrivate::getS2Keys(obj,
+    ExpressionKeysPrivate::getS2Keys(allocator,
+                                     obj,
                                      keyPattern,
                                      params,
                                      &actualKeys,
@@ -645,30 +665,7 @@ TEST(S2KeyGeneratorTest, MidPathSingleElementArrayIsConsideredMultikey) {
     KeyStringSet expectedKeys{keyString.release()};
 
     ASSERT_TRUE(areKeysetsEqual(expectedKeys, actualKeys));
-    assertMultikeyPathsEqual(MultikeyPaths{{0U}, std::set<size_t>{}}, actualMultikeyPaths);
+    assertMultikeyPathsEqual(MultikeyPaths{{0U}, MultikeyComponents{}}, actualMultikeyPaths);
 }
 
-// Test which verifies that the rounding functions used by s2 follow 'round to even' rounding
-// behavior.
-TEST(S2KeyGeneratorTest, VerifyS2RoundingBehavior) {
-    const double roundDownToEven = 2.5;
-    ASSERT_EQ(2, MathUtil::FastIntRound(roundDownToEven));
-    ASSERT_EQ(2LL, MathUtil::FastInt64Round(roundDownToEven));
-
-    const double roundUpToEven = 3.5;
-    ASSERT_EQ(4, MathUtil::FastIntRound(roundUpToEven));
-    ASSERT_EQ(4LL, MathUtil::FastInt64Round(roundUpToEven));
-
-    const double roundDownToEvenNegative = -3.5;
-    ASSERT_EQ(-4, MathUtil::FastIntRound(roundDownToEvenNegative));
-    ASSERT_EQ(-4LL, MathUtil::FastInt64Round(roundDownToEvenNegative));
-
-    const double roundUpToEvenNegative = -2.5;
-    ASSERT_EQ(-2, MathUtil::FastIntRound(roundUpToEvenNegative));
-    ASSERT_EQ(-2LL, MathUtil::FastInt64Round(roundUpToEvenNegative));
-
-    const double point = 944920918.5;
-    ASSERT_EQ(944920918, MathUtil::FastIntRound(point));
-    ASSERT_EQ(944920918LL, MathUtil::FastInt64Round(point));
-}
 }  // namespace
