@@ -45,7 +45,10 @@ public:
     static constexpr StringData kName = "$_internalSchemaMatchArrayIndex"_sd;
 
     InternalSchemaMatchArrayIndexMatchExpression(
-        StringData path, long long index, std::unique_ptr<ExpressionWithPlaceholder> expression);
+        StringData path,
+        long long index,
+        std::unique_ptr<ExpressionWithPlaceholder> expression,
+        clonable_ptr<ErrorAnnotation> annotation = nullptr);
 
     void debugString(StringBuilder& debug, int indentationLevel) const final;
 
@@ -74,8 +77,8 @@ public:
 
     std::unique_ptr<MatchExpression> shallowClone() const final;
 
-    boost::optional<std::vector<MatchExpression*>&> getChildVector() final {
-        return boost::none;
+    std::vector<std::unique_ptr<MatchExpression>>* getChildVector() final {
+        return nullptr;
     }
 
     size_t numChildren() const final {
@@ -87,6 +90,20 @@ public:
         return _expression->getFilter();
     }
 
+    void acceptVisitor(MatchExpressionMutableVisitor* visitor) final {
+        visitor->visit(this);
+    }
+    void acceptVisitor(MatchExpressionConstVisitor* visitor) const final {
+        visitor->visit(this);
+    }
+
+    /**
+     * Returns an index of an array element this expression applies to.
+     */
+    long long arrayIndex() const {
+        return _index;
+    }
+
 private:
     ExpressionOptimizerFunc getOptimizer() const final;
 
