@@ -44,18 +44,6 @@ class DocumentSourceSort final : public DocumentSource {
 public:
     static constexpr StringData kStageName = "$sort"_sd;
 
-    /**
-     * If there are any $limit stages that could be logically swapped forward to the position of the
-     * pipeline pointed to by 'itr' without changing the meaning of the query, removes these $limit
-     * stages from the Pipeline and returns the resulting limit. A single limit value is computed by
-     * taking the minimum after swapping each individual $limit stage forward.
-     *
-     * This method also implements the ability to swap a $limit before a $skip, by adding the value
-     * of the $skip to the value of the $limit.
-     */
-    static boost::optional<long long> extractLimitForPushdown(
-        Pipeline::SourceContainer::iterator itr, Pipeline::SourceContainer* container);
-
     const char* getSourceName() const final {
         return kStageName.rawData();
     }
@@ -78,7 +66,7 @@ public:
                                      TransactionRequirement::kAllowed,
                                      LookupRequirement::kAllowed,
                                      UnionRequirement::kAllowed,
-                                     ChangeStreamRequirement::kBlacklist);
+                                     ChangeStreamRequirement::kDenylist);
 
         // Can't swap with a $match if a limit has been absorbed, as $match can't swap with $limit.
         constraints.canSwapWithMatch = !_sortExecutor->hasLimit();
@@ -105,15 +93,23 @@ public:
         BSONElement elem, const boost::intrusive_ptr<ExpressionContext>& pExpCtx);
 
     /**
-     * Convenience method for creating a $sort stage. If maxMemoryUsageBytes is boost::none,
-     * then it will actually use the value of 'internalQueryMaxBlockingSortMemoryUsageBytes'.
+     * Creates a $sort stage. If maxMemoryUsageBytes is boost::none, then it will actually use the
+     * value of 'internalQueryMaxBlockingSortMemoryUsageBytes'.
      */
     static boost::intrusive_ptr<DocumentSourceSort> create(
         const boost::intrusive_ptr<ExpressionContext>& pExpCtx,
-        BSONObj sortOrder,
+        const SortPattern& sortOrder,
         uint64_t limit = 0,
         boost::optional<uint64_t> maxMemoryUsageBytes = boost::none);
 
+    /**
+     * Convenience to create a $sort stage from BSON with no limit and the default memory limit.
+     */
+    static boost::intrusive_ptr<DocumentSourceSort> create(
+        const boost::intrusive_ptr<ExpressionContext>& pExpCtx, const BSONObj& sortOrder) {
+        return create(pExpCtx, {sortOrder, pExpCtx});
+    }
+
     /**
      * Returns the the limit, if a subsequent $limit stage has been coalesced with this $sort stage.
      * Otherwise, returns boost::none.
@@ -146,6 +142,10 @@ public:
         return _sortExecutor->hasLimit();
     }
 
+    const SpecificStats* getSpecificStats() const final {
+        return &_sortExecutor->stats();
+    }
+
 protected:
     GetNextResult doGetNext() final;
     /**
@@ -156,7 +156,7 @@ protected:
 
 private:
     DocumentSourceSort(const boost::intrusive_ptr<ExpressionContext>& pExpCtx,
-                       const BSONObj& sortOrder,
+                       const SortPattern& sortOrder,
                        uint64_t limit,
                        uint64_t maxMemoryUsageBytes);
 
