@@ -289,6 +289,14 @@ public:
         _resolvedNamespaces = std::move(resolvedNamespaces);
     }
 
+    void setIsCappedDelete() {
+        _isCappedDelete = true;
+    }
+
+    bool getIsCappedDelete() const {
+        return _isCappedDelete;
+    }
+
     /**
      * Retrieves the Javascript Scope for the current thread or creates a new one if it has not been
      * created yet. Initializes the Scope with the 'jsScope' variables from the runtimeConstants.
@@ -359,6 +367,7 @@ public:
     bool fromMongos = false;
     bool needsMerge = false;
     bool inMongos = false;
+    bool forPerShardCursor = false;
     bool allowDiskUse = false;
     bool bypassDocumentValidation = false;
     bool hasWhereClause = false;
@@ -455,6 +464,8 @@ protected:
 
     int _interruptCounter = kInterruptCheckPeriod;
 
+    bool _isCappedDelete = false;
+
 private:
     boost::optional<ExpressionCounters> _expressionCounters = boost::none;
 };
