@@ -197,6 +197,7 @@ void findRelevantTaggedNodePathsAndIndices(MatchExpression* root,
 IndexEntry buildSimpleIndexEntry(const BSONObj& kp) {
     return {kp,
             IndexNames::nameToType(IndexNames::findPluginName(kp)),
+            IndexDescriptor::kLatestIndexVersion,
             false,
             {},
             {},
@@ -1069,6 +1070,7 @@ auto makeIndexEntry(BSONObj keyPattern,
         });
     return std::make_pair(IndexEntry(keyPattern,
                                      IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
+                                     IndexDescriptor::kLatestIndexVersion,
                                      multiKey,
                                      multiKeyPaths,
                                      multiKeyPathSet,
@@ -1193,7 +1195,7 @@ TEST(QueryPlannerIXSelectTest, NotEqualsNullCannotUseDottedMultiKeyIndex) {
 
 TEST(QueryPlannerIXSelectTest, NotEqualsNullCanUseIndexWhichIsMultiKeyOnAnotherPath) {
     auto entry = buildSimpleIndexEntry(BSON("a" << 1 << "mk" << 1));
-    entry.multikeyPaths = {{}, {0}};
+    entry.multikeyPaths = {MultikeyComponents{}, MultikeyComponents{0}};
     std::set<size_t> expectedIndices = {0};
     testRateIndices("{a: {$ne: null}}", "", kSimpleCollator, {entry}, "a,a", expectedIndices);
 }
@@ -1428,7 +1430,7 @@ TEST(QueryPlannerIXSelectTest, ExpandedIndexEntriesAreCorrectlyMarkedAsMultikeyO
 
         if (SimpleBSONObjComparator::kInstance.evaluate(entry.keyPattern == BSON("a.b" << 1))) {
             ASSERT_TRUE(entry.multikey);
-            ASSERT(entry.multikeyPaths[0] == std::set<std::size_t>{0u});
+            ASSERT(entry.multikeyPaths[0] == MultikeyComponents{0u});
         } else {
             ASSERT_BSONOBJ_EQ(entry.keyPattern, BSON("c.d" << 1));
             ASSERT_FALSE(entry.multikey);
