@@ -34,7 +34,7 @@
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/curop_failpoint_helpers.h"
-#include "mongo/db/logical_clock.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/op_observer.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/read_concern.h"
@@ -42,10 +42,13 @@
 #include "mongo/db/repl/optime.h"
 #include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/repl/speculative_majority_read_info.h"
+#include "mongo/db/repl/tenant_migration_access_blocker_util.h"
 #include "mongo/db/s/sharding_state.h"
 #include "mongo/db/server_options.h"
 #include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/vector_clock.h"
 #include "mongo/logv2/log.h"
+#include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/s/grid.h"
 #include "mongo/util/concurrency/notification.h"
 
@@ -105,7 +108,7 @@ private:
 /**
  *  Schedule a write via appendOplogNote command to the primary of this replica set.
  */
-Status makeNoopWriteIfNeeded(OperationContext* opCtx, LogicalTime clusterTime) {
+Status makeNoopWriteIfNeeded(OperationContext* opCtx, LogicalTime clusterTime, StringData dbName) {
     repl::ReplicationCoordinator* const replCoord = repl::ReplicationCoordinator::get(opCtx);
     invariant(replCoord->isReplEnabled());
 
@@ -137,26 +140,14 @@ Status makeNoopWriteIfNeeded(OperationContext* opCtx, LogicalTime clusterTime) {
     // one that waits for the notification gets the later clusterTime, so when the request finishes
     // it needs to be repeated with the later time.
     while (clusterTime > lastAppliedOpTime) {
-        // standalone replica set, so there is no need to advance the OpLog on the primary.
-        if (serverGlobalParams.clusterRole == ClusterRole::None) {
+        // Standalone replica set, so there is no need to advance the OpLog on the primary. The only
+        // exception is after a tenant migration because the target time may be from the other
+        // replica set and is not guaranteed to be in the oplog of this node's set.
+        if (serverGlobalParams.clusterRole == ClusterRole::None &&
+            !tenant_migration_access_blocker::hasActiveTenantMigration(opCtx, dbName)) {
             return Status::OK();
         }
 
-        bool isConfig = (serverGlobalParams.clusterRole == ClusterRole::ConfigServer);
-
-        if (!isConfig && !ShardingState::get(opCtx)->enabled()) {
-            return {ErrorCodes::ShardingStateNotInitialized,
-                    "Failed noop write because sharding state has not been initialized"};
-        }
-
-        auto myShard = isConfig ? Grid::get(opCtx)->shardRegistry()->getConfigShard()
-                                : Grid::get(opCtx)->shardRegistry()->getShard(
-                                      opCtx, ShardingState::get(opCtx)->shardId());
-
-        if (!myShard.isOK()) {
-            return myShard.getStatus();
-        }
-
         if (!remainingAttempts--) {
             std::stringstream ss;
             ss << "Requested clusterTime " << clusterTime.toString()
@@ -174,17 +165,21 @@ Status makeNoopWriteIfNeeded(OperationContext* opCtx, LogicalTime clusterTime) {
                             "attempts: {remainingAttempts}",
                             "clusterTime"_attr = clusterTime.toString(),
                             "remainingAttempts"_attr = remainingAttempts);
-                auto swRes = myShard.getValue()->runCommand(
+
+                auto onRemoteCmdScheduled = [](executor::TaskExecutor::CallbackHandle handle) {};
+                auto onRemoteCmdComplete = [](executor::TaskExecutor::CallbackHandle handle) {};
+                auto appendOplogNoteResponse = replCoord->runCmdOnPrimaryAndAwaitResponse(
                     opCtx,
-                    ReadPreferenceSetting(ReadPreference::PrimaryOnly),
-                    "admin",
+                    NamespaceString::kAdminDb.toString(),
                     BSON("appendOplogNote"
                          << 1 << "maxClusterTime" << clusterTime.asTimestamp() << "data"
                          << BSON("noop write for afterClusterTime read concern" << 1)
                          << WriteConcernOptions::kWriteConcernField
-                         << WriteConcernOptions::kImplicitDefault),
-                    Shard::RetryPolicy::kIdempotent);
-                status = swRes.getStatus();
+                         << WriteConcernOptions::kInternalWriteDefault),
+                    onRemoteCmdScheduled,
+                    onRemoteCmdComplete);
+
+                status = getStatusFromCommandResult(appendOplogNoteResponse);
                 std::get<1>(myWriteRequest)->set(status);
                 writeRequests.deleteWriteRequest(clusterTime);
             } catch (const DBException& ex) {
@@ -210,6 +205,20 @@ Status makeNoopWriteIfNeeded(OperationContext* opCtx, LogicalTime clusterTime) {
         if (status.isOK()) {
             return status;
         }
+
+        // If the write failed with StaleClusterTime it means that the noop write to the primary was
+        // not necessary to bump the clusterTime. It could be a race where the secondary decides to
+        // issue the noop write while some writes have already happened on the primary that have
+        // bumped the clusterTime beyond the 'clusterTime' the noop write requested.
+        if (status == ErrorCodes::StaleClusterTime) {
+            LOGV2_DEBUG(54102,
+                        2,
+                        "appendOplogNote request on clusterTime {clusterTime} failed with "
+                        "StaleClusterTime",
+                        "clusterTime"_attr = clusterTime.asTimestamp());
+            return Status::OK();
+        }
+
         lastAppliedOpTime = LogicalTime(replCoord->getMyLastAppliedOpTime().getTimestamp());
     }
     // This is when the noop write failed but the opLog caught up to clusterTime by replicating.
@@ -270,6 +279,7 @@ void setPrepareConflictBehaviorForReadConcernImpl(OperationContext* opCtx,
 
 Status waitForReadConcernImpl(OperationContext* opCtx,
                               const repl::ReadConcernArgs& readConcernArgs,
+                              StringData dbName,
                               bool allowAfterClusterTime) {
     // If we are in a direct client within a transaction, then we may be holding locks, so it is
     // illegal to wait for read concern. This is fine, since the outer operation should have handled
@@ -300,6 +310,18 @@ Status waitForReadConcernImpl(OperationContext* opCtx,
         }
     }
 
+    if (readConcernArgs.getLevel() == repl::ReadConcernLevel::kSnapshotReadConcern) {
+        if (replCoord->getReplicationMode() != repl::ReplicationCoordinator::modeReplSet) {
+            return {ErrorCodes::NotAReplicaSet,
+                    "node needs to be a replica set member to use readConcern: snapshot"};
+        }
+        if (!opCtx->inMultiDocumentTransaction() && !serverGlobalParams.enableMajorityReadConcern) {
+            return {ErrorCodes::ReadConcernMajorityNotEnabled,
+                    "read concern level snapshot is not supported when "
+                    "enableMajorityReadConcern=false"};
+        }
+    }
+
     auto afterClusterTime = readConcernArgs.getArgsAfterClusterTime();
     auto atClusterTime = readConcernArgs.getArgsAtClusterTime();
 
@@ -328,8 +350,9 @@ Status waitForReadConcernImpl(OperationContext* opCtx,
             // STARTUP or STARTUP2.
             const auto memberState = replCoord->getMemberState();
 
-            const auto currentTime = LogicalClock::get(opCtx)->getClusterTime();
-            if (currentTime == LogicalTime::kUninitialized) {
+            const auto currentTime = VectorClock::get(opCtx)->getTime();
+            const auto clusterTime = currentTime.clusterTime();
+            if (!VectorClock::isValidComponentTime(clusterTime)) {
                 // currentTime should only be uninitialized if we are in startup recovery or initial
                 // sync.
                 invariant(memberState.startup() || memberState.startup2());
@@ -337,18 +360,18 @@ Status waitForReadConcernImpl(OperationContext* opCtx,
                         str::stream() << "Current clusterTime is uninitialized, cannot service the "
                                          "requested clusterTime. Requested clusterTime: "
                                       << targetClusterTime->toString()
-                                      << "; current clusterTime: " << currentTime.toString()};
+                                      << "; current clusterTime: " << clusterTime.toString()};
             }
-            if (currentTime < *targetClusterTime) {
+            if (clusterTime < *targetClusterTime) {
                 return {ErrorCodes::InvalidOptions,
                         str::stream() << "readConcern " << readConcernName
                                       << " value must not be greater than the current clusterTime. "
                                          "Requested clusterTime: "
                                       << targetClusterTime->toString()
-                                      << "; current clusterTime: " << currentTime.toString()};
+                                      << "; current clusterTime: " << clusterTime.toString()};
             }
 
-            auto status = makeNoopWriteIfNeeded(opCtx, *targetClusterTime);
+            auto status = makeNoopWriteIfNeeded(opCtx, *targetClusterTime, dbName);
             if (!status.isOK()) {
                 LOGV2(20990,
                       "Failed noop write at clusterTime: {targetClusterTime} due to {error}",
@@ -366,16 +389,19 @@ Status waitForReadConcernImpl(OperationContext* opCtx,
         }
     }
 
-    if (readConcernArgs.getLevel() == repl::ReadConcernLevel::kSnapshotReadConcern) {
-        if (replCoord->getReplicationMode() != repl::ReplicationCoordinator::modeReplSet) {
-            return {ErrorCodes::NotAReplicaSet,
-                    "node needs to be a replica set member to use readConcern: snapshot"};
-        }
-    }
-
+    auto ru = opCtx->recoveryUnit();
     if (atClusterTime) {
-        opCtx->recoveryUnit()->setTimestampReadSource(RecoveryUnit::ReadSource::kProvided,
-                                                      atClusterTime->asTimestamp());
+        ru->setTimestampReadSource(RecoveryUnit::ReadSource::kProvided,
+                                   atClusterTime->asTimestamp());
+    } else if (readConcernArgs.getLevel() == repl::ReadConcernLevel::kSnapshotReadConcern &&
+               replCoord->getReplicationMode() == repl::ReplicationCoordinator::Mode::modeReplSet &&
+               !opCtx->inMultiDocumentTransaction()) {
+        auto opTime = replCoord->getCurrentCommittedSnapshotOpTime();
+        uassert(ErrorCodes::SnapshotUnavailable,
+                "No committed OpTime for snapshot read",
+                !opTime.isNull());
+        ru->setTimestampReadSource(RecoveryUnit::ReadSource::kProvided, opTime.getTimestamp());
+        repl::ReadConcernArgs::get(opCtx).setArgsAtClusterTimeForSnapshot(opTime.getTimestamp());
     } else if (readConcernArgs.getLevel() == repl::ReadConcernLevel::kMajorityReadConcern &&
                replCoord->getReplicationMode() == repl::ReplicationCoordinator::Mode::modeReplSet) {
         // This block is not used for kSnapshotReadConcern because snapshots are always speculative;
@@ -390,7 +416,7 @@ Status waitForReadConcernImpl(OperationContext* opCtx,
             // always reading at the minimum of the all-committed and lastApplied timestamps. This
             // allows for safe behavior on both primaries and secondaries, where the behavior of the
             // all-committed and lastApplied timestamps differ significantly.
-            opCtx->recoveryUnit()->setTimestampReadSource(RecoveryUnit::ReadSource::kNoOverlap);
+            ru->setTimestampReadSource(RecoveryUnit::ReadSource::kNoOverlap);
             auto& speculativeReadInfo = repl::SpeculativeMajorityReadInfo::get(opCtx);
             speculativeReadInfo.setIsSpeculativeRead();
             return Status::OK();
@@ -404,14 +430,14 @@ Status waitForReadConcernImpl(OperationContext* opCtx,
             "Waiting for 'committed' snapshot to be available for reading: {readConcernArgs}",
             "readConcernArgs"_attr = readConcernArgs);
 
-        opCtx->recoveryUnit()->setTimestampReadSource(RecoveryUnit::ReadSource::kMajorityCommitted);
-        Status status = opCtx->recoveryUnit()->obtainMajorityCommittedSnapshot();
+        ru->setTimestampReadSource(RecoveryUnit::ReadSource::kMajorityCommitted);
+        Status status = ru->majorityCommittedSnapshotAvailable();
 
         // Wait until a snapshot is available.
         while (status == ErrorCodes::ReadConcernMajorityNotAvailableYet) {
             LOGV2_DEBUG(20992, debugLevel, "Snapshot not available yet.");
             replCoord->waitUntilSnapshotCommitted(opCtx, Timestamp());
-            status = opCtx->recoveryUnit()->obtainMajorityCommittedSnapshot();
+            status = ru->majorityCommittedSnapshotAvailable();
         }
 
         if (!status.isOK()) {
@@ -420,11 +446,8 @@ Status waitForReadConcernImpl(OperationContext* opCtx,
 
         LOGV2_DEBUG(20993,
                     debugLevel,
-                    "Using 'committed' snapshot: {CurOp_get_opCtx_opDescription} with readTs: "
-                    "{opCtx_recoveryUnit_getPointInTimeReadTimestamp}",
-                    "CurOp_get_opCtx_opDescription"_attr = CurOp::get(opCtx)->opDescription(),
-                    "opCtx_recoveryUnit_getPointInTimeReadTimestamp"_attr =
-                        opCtx->recoveryUnit()->getPointInTimeReadTimestamp());
+                    "Using 'committed' snapshot",
+                    "operation_description"_attr = CurOp::get(opCtx)->opDescription());
     }
     return Status::OK();
 }
@@ -498,7 +521,11 @@ Status waitForSpeculativeMajorityReadConcernImpl(
         // Speculative majority reads are required to use the 'kNoOverlap' read source.
         invariant(opCtx->recoveryUnit()->getTimestampReadSource() ==
                   RecoveryUnit::ReadSource::kNoOverlap);
-        boost::optional<Timestamp> readTs = opCtx->recoveryUnit()->getPointInTimeReadTimestamp();
+
+        // Storage engine operations require at least Global IS.
+        Lock::GlobalLock lk(opCtx, MODE_IS);
+        boost::optional<Timestamp> readTs =
+            opCtx->recoveryUnit()->getPointInTimeReadTimestamp(opCtx);
         invariant(readTs);
         waitTs = *readTs;
     }
