@@ -32,10 +32,11 @@
 #include "mongo/db/repl/oplog_applier_impl_test_fixture.h"
 
 #include "mongo/db/catalog/document_validation.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/dbdirectclient.h"
-#include "mongo/db/logical_clock.h"
+#include "mongo/db/index_builds_coordinator.h"
 #include "mongo/db/op_observer_registry.h"
 #include "mongo/db/query/internal_plans.h"
 #include "mongo/db/repl/drop_pending_collection_reaper.h"
@@ -46,6 +47,7 @@
 #include "mongo/db/repl/replication_recovery_mock.h"
 #include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/repl/storage_interface_impl.h"
+#include "mongo/db/vector_clock_mutable.h"
 
 namespace mongo {
 namespace repl {
@@ -71,16 +73,23 @@ void OplogApplierImplOpObserver::onDelete(OperationContext* opCtx,
                                           const NamespaceString& nss,
                                           OptionalCollectionUUID uuid,
                                           StmtId stmtId,
-                                          bool fromMigrate,
-                                          const boost::optional<BSONObj>& deletedDoc) {
+                                          const OpObserver::OplogDeleteEntryArgs& args) {
     if (!onDeleteFn) {
         return;
     }
-    onDeleteFn(opCtx, nss, uuid, stmtId, fromMigrate, deletedDoc);
+    onDeleteFn(opCtx, nss, uuid, stmtId, args);
+}
+
+void OplogApplierImplOpObserver::onUpdate(OperationContext* opCtx,
+                                          const OplogUpdateEntryArgs& args) {
+    if (!onUpdateFn) {
+        return;
+    }
+    onUpdateFn(opCtx, args);
 }
 
 void OplogApplierImplOpObserver::onCreateCollection(OperationContext* opCtx,
-                                                    Collection* coll,
+                                                    const CollectionPtr& coll,
                                                     const NamespaceString& collectionName,
                                                     const CollectionOptions& options,
                                                     const BSONObj& idIndex,
@@ -90,6 +99,55 @@ void OplogApplierImplOpObserver::onCreateCollection(OperationContext* opCtx,
     }
     onCreateCollectionFn(opCtx, coll, collectionName, options, idIndex);
 }
+
+void OplogApplierImplOpObserver::onRenameCollection(OperationContext* opCtx,
+                                                    const NamespaceString& fromCollection,
+                                                    const NamespaceString& toCollection,
+                                                    OptionalCollectionUUID uuid,
+                                                    OptionalCollectionUUID dropTargetUUID,
+                                                    std::uint64_t numRecords,
+                                                    bool stayTemp) {
+    if (!onRenameCollectionFn) {
+        return;
+    }
+    onRenameCollectionFn(
+        opCtx, fromCollection, toCollection, uuid, dropTargetUUID, numRecords, stayTemp);
+}
+
+void OplogApplierImplOpObserver::onCreateIndex(OperationContext* opCtx,
+                                               const NamespaceString& nss,
+                                               CollectionUUID uuid,
+                                               BSONObj indexDoc,
+                                               bool fromMigrate) {
+    if (!onCreateIndexFn) {
+        return;
+    }
+    onCreateIndexFn(opCtx, nss, uuid, indexDoc, fromMigrate);
+}
+
+void OplogApplierImplOpObserver::onDropIndex(OperationContext* opCtx,
+                                             const NamespaceString& nss,
+                                             OptionalCollectionUUID uuid,
+                                             const std::string& indexName,
+                                             const BSONObj& idxDescriptor) {
+    if (!onDropIndexFn) {
+        return;
+    }
+    onDropIndexFn(opCtx, nss, uuid, indexName, idxDescriptor);
+}
+
+void OplogApplierImplOpObserver::onCollMod(OperationContext* opCtx,
+                                           const NamespaceString& nss,
+                                           const UUID& uuid,
+                                           const BSONObj& collModCmd,
+                                           const CollectionOptions& oldCollOptions,
+                                           boost::optional<IndexCollModInfo> indexInfo) {
+    if (!onCollModFn) {
+        return;
+    }
+    onCollModFn(opCtx, nss, uuid, collModCmd, oldCollOptions, indexInfo);
+}
+
 void OplogApplierImplTest::setUp() {
     ServiceContextMongoDTest::setUp();
 
@@ -104,7 +162,6 @@ void OplogApplierImplTest::setUp() {
 
     DropPendingCollectionReaper::set(
         serviceContext, std::make_unique<DropPendingCollectionReaper>(getStorageInterface()));
-    repl::setOplogCollectionName(serviceContext);
     repl::createOplog(_opCtx.get());
 
     _consistencyMarkers = std::make_unique<ReplicationConsistencyMarkersMock>();
@@ -118,12 +175,13 @@ void OplogApplierImplTest::setUp() {
     // Initialize the featureCompatibilityVersion server parameter. This is necessary because this
     // test fixture does not create a featureCompatibilityVersion document from which to initialize
     // the server parameter.
-    serverGlobalParams.featureCompatibility.setVersion(
-        ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo44);
+    // (Generic FCV reference): This FCV reference should exist across LTS binary versions.
+    serverGlobalParams.mutableFeatureCompatibility.setVersion(
+        ServerGlobalParams::FeatureCompatibility::kLatest);
 
     // This is necessary to generate ghost timestamps for index builds that are not 0, since 0 is an
     // invalid timestamp.
-    ASSERT_OK(LogicalClock::get(_opCtx.get())->advanceClusterTime(LogicalTime(Timestamp(1, 0))));
+    VectorClockMutable::get(_opCtx.get())->tickClusterTimeTo(LogicalTime(Timestamp(1, 0)));
 }
 
 void OplogApplierImplTest::tearDown() {
@@ -150,7 +208,8 @@ Status OplogApplierImplTest::_applyOplogEntryOrGroupedInsertsWrapper(
     OplogApplication::Mode oplogApplicationMode) {
     UnreplicatedWritesBlock uwb(opCtx);
     DisableDocumentValidation validationDisabler(opCtx);
-    return applyOplogEntryOrGroupedInserts(opCtx, batch, oplogApplicationMode);
+    const bool dataIsConsistent = true;
+    return applyOplogEntryOrGroupedInserts(opCtx, batch, oplogApplicationMode, dataIsConsistent);
 }
 
 void OplogApplierImplTest::_testApplyOplogEntryOrGroupedInsertsCrudOperation(
@@ -164,7 +223,7 @@ void OplogApplierImplTest::_testApplyOplogEntryOrGroupedInsertsCrudOperation(
         ASSERT_TRUE(
             opCtx->lockState()->isCollectionLockedForMode(NamespaceString("test.t"), MODE_IX));
         ASSERT_FALSE(opCtx->writesAreReplicated());
-        ASSERT_TRUE(documentValidationDisabled(opCtx));
+        ASSERT_TRUE(DocumentValidationSettings::get(opCtx).isSchemaValidationDisabled());
     };
 
     _opObserver->onInsertsFn =
@@ -173,7 +232,10 @@ void OplogApplierImplTest::_testApplyOplogEntryOrGroupedInsertsCrudOperation(
             checkOpCtx(opCtx);
             ASSERT_EQUALS(NamespaceString("test.t"), nss);
             ASSERT_EQUALS(1U, docs.size());
-            ASSERT_BSONOBJ_EQ(op.getObject(), docs[0]);
+            // For upserts we don't know the intended value of the document.
+            if (op.getOpType() == repl::OpTypeEnum::kInsert) {
+                ASSERT_BSONOBJ_EQ(op.getObject(), docs[0]);
+            }
             return Status::OK();
         };
 
@@ -181,13 +243,19 @@ void OplogApplierImplTest::_testApplyOplogEntryOrGroupedInsertsCrudOperation(
                                   const NamespaceString& nss,
                                   OptionalCollectionUUID uuid,
                                   StmtId stmtId,
-                                  bool fromMigrate,
-                                  const boost::optional<BSONObj>& deletedDoc) {
+                                  const OpObserver::OplogDeleteEntryArgs& args) {
         applyOpCalled = true;
         checkOpCtx(opCtx);
         ASSERT_EQUALS(NamespaceString("test.t"), nss);
-        ASSERT(deletedDoc);
-        ASSERT_BSONOBJ_EQ(op.getObject(), *deletedDoc);
+        ASSERT(args.deletedDoc);
+        ASSERT_BSONOBJ_EQ(op.getObject(), *(args.deletedDoc));
+        return Status::OK();
+    };
+
+    _opObserver->onUpdateFn = [&](OperationContext* opCtx, const OplogUpdateEntryArgs& args) {
+        applyOpCalled = true;
+        checkOpCtx(opCtx);
+        ASSERT_EQUALS(NamespaceString("test.t"), args.nss);
         return Status::OK();
     };
 
@@ -218,7 +286,9 @@ Status OplogApplierImplTest::runOpsSteadyState(std::vector<OplogEntry> ops) {
         opsPtrs.push_back(&op);
     }
     WorkerMultikeyPathInfo pathInfo;
-    return oplogApplier.applyOplogBatchPerWorker(_opCtx.get(), &opsPtrs, &pathInfo);
+    const bool dataIsConsistent = true;
+    return oplogApplier.applyOplogBatchPerWorker(
+        _opCtx.get(), &opsPtrs, &pathInfo, dataIsConsistent);
 }
 
 Status OplogApplierImplTest::runOpInitialSync(const OplogEntry& op) {
@@ -251,11 +321,18 @@ Status OplogApplierImplTest::runOpsInitialSync(std::vector<OplogEntry> ops) {
     // its own batch and update oplog visibility after each batch to make sure all previously
     // applied entries are visible to subsequent batches.
     for (auto& op : ops) {
-        auto status = oplogApplier.applyOplogBatch(_opCtx.get(), {op});
-        if (!status.isOK()) {
-            return status.getStatus();
+        auto applyResult = oplogApplier.applyOplogBatch(_opCtx.get(), {op});
+        if (!applyResult.isOK()) {
+            std::vector<BSONObj> docsFromOps;
+            for (const auto& opForContext : ops) {
+                docsFromOps.push_back(opForContext.getEntry().toBSON());
+            }
+            auto status = applyResult.getStatus();
+            return status.withContext(str::stream()
+                                      << "failed to apply operation: " << op.toBSONForLogging()
+                                      << ". " << BSON("ops" << docsFromOps));
         }
-        auto lastApplied = status.getValue();
+        auto lastApplied = applyResult.getValue();
         const bool orderedCommit = true;
         // Update oplog visibility by notifying the storage engine of the new oplog entries.
         storageInterface->oplogDiskLocRegister(
@@ -296,9 +373,8 @@ void checkTxnTable(OperationContext* opCtx,
 CollectionReader::CollectionReader(OperationContext* opCtx, const NamespaceString& nss)
     : _collToScan(opCtx, nss),
       _exec(InternalPlanner::collectionScan(opCtx,
-                                            nss.ns(),
-                                            _collToScan.getCollection(),
-                                            PlanExecutor::NO_YIELD,
+                                            &_collToScan.getCollection(),
+                                            PlanYieldPolicy::YieldPolicy::NO_YIELD,
                                             InternalPlanner::FORWARD)) {}
 
 StatusWith<BSONObj> CollectionReader::next() {
@@ -320,5 +396,100 @@ bool docExists(OperationContext* opCtx, const NamespaceString& nss, const BSONOb
     auto result = client.findOne(nss.ns(), {doc});
     return !result.isEmpty();
 }
+
+/**
+ * Creates an OplogEntry with given parameters and preset defaults for this test suite.
+ */
+OplogEntry makeOplogEntry(OpTypeEnum opType,
+                          NamespaceString nss,
+                          OptionalCollectionUUID uuid,
+                          BSONObj o,
+                          boost::optional<BSONObj> o2) {
+    return {DurableOplogEntry(OpTime(Timestamp(1, 1), 1),  // optime
+                              boost::none,                 // hash
+                              opType,                      // opType
+                              nss,                         // namespace
+                              uuid,                        // uuid
+                              boost::none,                 // fromMigrate
+                              OplogEntry::kOplogVersion,   // version
+                              o,                           // o
+                              o2,                          // o2
+                              {},                          // sessionInfo
+                              boost::none,                 // upsert
+                              Date_t(),                    // wall clock time
+                              {},                          // statement ids
+                              boost::none,    // optime of previous write within same transaction
+                              boost::none,    // pre-image optime
+                              boost::none,    // post-image optime
+                              boost::none,    // ShardId of resharding recipient
+                              boost::none,    // _id
+                              boost::none)};  // needsRetryImage
+}
+
+OplogEntry makeOplogEntry(OpTypeEnum opType, NamespaceString nss, OptionalCollectionUUID uuid) {
+    return makeOplogEntry(opType, nss, uuid, BSON("_id" << 0), boost::none);
+}
+
+CollectionOptions createOplogCollectionOptions() {
+    CollectionOptions options;
+    options.capped = true;
+    options.cappedSize = 64 * 1024 * 1024LL;
+    options.autoIndexId = CollectionOptions::NO;
+    return options;
+}
+
+CollectionOptions createRecordPreImageCollectionOptions() {
+    CollectionOptions options;
+    options.recordPreImages = true;
+    return options;
+}
+
+void createCollection(OperationContext* opCtx,
+                      const NamespaceString& nss,
+                      const CollectionOptions& options) {
+    writeConflictRetry(opCtx, "createCollection", nss.ns(), [&] {
+        Lock::DBLock dbLk(opCtx, nss.db(), MODE_IX);
+        Lock::CollectionLock collLk(opCtx, nss, MODE_X);
+        OldClientContext ctx(opCtx, nss.ns());
+        auto db = ctx.db();
+        ASSERT_TRUE(db);
+        mongo::WriteUnitOfWork wuow(opCtx);
+        auto coll = db->createCollection(opCtx, nss, options);
+        ASSERT_TRUE(coll);
+        wuow.commit();
+    });
+}
+
+UUID createCollectionWithUuid(OperationContext* opCtx, const NamespaceString& nss) {
+    CollectionOptions options;
+    options.uuid = UUID::gen();
+    createCollection(opCtx, nss, options);
+    return options.uuid.get();
+}
+
+void createDatabase(OperationContext* opCtx, StringData dbName) {
+    Lock::GlobalWrite globalLock(opCtx);
+    bool justCreated;
+    auto databaseHolder = DatabaseHolder::get(opCtx);
+    auto db = databaseHolder->openDb(opCtx, dbName, &justCreated);
+    ASSERT_TRUE(db);
+    ASSERT_TRUE(justCreated);
+}
+
+bool collectionExists(OperationContext* opCtx, const NamespaceString& nss) {
+    return AutoGetCollectionForRead(opCtx, nss).getCollection() != nullptr;
+}
+
+void createIndex(OperationContext* opCtx,
+                 const NamespaceString& nss,
+                 const UUID collUUID,
+                 const BSONObj& spec) {
+    Lock::DBLock dbLk(opCtx, nss.db(), MODE_IX);
+    Lock::CollectionLock collLk(opCtx, nss, MODE_X);
+    auto indexBuildsCoord = IndexBuildsCoordinator::get(opCtx);
+    indexBuildsCoord->createIndex(
+        opCtx, collUUID, spec, IndexBuildsManager::IndexConstraints::kEnforce, false);
+}
+
 }  // namespace repl
 }  // namespace mongo
