@@ -85,6 +85,14 @@ public:
     void recoverFromOplogUpTo(OperationContext* opCtx, Timestamp endPoint) override;
 
 private:
+    enum class RecoveryMode {
+        kStartupFromStableTimestamp,
+        kStartupFromUnstableCheckpoint,
+        kRollbackFromStableTimestamp,
+        // There is no RecoveryMode::kRollbackFromUnstableCheckpoint, rollback can only recover from
+        // a stable timestamp.
+    };
+
     /**
      * Confirms that the data and oplog all indicate that the nodes has an unstable checkpoint
      * that is fully up to date.
@@ -97,8 +105,8 @@ private:
      */
     void _recoverFromStableTimestamp(OperationContext* opCtx,
                                      Timestamp stableTimestamp,
-                                     OpTime appliedThrough,
-                                     OpTime topOfOplog);
+                                     OpTime topOfOplog,
+                                     RecoveryMode recoveryMode);
 
     /**
      * After truncating the oplog, completes recovery if we're recovering from an unstable
@@ -109,20 +117,22 @@ private:
                                         OpTime topOfOplog);
 
     /**
-     * Applies all oplog entries from oplogApplicationStartPoint (inclusive) to topOfOplog
+     * Applies all oplog entries from oplogApplicationStartPoint (exclusive) to topOfOplog
      * (inclusive). This fasserts if oplogApplicationStartPoint is not in the oplog.
      */
     void _applyToEndOfOplog(OperationContext* opCtx,
                             const Timestamp& oplogApplicationStartPoint,
-                            const Timestamp& topOfOplog);
+                            const Timestamp& topOfOplog,
+                            RecoveryMode recoveryMode);
 
     /**
-     * Applies all oplog entries from startPoint (inclusive) to endPoint (inclusive). Returns the
+     * Applies all oplog entries from startPoint (exclusive) to endPoint (inclusive). Returns the
      * Timestamp of the last applied operation.
      */
     Timestamp _applyOplogOperations(OperationContext* opCtx,
                                     const Timestamp& startPoint,
-                                    const Timestamp& endPoint);
+                                    const Timestamp& endPoint,
+                                    RecoveryMode recoveryMode);
 
     /**
      * Gets the last applied OpTime from the end of the oplog. Returns CollectionIsEmpty if there is
@@ -132,8 +142,11 @@ private:
 
     /**
      * Truncates the oplog after the "truncateAfterTimestamp" entry.
+     * If the stableTimestamp is set, may move it backwards to the new top of oplog.
      */
-    void _truncateOplogTo(OperationContext* opCtx, Timestamp truncateAfterTimestamp);
+    void _truncateOplogTo(OperationContext* opCtx,
+                          Timestamp truncateAfterTimestamp,
+                          boost::optional<Timestamp>* stableTimestamp);
 
     /**
      * Uses the oplogTruncateAfterPoint, accessed via '_consistencyMarkers', to decide whether to
@@ -146,9 +159,24 @@ private:
      * stable timestamp. The oplogTruncateAfterPoint can lag behind the stable timestamp because the
      * oplogTruncateAfterPoint is updated on primaries by an asynchronously looping thread that can
      * potentially be starved.
+     *
+     * If the stable timestamp is at a hole, this will move the stable timestamp back to the new
+     * top of oplog.  This can happen on primaries when EMRC=false or in single-node replica sets.
      */
     void _truncateOplogIfNeededAndThenClearOplogTruncateAfterPoint(
-        OperationContext* opCtx, boost::optional<Timestamp> stableTimestamp);
+        OperationContext* opCtx, boost::optional<Timestamp>* stableTimestamp);
+
+    /**
+     * Checks if the proposed oplog application start point (which is typically derived from the
+     * stable timestamp) exists in the oplog. If it does, this returns that same start point
+     * unchanged. If that point is not in the oplog, this function returns an entry before
+     * that start point.
+     * It is safe to do as as we make sure that we always keep an oplog entry that is less than
+     * or equal to the stable timestamp so such a correction always pushes the start point back and
+     * never forward. Applying entries from an earlier point is permissible due to oplog entry
+     * idempotency (and also due to the order being preserved.)
+     */
+    Timestamp _adjustStartPointIfNecessary(OperationContext* opCtx, Timestamp startPoint);
 
     StorageInterface* _storageInterface;
     ReplicationConsistencyMarkers* _consistencyMarkers;
