@@ -82,14 +82,6 @@ public:
      */
     void throwIfReshardingInProgress(NamespaceString const& nss) const;
 
-    /**
-     * The caller should disallow writes when
-     *      1. The coordinator is in the mirroring state, OR
-     *      2. The coordinator is in the decision persisted state, but the UUID is still the
-     *         original UUID.
-     */
-    bool disallowWritesForResharding(const UUID& currentCollectionUUID) const;
-
     /**
      * Returns the current shard version for the collection or UNSHARDED if it is not sharded.
      *
