@@ -38,6 +38,7 @@
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/client.h"
 #include "mongo/db/commands.h"
+#include "mongo/db/dbdirectclient.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/s/database_sharding_state.h"
@@ -55,6 +56,30 @@
 namespace mongo {
 namespace {
 
+/**
+ * Inserts a database collection entry with fixed metadata for the `config` or `admin` database. If
+ * the entry key already exists, it's not updated.
+ */
+Status insertDatabaseEntryForBackwardCompatibility(OperationContext* opCtx,
+                                                   const StringData& dbName) {
+    invariant(dbName == NamespaceString::kAdminDb || dbName == NamespaceString::kConfigDb);
+
+    DBDirectClient client(opCtx);
+    auto commandResponse = client.runCommand([&] {
+        auto dbMetadata = DatabaseType(dbName.toString(),
+                                       ShardId::kConfigServerId,
+                                       dbName == NamespaceString::kConfigDb,
+                                       DatabaseVersion::makeFixed());
+
+        write_ops::InsertCommandRequest insertOp(NamespaceString::kShardConfigDatabasesNamespace);
+        insertOp.setDocuments({dbMetadata.toBSON()});
+        return insertOp.serialize({});
+    }());
+
+    auto commandStatus = getStatusFromWriteCommandReply(commandResponse->getCommandReply());
+    return commandStatus.code() == ErrorCodes::DuplicateKey ? Status::OK() : commandStatus;
+}
+
 template <typename Derived>
 class FlushDatabaseCacheUpdatesCmdBase : public TypedCommand<Derived> {
 public:
@@ -119,6 +144,26 @@ public:
                     "Can't call _flushDatabaseCacheUpdates if in read-only mode",
                     !storageGlobalParams.readOnly);
 
+            if (_dbName() == NamespaceString::kAdminDb || _dbName() == NamespaceString::kConfigDb) {
+                // The admin and config databases have fixed metadata that does not need to be
+                // refreshed.
+
+                if (Base::request().getSyncFromConfig()) {
+                    // To ensure compatibility with old secondaries that still call the
+                    // _flushDatabaseCacheUpdates command to get updated database metadata from
+                    // primary, an entry with fixed metadata is inserted in the
+                    // config.cache.databases collection.
+
+                    LOGV2_DEBUG(6910800,
+                                1,
+                                "Inserting a database collection entry with fixed metadata",
+                                "db"_attr = _dbName());
+                    uassertStatusOK(insertDatabaseEntryForBackwardCompatibility(opCtx, _dbName()));
+                }
+
+                return;
+            }
+
             auto& oss = OperationShardingState::get(opCtx);
 
             {
