@@ -197,9 +197,11 @@ void retryIdempotentWorkAsPrimaryUntilSuccessOrStepdown(
     const auto initialTerm = repl::ReplicationCoordinator::get(opCtx)->getTerm();
 
     for (int attempt = 1;; attempt++) {
-        // If the server is already doing a clean shutdown, join the shutdown.
+        // Since we can't differenciate if a shutdown exception is coming from a remote node or
+        // locally we need to directly inspect the the global shutdown state to correctly interrupt
+        // this task in case this node is shutting down.
         if (globalInShutdownDeprecated()) {
-            shutdown(waitForShutdown());
+            uasserted(ErrorCodes::ShutdownInProgress, "Shutdown in progress");
         }
 
         // If the node is no longer primary, stop retrying.
@@ -932,7 +934,9 @@ void resumeMigrationCoordinationsOnStepUp(OperationContext* opCtx) {
                 "unfinishedMigrationsCount"_attr = unfinishedMigrationsCount);
 }
 
-void recoverMigrationCoordinations(OperationContext* opCtx, NamespaceString nss) {
+void recoverMigrationCoordinations(OperationContext* opCtx,
+                                   NamespaceString nss,
+                                   CancellationToken cancellationToken) {
     LOGV2_DEBUG(4798501, 2, "Starting migration recovery", "namespace"_attr = nss);
 
     unsigned migrationRecoveryCount = 0;
@@ -941,7 +945,8 @@ void recoverMigrationCoordinations(OperationContext* opCtx, NamespaceString nss)
     store.forEach(
         opCtx,
         QUERY(MigrationCoordinatorDocument::kNssFieldName << nss.toString()),
-        [&opCtx, &migrationRecoveryCount](const MigrationCoordinatorDocument& doc) {
+        [&opCtx, &migrationRecoveryCount, &cancellationToken](
+            const MigrationCoordinatorDocument& doc) {
             LOGV2_DEBUG(4798502,
                         2,
                         "Recovering migration",
@@ -981,7 +986,7 @@ void recoverMigrationCoordinations(OperationContext* opCtx, NamespaceString nss)
                           "simulate an error response for forceShardFilteringMetadataRefresh");
             }
 
-            auto setFilteringMetadata = [&opCtx, &currentMetadata, &doc]() {
+            auto setFilteringMetadata = [&opCtx, &currentMetadata, &doc, &cancellationToken]() {
                 AutoGetDb autoDb(opCtx, doc.getNss().db(), MODE_IX);
                 Lock::CollectionLock collLock(opCtx, doc.getNss(), MODE_IX);
                 auto* const csr = CollectionShardingRuntime::get(opCtx, doc.getNss());
@@ -989,7 +994,10 @@ void recoverMigrationCoordinations(OperationContext* opCtx, NamespaceString nss)
                 auto optMetadata = csr->getCurrentMetadataIfKnown();
                 invariant(!optMetadata);
 
-                csr->setFilteringMetadata(opCtx, std::move(currentMetadata));
+                auto csrLock = CollectionShardingRuntime::CSRLock::lockExclusive(opCtx, csr);
+                if (!cancellationToken.isCanceled()) {
+                    csr->setFilteringMetadata_withLock(opCtx, std::move(currentMetadata), csrLock);
+                }
             };
 
             if (!currentMetadata.isSharded() ||
