@@ -32,15 +32,14 @@
 #include <boost/optional.hpp>
 
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/s/chunk_version.h"
-#include "mongo/s/database_version_gen.h"
-#include "mongo/util/concurrency/notification.h"
+#include "mongo/s/database_version.h"
+#include "mongo/util/future.h"
 #include "mongo/util/string_map.h"
 
 namespace mongo {
 
-class OperationContext;
-
 /**
  * A decoration on OperationContext representing per-operation shard version metadata sent to mongod
  * from mongos as a command parameter.
@@ -70,17 +69,21 @@ public:
     static bool isOperationVersioned(OperationContext* opCtx);
 
     /**
-     * Requests on a sharded collection that are broadcast without a shardVersion should not cause
-     * the collection to be created on a shard that does not know about the collection already,
-     * since the collection options will not be propagated. Such requests specify to disallow
-     * collection creation, which is saved here.
+     * Instantiating this object on the stack indicates to the storage execution subsystem that it
+     * is allowed to create a collection in this context and that the caller is responsible for
+     * notifying the shard Sharding sybsystem of the collection creation.
+     *
+     * DO NOT add any new usages of this class without including someone from the Sharding Team on
+     * the code review.
      */
-    void setAllowImplicitCollectionCreation(const BSONElement& allowImplicitCollectionCreationElem);
+    class ScopedAllowImplicitCollectionCreate_UNSAFE {
+    public:
+        ScopedAllowImplicitCollectionCreate_UNSAFE(OperationContext* opCtx);
+        ~ScopedAllowImplicitCollectionCreate_UNSAFE();
 
-    /**
-     * Specifies whether the request is allowed to create database/collection implicitly.
-     */
-    bool allowImplicitCollectionCreation() const;
+    private:
+        OperationContext* const _opCtx;
+    };
 
     /**
      * Parses shardVersion and databaseVersion from 'cmdObj' and stores the results in this object
@@ -99,25 +102,26 @@ public:
     void initializeClientRoutingVersionsFromCommand(NamespaceString nss, const BSONObj& cmdObj);
 
     /**
-     * Stores the given shardVersion and databaseVersion for the given namespace.
+     * Stores the given shardVersion and databaseVersion for the given namespace. Note: The shard
+     * version for the given namespace stored in the OperationShardingState can be overwritten if it
+     * has not been checked yet.
      */
     void initializeClientRoutingVersions(NamespaceString nss,
                                          const boost::optional<ChunkVersion>& shardVersion,
                                          const boost::optional<DatabaseVersion>& dbVersion);
 
     /**
-     * Returns whether or not there is a shard version associated with this operation.
+     * Returns whether or not there is a shard version for the namespace associated with this
+     * operation.
      */
-    bool hasShardVersion() const;
+    bool hasShardVersion(const NamespaceString& nss) const;
 
     /**
      * Returns the shard version (i.e. maximum chunk version) of a namespace being used by the
      * operation. Documents in chunks which did not belong on this shard at this shard version
      * will be filtered out.
-     *
-     * Returns ChunkVersion::UNSHARDED() if setGlobalUnshardedShardVersion has been called.
      */
-    boost::optional<ChunkVersion> getShardVersion(const NamespaceString& nss) const;
+    boost::optional<ChunkVersion> getShardVersion(const NamespaceString& nss);
 
     /**
      * Returns true if the client sent a databaseVersion for any namespace.
@@ -130,12 +134,6 @@ public:
      */
     boost::optional<DatabaseVersion> getDbVersion(const StringData dbName) const;
 
-    /**
-     * Makes the OperationShardingState behave as if an UNSHARDED shardVersion was sent for every
-     * possible namespace.
-     */
-    void setGlobalUnshardedShardVersion();
-
     /**
      * This call is a no op if there isn't a currently active migration critical section. Otherwise
      * it will wait for the critical section to complete up to the remaining operation time.
@@ -150,7 +148,7 @@ public:
      * migration for the namespace and that it would be prudent to wait for the critical section to
      * complete before retrying so the router doesn't make wasteful requests.
      */
-    void setMigrationCriticalSectionSignal(std::shared_ptr<Notification<void>> critSecSignal);
+    void setMigrationCriticalSectionSignal(boost::optional<SharedSemiFuture<void>> critSecSignal);
 
     /**
      * This call is a no op if there isn't a currently active movePrimary critical section.
@@ -167,7 +165,7 @@ public:
      * movePrimary for the namespace and that it would be prudent to wait for the critical section
      * to complete before retrying so the router doesn't make wasteful requests.
      */
-    void setMovePrimaryCriticalSectionSignal(std::shared_ptr<Notification<void>> critSecSignal);
+    void setMovePrimaryCriticalSectionSignal(boost::optional<SharedSemiFuture<void>> critSecSignal);
 
     /**
      * Stores the failed status in _shardingOperationFailedStatus.
@@ -186,11 +184,10 @@ public:
     boost::optional<Status> resetShardingOperationFailedStatus();
 
 private:
-    // Specifies whether the request is allowed to create database/collection implicitly
-    bool _allowImplicitCollectionCreation{true};
+    friend class ShardServerOpObserver;  // For access to _allowCollectionCreation below
 
-    // Should be set to true if all collections accessed are expected to be unsharded.
-    bool _globalUnshardedShardVersion = false;
+    // Specifies whether the request is allowed to create database/collection implicitly
+    bool _allowCollectionCreation{false};
 
     // The OperationShardingState class supports storing shardVersions for multiple namespaces (and
     // databaseVersions for multiple databases), even though client code has not been written yet to
@@ -198,14 +195,17 @@ private:
     StringMap<ChunkVersion> _shardVersions;
     StringMap<DatabaseVersion> _databaseVersions;
 
+    // Stores shards that have undergone a version check.
+    StringSet _shardVersionsChecked;
+
     // This value will only be non-null if version check during the operation execution failed due
     // to stale version and there was a migration for that namespace, which was in critical section.
-    std::shared_ptr<Notification<void>> _migrationCriticalSectionSignal;
+    boost::optional<SharedSemiFuture<void>> _migrationCriticalSectionSignal;
 
     // This value will only be non-null if version check during the operation execution failed due
     // to stale version and there was a movePrimary for that namespace, which was in critical
     // section.
-    std::shared_ptr<Notification<void>> _movePrimaryCriticalSectionSignal;
+    boost::optional<SharedSemiFuture<void>> _movePrimaryCriticalSectionSignal;
 
     // This value can only be set when a rerouting exception occurs during a write operation, and
     // must be handled before this object gets destructed.
