@@ -39,6 +39,7 @@
 #include "mongo/db/ops/insert.h"
 #include "mongo/db/persistent_task_store.h"
 #include "mongo/db/s/collection_sharding_runtime.h"
+#include "mongo/db/s/recoverable_critical_section_service.h"
 #include "mongo/db/s/sharding_ddl_util.h"
 #include "mongo/db/s/sharding_logging.h"
 #include "mongo/db/s/sharding_state.h"
@@ -90,13 +91,14 @@ RenameCollectionCoordinator::RenameCollectionCoordinator(ShardingDDLCoordinatorS
                                                          const BSONObj& initialState)
     : ShardingDDLCoordinator(service, initialState),
       _doc(RenameCollectionCoordinatorDocument::parse(
-          IDLParserErrorContext("RenameCollectionCoordinatorDocument"), initialState)) {}
+          IDLParserErrorContext("RenameCollectionCoordinatorDocument"), initialState)),
+      _request(_doc.getRenameCollectionRequest()) {}
 
 void RenameCollectionCoordinator::checkIfOptionsConflict(const BSONObj& doc) const {
     const auto otherDoc = RenameCollectionCoordinatorDocument::parse(
         IDLParserErrorContext("RenameCollectionCoordinatorDocument"), doc);
 
-    const auto& selfReq = _doc.getRenameCollectionRequest().toBSON();
+    const auto& selfReq = _request.toBSON();
     const auto& otherReq = otherDoc.getRenameCollectionRequest().toBSON();
 
     uassert(ErrorCodes::ConflictingOperationInProgress,
@@ -107,7 +109,7 @@ void RenameCollectionCoordinator::checkIfOptionsConflict(const BSONObj& doc) con
 
 std::vector<StringData> RenameCollectionCoordinator::_acquireAdditionalLocks(
     OperationContext* opCtx) {
-    return {_doc.getTo().ns()};
+    return {_request.getTo().ns()};
 }
 
 boost::optional<BSONObj> RenameCollectionCoordinator::reportForCurrentOp(
@@ -118,7 +120,12 @@ boost::optional<BSONObj> RenameCollectionCoordinator::reportForCurrentOp(
     if (const auto& optComment = getForwardableOpMetadata().getComment()) {
         cmdBob.append(optComment.get().firstElement());
     }
-    cmdBob.appendElements(_doc.getRenameCollectionRequest().toBSON());
+    cmdBob.appendElements(_request.toBSON());
+
+    const auto currPhase = [&]() {
+        stdx::lock_guard l{_docMutex};
+        return _doc.getPhase();
+    }();
 
     BSONObjBuilder bob;
     bob.append("type", "op");
@@ -126,7 +133,7 @@ boost::optional<BSONObj> RenameCollectionCoordinator::reportForCurrentOp(
     bob.append("op", "command");
     bob.append("ns", nss().toString());
     bob.append("command", cmdBob.obj());
-    bob.append("currentPhase", _doc.getPhase());
+    bob.append("currentPhase", currPhase);
     bob.append("active", true);
     return bob.obj();
 }
@@ -139,15 +146,20 @@ void RenameCollectionCoordinator::_enterPhase(Phase newPhase) {
                 2,
                 "Rename collection coordinator phase transition",
                 "fromNs"_attr = nss(),
-                "toNs"_attr = _doc.getTo(),
+                "toNs"_attr = _request.getTo(),
                 "newPhase"_attr = RenameCollectionCoordinatorPhase_serializer(newDoc.getPhase()),
                 "oldPhase"_attr = RenameCollectionCoordinatorPhase_serializer(_doc.getPhase()));
 
     if (_doc.getPhase() == Phase::kUnset) {
-        _doc = _insertStateDocument(std::move(newDoc));
-        return;
+        newDoc = _insertStateDocument(std::move(newDoc));
+    } else {
+        newDoc = _updateStateDocument(cc().makeOperationContext().get(), std::move(newDoc));
+    }
+
+    {
+        stdx::unique_lock ul{_docMutex};
+        _doc = std::move(newDoc);
     }
-    _doc = _updateStateDocument(cc().makeOperationContext().get(), std::move(newDoc));
 }
 
 void RenameCollectionCoordinator::_performNoopRetryableWriteOnParticipants(
@@ -170,13 +182,16 @@ ExecutorFuture<void> RenameCollectionCoordinator::_runImpl(
     return ExecutorFuture<void>(**executor)
         .then(_executePhase(
             Phase::kCheckPreconditions,
-            [this, anchor = shared_from_this()] {
+            [this, executor = executor, anchor = shared_from_this()] {
                 auto opCtxHolder = cc().makeOperationContext();
                 auto* opCtx = opCtxHolder.get();
                 getForwardableOpMetadata().setOn(opCtx);
 
                 const auto& fromNss = nss();
-                const auto& toNss = _doc.getTo();
+                const auto& toNss = _request.getTo();
+
+                const auto criticalSectionReason =
+                    sharding_ddl_util::getCriticalSectionReasonForRename(fromNss, toNss);
 
                 try {
                     // Make sure the source collection exists
@@ -195,6 +210,28 @@ ExecutorFuture<void> RenameCollectionCoordinator::_runImpl(
                         sharding_ddl_util::checkDbPrimariesOnTheSameShard(opCtx, fromNss, toNss);
                     }
 
+                    const auto optTargetCollType = getShardedCollection(opCtx, toNss);
+                    const bool targetIsSharded = (bool)optTargetCollType;
+                    _doc.setTargetIsSharded(targetIsSharded);
+                    _doc.setTargetUUID(getCollectionUUID(
+                        opCtx, toNss, optTargetCollType, /*throwNotFound*/ false));
+
+                    auto criticalSection = RecoverableCriticalSectionService::get(opCtx);
+                    if (!targetIsSharded) {
+                        // (SERVER-67325) Acquire critical section on the target collection in order
+                        // to disallow concurrent `createCollection`
+                        criticalSection->acquireRecoverableCriticalSectionBlockWrites(
+                            opCtx,
+                            toNss,
+                            criticalSectionReason,
+                            ShardingCatalogClient::kLocalWriteConcern);
+                        criticalSection->promoteRecoverableCriticalSectionToBlockAlsoReads(
+                            opCtx,
+                            toNss,
+                            criticalSectionReason,
+                            ShardingCatalogClient::kLocalWriteConcern);
+                    }
+
                     // Make sure the target namespace is not a view
                     {
                         Lock::DBLock dbLock(opCtx, toNss.db(), MODE_IS);
@@ -207,15 +244,39 @@ ExecutorFuture<void> RenameCollectionCoordinator::_runImpl(
                         }
                     }
 
-                    const auto optTargetCollType = getShardedCollection(opCtx, toNss);
-                    _doc.setTargetIsSharded((bool)optTargetCollType);
-                    _doc.setTargetUUID(getCollectionUUID(
-                        opCtx, toNss, optTargetCollType, /*throwNotFound*/ false));
+                    const bool targetExists = [&]() {
+                        if (targetIsSharded) {
+                            return true;
+                        }
+                        auto collectionCatalog = CollectionCatalog::get(opCtx);
+                        auto targetColl =
+                            collectionCatalog->lookupCollectionByNamespace(opCtx, toNss);
+                        return (bool)targetColl;  // true if exists and is unsharded
+                    }();
+
+                    if (targetExists) {
+                        // Release the critical section because the target collection
+                        // already exists, hence no risk of concurrent `createCollection`
+                        criticalSection->releaseRecoverableCriticalSection(
+                            opCtx,
+                            toNss,
+                            criticalSectionReason,
+                            ShardingCatalogClient::kLocalWriteConcern);
+                    }
 
                     sharding_ddl_util::checkRenamePreconditions(
                         opCtx, sourceIsSharded, toNss, _doc.getDropTarget());
 
+                    sharding_ddl_util::checkCatalogConsistencyAcrossShardsForRename(
+                        opCtx, fromNss, toNss, _doc.getDropTarget(), executor);
+
                 } catch (const DBException&) {
+                    auto criticalSection = RecoverableCriticalSectionService::get(opCtx);
+                    criticalSection->releaseRecoverableCriticalSection(
+                        opCtx,
+                        toNss,
+                        criticalSectionReason,
+                        ShardingCatalogClient::kLocalWriteConcern);
                     _completeOnError = true;
                     throw;
                 }
@@ -228,7 +289,7 @@ ExecutorFuture<void> RenameCollectionCoordinator::_runImpl(
                 getForwardableOpMetadata().setOn(opCtx);
 
                 const auto& fromNss = nss();
-                const auto& toNss = _doc.getTo();
+                const auto& toNss = _request.getTo();
 
                 ShardingLogging::get(opCtx)->logChange(
                     opCtx,
@@ -271,8 +332,7 @@ ExecutorFuture<void> RenameCollectionCoordinator::_runImpl(
                     fromNss, _doc.getSourceUUID().get());
                 renameCollParticipantRequest.setDbName(fromNss.db());
                 renameCollParticipantRequest.setTargetUUID(_doc.getTargetUUID());
-                renameCollParticipantRequest.setRenameCollectionRequest(
-                    _doc.getRenameCollectionRequest());
+                renameCollParticipantRequest.setRenameCollectionRequest(_request);
 
                 auto participants = Grid::get(opCtx)->shardRegistry()->getAllShardIds(opCtx);
                 // We need to send the command to all the shards because both
@@ -307,7 +367,7 @@ ExecutorFuture<void> RenameCollectionCoordinator::_runImpl(
                     _performNoopRetryableWriteOnParticipants(opCtx, **executor);
                 }
 
-                ConfigsvrRenameCollectionMetadata req(nss(), _doc.getTo());
+                ConfigsvrRenameCollectionMetadata req(nss(), _request.getTo());
                 req.setOptFromCollection(_doc.getOptShardedCollInfo());
                 const auto cmdObj = CommandHelpers::appendMajorityWriteConcern(req.toBSON({}));
                 const auto& configShard = Grid::get(opCtx)->shardRegistry()->getConfigShard();
@@ -352,8 +412,7 @@ ExecutorFuture<void> RenameCollectionCoordinator::_runImpl(
                 ShardsvrRenameCollectionUnblockParticipant unblockParticipantRequest(
                     fromNss, _doc.getSourceUUID().get());
                 unblockParticipantRequest.setDbName(fromNss.db());
-                unblockParticipantRequest.setRenameCollectionRequest(
-                    _doc.getRenameCollectionRequest());
+                unblockParticipantRequest.setRenameCollectionRequest(_request);
                 auto const cmdObj = CommandHelpers::appendMajorityWriteConcern(
                     unblockParticipantRequest.toBSON({}));
                 auto participants = Grid::get(opCtx)->shardRegistry()->getAllShardIds(opCtx);
@@ -375,28 +434,29 @@ ExecutorFuture<void> RenameCollectionCoordinator::_runImpl(
                         opCtx, fromNss.db(), cmdObj, participants, **executor);
                 }
             }))
-        .then(_executePhase(
-            Phase::kSetResponse,
-            [this, anchor = shared_from_this()] {
-                auto opCtxHolder = cc().makeOperationContext();
-                auto* opCtx = opCtxHolder.get();
-                getForwardableOpMetadata().setOn(opCtx);
+        .then(_executePhase(Phase::kSetResponse,
+                            [this, anchor = shared_from_this()] {
+                                auto opCtxHolder = cc().makeOperationContext();
+                                auto* opCtx = opCtxHolder.get();
+                                getForwardableOpMetadata().setOn(opCtx);
 
-                // Retrieve the new collection version
-                const auto catalog = Grid::get(opCtx)->catalogCache();
-                const auto cm = uassertStatusOK(
-                    catalog->getCollectionRoutingInfoWithRefresh(opCtx, _doc.getTo()));
-                _response = RenameCollectionResponse(cm.isSharded() ? cm.getVersion()
-                                                                    : ChunkVersion::UNSHARDED());
+                                // Retrieve the new collection version
+                                const auto catalog = Grid::get(opCtx)->catalogCache();
+                                const auto cm =
+                                    uassertStatusOK(catalog->getCollectionRoutingInfoWithRefresh(
+                                        opCtx, _request.getTo()));
+                                _response = RenameCollectionResponse(
+                                    cm.isSharded() ? cm.getVersion() : ChunkVersion::UNSHARDED());
 
-                ShardingLogging::get(opCtx)->logChange(
-                    opCtx,
-                    "renameCollection.end",
-                    nss().ns(),
-                    BSON("source" << nss().toString() << "destination" << _doc.getTo().toString()),
-                    ShardingCatalogClient::kMajorityWriteConcern);
-                LOGV2(5460504, "Collection renamed", "namespace"_attr = nss());
-            }))
+                                ShardingLogging::get(opCtx)->logChange(
+                                    opCtx,
+                                    "renameCollection.end",
+                                    nss().ns(),
+                                    BSON("source" << nss().toString() << "destination"
+                                                  << _request.getTo().toString()),
+                                    ShardingCatalogClient::kMajorityWriteConcern);
+                                LOGV2(5460504, "Collection renamed", "namespace"_attr = nss());
+                            }))
         .onError([this, anchor = shared_from_this()](const Status& status) {
             if (!status.isA<ErrorCategory::NotPrimaryError>() &&
                 !status.isA<ErrorCategory::ShutdownError>()) {
