@@ -56,6 +56,7 @@
 #include "mongo/db/s/resharding/resharding_server_parameters_gen.h"
 #include "mongo/db/s/resharding_util.h"
 #include "mongo/db/s/sharding_state.h"
+#include "mongo/db/write_concern_options.h"
 #include "mongo/logv2/log.h"
 #include "mongo/s/catalog/sharding_catalog_client.h"
 #include "mongo/s/grid.h"
@@ -63,6 +64,7 @@
 
 namespace mongo {
 
+MONGO_FAIL_POINT_DEFINE(reshardingPauseDonorBeforeCatalogCacheRefresh);
 MONGO_FAIL_POINT_DEFINE(reshardingDonorFailsAfterTransitionToDonatingOplogEntries);
 MONGO_FAIL_POINT_DEFINE(removeDonorDocFailpoint);
 
@@ -71,6 +73,8 @@ using namespace fmt::literals;
 namespace {
 
 const WriteConcernOptions kNoWaitWriteConcern{1, WriteConcernOptions::SyncMode::UNSET, Seconds(0)};
+const WriteConcernOptions kMajorityWriteConcern{
+    WriteConcernOptions::kMajority, WriteConcernOptions::SyncMode::UNSET, Seconds(0)};
 
 Date_t getCurrentTime() {
     const auto svcCtx = cc().getServiceContext();
@@ -159,7 +163,7 @@ public:
             query,
             update,
             false, /* upsert */
-            ShardingCatalogClient::kMajorityWriteConcern,
+            kMajorityWriteConcern,
             Milliseconds::max()));
 
         if (!docWasModified) {
@@ -203,6 +207,8 @@ ReshardingDonorService::DonorStateMachine::DonorStateMachine(
       _metadata{donorDoc.getCommonReshardingMetadata()},
       _recipientShardIds{donorDoc.getRecipientShards()},
       _donorCtx{donorDoc.getMutableState()},
+      _donorMetricsToRestore{donorDoc.getMetrics() ? donorDoc.getMetrics().get()
+                                                   : ReshardingDonorMetrics()},
       _externalState{std::move(externalState)},
       _markKilledExecutor(std::make_shared<ThreadPool>([] {
           ThreadPool::Options options;
@@ -397,12 +403,20 @@ ExecutorFuture<void> ReshardingDonorService::DonorStateMachine::_finishReshardin
 Status ReshardingDonorService::DonorStateMachine::_runMandatoryCleanup(
     Status status, const CancellationToken& stepdownToken) {
     if (!status.isOK()) {
+        // If the stepdownToken was triggered, it takes priority in order to make sure that
+        // the promise is set with an error that can be retried with. If it ran into an
+        // unrecoverable error, it would have fasserted earlier.
+        auto statusForPromise = stepdownToken.isCanceled()
+            ? Status{ErrorCodes::InterruptedDueToReplStateChange,
+                     "Resharding operation donor state machine interrupted due to replica set "
+                     "stepdown"}
+            : status;
+
         stdx::lock_guard<Latch> lk(_mutex);
 
-        ensureFulfilledPromise(lk, _critSecWasAcquired, status);
-        ensureFulfilledPromise(lk, _critSecWasPromoted, status);
-
-        ensureFulfilledPromise(lk, _completionPromise, status);
+        ensureFulfilledPromise(lk, _critSecWasAcquired, statusForPromise);
+        ensureFulfilledPromise(lk, _critSecWasPromoted, statusForPromise);
+        ensureFulfilledPromise(lk, _completionPromise, statusForPromise);
     }
 
     if (stepdownToken.isCanceled()) {
@@ -553,6 +567,8 @@ void ReshardingDonorService::DonorStateMachine::
     // with a SnapshotUnavailable error response.
     {
         auto opCtx = _cancelableOpCtxFactory->makeOperationContext(&cc());
+        reshardingPauseDonorBeforeCatalogCacheRefresh.pauseWhileSet(opCtx.get());
+
         _externalState->refreshCatalogCache(opCtx.get(), _metadata.getTempReshardingNss());
         _externalState->waitForCollectionFlush(opCtx.get(), _metadata.getTempReshardingNss());
     }
@@ -1005,8 +1021,9 @@ ReshardingMetrics* ReshardingDonorService::DonorStateMachine::_metrics() const {
 }
 
 void ReshardingDonorService::DonorStateMachine::_startMetrics() {
-    if (_donorCtx.getState() > DonorStateEnum::kPreparingToDonate) {
-        _metrics()->onStepUp(ReshardingMetrics::Role::kDonor);
+    auto donorState = _donorCtx.getState();
+    if (donorState > DonorStateEnum::kPreparingToDonate) {
+        _metrics()->onStepUp(donorState, _donorMetricsToRestore);
     } else {
         _metrics()->onStart(ReshardingMetrics::Role::kDonor, getCurrentTime());
     }
