@@ -91,7 +91,7 @@ boost::optional<repl::OplogEntry> forgeNoopEntryFromImageCollection(
     repl::OpTimeBase opTimeBase(Timestamp::min());
     opTimeBase.setTerm(-1);
     forgedNoop.setOpTimeBase(opTimeBase);
-    forgedNoop.setStatementId(0);
+    forgedNoop.setStatementIds({0});
     forgedNoop.setPrevWriteOpTimeInTransaction(repl::OpTime(Timestamp::min(), -1));
     return repl::OplogEntry::parse(forgedNoop.toBSON()).getValue();
 }
@@ -104,7 +104,8 @@ boost::optional<repl::OplogEntry> fetchPrePostImageOplog(OperationContext* opCtx
             // No pre/post image was found. Defensively strip the `needsRetryImage` value to remove
             // any notion this operation was a retryable findAndModify. If the request is retried on
             // the destination, it will surface an error to the user.
-            auto mutableOplog = fassert(5676405, repl::MutableOplogEntry::parse(oplog->toBSON()));
+            auto mutableOplog =
+                fassert(5676405, repl::MutableOplogEntry::parse(oplog->getEntry().toBSON()));
             mutableOplog.setNeedsRetryImage(boost::none);
             *oplog = repl::OplogEntry(mutableOplog.toBSON());
         }
@@ -122,10 +123,8 @@ boost::optional<repl::OplogEntry> fetchPrePostImageOplog(OperationContext* opCtx
 
     auto opTime = opTimeToFetch.value();
     DBDirectClient client(opCtx);
-    auto oplogBSON = client.findOne(NamespaceString::kRsOplogNamespace.ns(),
-                                    opTime.asQuery(),
-                                    nullptr,
-                                    QueryOption_OplogReplay);
+    auto oplogBSON =
+        client.findOne(NamespaceString::kRsOplogNamespace.ns(), opTime.asQuery(), nullptr);
 
     return uassertStatusOK(repl::OplogEntry::parse(oplogBSON));
 }
@@ -140,23 +139,27 @@ repl::OplogEntry makeOplogEntry(repl::OpTime opTime,
                                 const boost::optional<BSONObj>& o2Field,
                                 const OperationSessionInfo& sessionInfo,
                                 Date_t wallClockTime,
-                                const boost::optional<StmtId>& statementId) {
-    return repl::OplogEntry(opTime,                           // optime
-                            hash,                             // hash
-                            opType,                           // op type
-                            {},                               // namespace
-                            boost::none,                      // uuid
-                            boost::none,                      // fromMigrate
-                            repl::OplogEntry::kOplogVersion,  // version
-                            oField,                           // o
-                            o2Field,                          // o2
-                            sessionInfo,                      // session info
-                            boost::none,                      // upsert
-                            wallClockTime,                    // wall clock time
-                            statementId,                      // statement id
-                            boost::none,   // optime of previous write within same transaction
-                            boost::none,   // pre-image optime
-                            boost::none);  // post-image optime
+                                const std::vector<StmtId>& statementIds) {
+    return {
+        repl::DurableOplogEntry(opTime,                           // optime
+                                hash,                             // hash
+                                opType,                           // op type
+                                {},                               // namespace
+                                boost::none,                      // uuid
+                                boost::none,                      // fromMigrate
+                                repl::OplogEntry::kOplogVersion,  // version
+                                oField,                           // o
+                                o2Field,                          // o2
+                                sessionInfo,                      // session info
+                                boost::none,                      // upsert
+                                wallClockTime,                    // wall clock time
+                                statementIds,                     // statement ids
+                                boost::none,    // optime of previous write within same transaction
+                                boost::none,    // pre-image optime
+                                boost::none,    // post-image optime
+                                boost::none,    // ShardId of resharding recipient
+                                boost::none,    // _id
+                                boost::none)};  // needsRetryImage
 }
 
 /**
@@ -176,7 +179,7 @@ repl::OplogEntry makeSentinelOplogEntry(const LogicalSessionId& lsid,
                           TransactionParticipant::kDeadEndSentinel,  // o2
                           sessionInfo,                               // session info
                           wallClockTime,                             // wall clock time
-                          kIncompleteHistoryStmtId);                 // statement id
+                          {kIncompleteHistoryStmtId});               // statement id
 }
 
 }  // namespace
@@ -189,7 +192,6 @@ SessionCatalogMigrationSource::SessionCatalogMigrationSource(OperationContext* o
       _rollbackIdAtInit(repl::ReplicationProcess::get(opCtx)->getRollbackID()),
       _chunkRange(std::move(chunk)),
       _keyPattern(shardKey) {
-    // Exclude entries for transaction.
     Query query;
     // Sort is not needed for correctness. This is just for making it easier to write deterministic
     // tests.
@@ -218,7 +220,15 @@ SessionCatalogMigrationSource::SessionCatalogMigrationSource(OperationContext* o
 
                 WriteUnitOfWork wuow(opCtx);
                 opCtx->getClient()->getServiceContext()->getOpObserver()->onInternalOpMessage(
-                    opCtx, _ns, {}, {}, message);
+                    opCtx,
+                    _ns,
+                    {},
+                    {},
+                    message,
+                    boost::none,
+                    boost::none,
+                    boost::none,
+                    boost::none);
                 wuow.commit();
             });
     }
@@ -228,6 +238,14 @@ SessionCatalogMigrationSource::SessionCatalogMigrationSource(OperationContext* o
     WriteConcernOptions majority(
         WriteConcernOptions::kMajority, WriteConcernOptions::SyncMode::UNSET, 0);
     uassertStatusOK(waitForWriteConcern(opCtx, opTimeToWait, majority, &result));
+
+    AutoGetCollection collection(
+        opCtx, NamespaceString::kSessionTransactionsTableNamespace, MODE_IS);
+    // Session docs contain at least LSID, TxnNumber, Timestamp, and some BSON overhead.
+    const int64_t defaultSessionDocSize =
+        sizeof(LogicalSessionId) + sizeof(TxnNumber) + sizeof(Timestamp) + 16;
+    _averageSessionDocSize =
+        collection ? collection->averageObjectSize(opCtx) : defaultSessionDocSize;
 }
 
 bool SessionCatalogMigrationSource::hasMoreOplog() {
@@ -239,6 +257,15 @@ bool SessionCatalogMigrationSource::hasMoreOplog() {
     return _hasNewWrites(lk);
 }
 
+bool SessionCatalogMigrationSource::inCatchupPhase() {
+    return !_hasMoreOplogFromSessionCatalog();
+}
+
+int64_t SessionCatalogMigrationSource::untransferredCatchUpDataSize() {
+    invariant(inCatchupPhase());
+    return _newWriteOpTimeList.size() * _averageSessionDocSize;
+}
+
 void SessionCatalogMigrationSource::onCommitCloneStarted() {
     stdx::lock_guard<Latch> _lk(_newOplogMutex);
 
@@ -313,21 +340,19 @@ std::shared_ptr<Notification<bool>> SessionCatalogMigrationSource::getNotificati
 bool SessionCatalogMigrationSource::_handleWriteHistory(WithLock, OperationContext* opCtx) {
     while (_currentOplogIterator) {
         if (auto nextOplog = _currentOplogIterator->getNext(opCtx)) {
-            auto nextStmtId = nextOplog->getStatementId();
+            auto nextStmtIds = nextOplog->getStatementIds();
 
             // Skip the rest of the chain for this session since the ns is unrelated with the
             // current one being migrated. It is ok to not check the rest of the chain because
             // retryable writes doesn't allow touching different namespaces.
-            if (!nextStmtId ||
-                (nextStmtId && *nextStmtId != kIncompleteHistoryStmtId &&
-                 nextOplog->getNss() != _ns)) {
+            if (nextStmtIds.empty() ||
+                (nextStmtIds.front() != kIncompleteHistoryStmtId && nextOplog->getNss() != _ns)) {
                 _currentOplogIterator.reset();
                 return false;
             }
 
             if (nextOplog->isCrudOpType()) {
-                auto shardKey =
-                    _keyPattern.extractShardKeyFromDoc(nextOplog->getObjectContainingDocumentKey());
+                auto shardKey = _keyPattern.extractShardKeyFromOplogEntry(*nextOplog);
                 if (!_chunkRange.containsKey(shardKey)) {
                     continue;
                 }
@@ -412,18 +437,14 @@ bool SessionCatalogMigrationSource::_fetchNextNewWriteOplog(OperationContext* op
     }
 
     DBDirectClient client(opCtx);
-    const auto& newWriteOplogDoc = client.findOne(NamespaceString::kRsOplogNamespace.ns(),
-                                                  nextOpTimeToFetch.asQuery(),
-                                                  nullptr,
-                                                  QueryOption_OplogReplay);
-
+    const auto& newWriteOplogDoc = client.findOne(
+        NamespaceString::kRsOplogNamespace.ns(), nextOpTimeToFetch.asQuery(), nullptr);
 
     uassert(40620,
             str::stream() << "Unable to fetch oplog entry with opTime: "
                           << nextOpTimeToFetch.toBSON(),
             !newWriteOplogDoc.isEmpty());
 
-
     auto newWriteOplogEntry = uassertStatusOK(repl::OplogEntry::parse(newWriteOplogDoc));
 
     // If this oplog entry corresponds to transaction prepare/commit, replace it with a sentinel
