@@ -555,36 +555,44 @@ CreateCollectionResponse shardCollection(OperationContext* opCtx,
         }
 
         // Fail if there are partially written chunks from a previous failed shardCollection.
-
-        if (feature_flags::gShardingFullDDLSupportTimestampedVersion.isEnabledAndIgnoreFCV()) {
-            if (serverGlobalParams.featureCompatibility.getVersion() ==
-                    FCVersion::kFullyDowngradedTo44 ||
-                serverGlobalParams.featureCompatibility.getVersion() == FCVersion::kVersion49) {
-                checkForExistingChunks(opCtx, nss, boost::none);
-            } else if (serverGlobalParams.featureCompatibility.getVersion() ==
-                       FCVersion::kVersion50) {
-                if (auto optUUID = sharding_ddl_util::getCollectionUUID(opCtx, nss))
-                    checkForExistingChunks(opCtx, nss, optUUID);
-            } else {
-                // In the intermediate state must check for leftovers from both formats
-                checkForExistingChunks(opCtx, nss, boost::none);
-                if (auto optUUID = sharding_ddl_util::getCollectionUUID(opCtx, nss))
-                    checkForExistingChunks(opCtx, nss, optUUID);
-            }
-        } else {
+        if (serverGlobalParams.featureCompatibility.getVersion() ==
+                FCVersion::kFullyDowngradedTo44 ||
+            serverGlobalParams.featureCompatibility.getVersion() == FCVersion::kVersion49) {
             checkForExistingChunks(opCtx, nss, boost::none);
+        } else if (serverGlobalParams.featureCompatibility.getVersion() == FCVersion::kVersion50) {
+            if (auto optUUID = sharding_ddl_util::getCollectionUUID(opCtx, nss))
+                checkForExistingChunks(opCtx, nss, optUUID);
+        } else {
+            // In the intermediate state must check for leftovers from both formats
+            checkForExistingChunks(opCtx, nss, boost::none);
+            if (auto optUUID = sharding_ddl_util::getCollectionUUID(opCtx, nss))
+                checkForExistingChunks(opCtx, nss, optUUID);
         }
 
+
         checkCollation(opCtx, request);
 
-        // Create the collection locally
-        shardkeyutil::validateShardKeyIndexExistsOrCreateIfPossible(
-            opCtx,
-            nss,
-            ShardKeyPattern(request.getKey()),
-            *request.getCollation(),
-            request.getUnique(),
-            shardkeyutil::ValidationBehaviorsShardCollection(opCtx));
+        if (request.getImplicitlyCreateIndex().value_or(true)) {
+            // Create the collection locally
+            shardkeyutil::validateShardKeyIndexExistsOrCreateIfPossible(
+                opCtx,
+                nss,
+                ShardKeyPattern(request.getKey()),
+                *request.getCollation(),
+                request.getUnique(),
+                request.getEnforceUniquenessCheck().value_or(true),
+                shardkeyutil::ValidationBehaviorsShardCollection(opCtx));
+        } else {
+            uassert(6373201,
+                    "Must have an index compatible with the proposed shard key",
+                    validShardKeyIndexExists(
+                        opCtx,
+                        nss,
+                        ShardKeyPattern(request.getKey()),
+                        *request.getCollation(),
+                        request.getUnique() && request.getEnforceUniquenessCheck().value_or(true),
+                        shardkeyutil::ValidationBehaviorsShardCollection(opCtx)));
+        }
 
         // Wait until the index is majority written, to prevent having the collection commited to
         // the config server, but the index creation rolled backed on stepdowns.
