@@ -433,6 +433,7 @@ void ShardServerCatalogCacheLoader::onStepDown() {
 void ShardServerCatalogCacheLoader::onStepUp() {
     stdx::lock_guard<Latch> lg(_mutex);
     invariant(_role != ReplicaSetRole::None);
+    _contexts.interrupt(ErrorCodes::InterruptedDueToReplStateChange);
     ++_term;
     _role = ReplicaSetRole::Primary;
 }
@@ -497,6 +498,14 @@ SemiFuture<CollectionAndChangedChunks> ShardServerCatalogCacheLoader::getChunksS
 }
 
 SemiFuture<DatabaseType> ShardServerCatalogCacheLoader::getDatabase(StringData dbName) {
+    // The admin and config database have fixed metadata that does not need to be refreshed.
+    if (dbName == NamespaceString::kAdminDb || dbName == NamespaceString::kConfigDb) {
+        return DatabaseType(dbName.toString(),
+                            ShardId::kConfigServerId,
+                            dbName == NamespaceString::kConfigDb,
+                            DatabaseVersion::makeFixed());
+    }
+
     const auto [isPrimary, term] = [&] {
         stdx::lock_guard<Latch> lock(_mutex);
         return std::make_tuple(_role == ReplicaSetRole::Primary, _term);
@@ -575,9 +584,24 @@ void ShardServerCatalogCacheLoader::waitForCollectionFlush(OperationContext* opC
             }
         }
 
-        // It is not safe to use taskList after this call, because it will unlock and lock the tasks
-        // mutex, so we just loop around.
-        taskList.waitForActiveTaskCompletion(lg);
+        // Wait for the active task to complete
+        {
+            const auto activeTaskNum = taskList.front().taskNum;
+
+            // Increase the use_count of the condition variable shared pointer, because the entire
+            // task list might get deleted during the unlocked interval
+            auto condVar = taskList._activeTaskCompletedCondVar;
+
+            // It is not safe to use taskList after this call, because it will unlock and lock the
+            // tasks mutex, so we just loop around.
+            // It is only correct to wait again on condVar if the taskNum has not changed, meaning
+            // that it must still be the same task list.
+            opCtx->waitForConditionOrInterrupt(*condVar, lg, [&]() {
+                const auto it = _collAndChunkTaskLists.find(nss);
+                return it == _collAndChunkTaskLists.end() || it->second.empty() ||
+                    it->second.front().taskNum != activeTaskNum;
+            });
+        }
     }
 }
 
@@ -628,9 +652,24 @@ void ShardServerCatalogCacheLoader::waitForDatabaseFlush(OperationContext* opCtx
             }
         }
 
-        // It is not safe to use taskList after this call, because it will unlock and lock the tasks
-        // mutex, so we just loop around.
-        taskList.waitForActiveTaskCompletion(lg);
+        // Wait for the active task to complete
+        {
+            const auto activeTaskNum = taskList.front().taskNum;
+
+            // Increase the use_count of the condition variable shared pointer, because the entire
+            // task list might get deleted during the unlocked interval
+            auto condVar = taskList._activeTaskCompletedCondVar;
+
+            // It is not safe to use taskList after this call, because it will unlock and lock the
+            // tasks mutex, so we just loop around.
+            // It is only correct to wait again on condVar if the taskNum has not changed, meaning
+            // that it must still be the same task list.
+            opCtx->waitForConditionOrInterrupt(*condVar, lg, [&]() {
+                const auto it = _dbTaskLists.find(dbName.toString());
+                return it == _dbTaskLists.end() || it->second.empty() ||
+                    it->second.front().taskNum != activeTaskNum;
+            });
+        }
     }
 }
 
@@ -639,7 +678,14 @@ StatusWith<CollectionAndChangedChunks> ShardServerCatalogCacheLoader::_runSecond
     const NamespaceString& nss,
     const ChunkVersion& catalogCacheSinceVersion) {
 
+    Timer t;
     forcePrimaryCollectionRefreshAndWaitForReplication(opCtx, nss);
+    LOGV2_FOR_CATALOG_REFRESH(5965800,
+                              2,
+                              "Cache loader on secondary successfully waited for primary refresh "
+                              "and replication of collection",
+                              "namespace"_attr = nss,
+                              "duration"_attr = Milliseconds(t.millis()));
 
     // Read the local metadata.
 
@@ -789,7 +835,15 @@ ShardServerCatalogCacheLoader::_schedulePrimaryGetChunksSince(
 
 StatusWith<DatabaseType> ShardServerCatalogCacheLoader::_runSecondaryGetDatabase(
     OperationContext* opCtx, StringData dbName) {
+    Timer t;
     forcePrimaryDatabaseRefreshAndWaitForReplication(opCtx, dbName);
+    LOGV2_FOR_CATALOG_REFRESH(5965801,
+                              2,
+                              "Cache loader on secondary successfully waited for primary refresh "
+                              "and replication of database",
+                              "db"_attr = dbName,
+                              "duration"_attr = Milliseconds(t.millis()));
+
     return getPersistedDbMetadata(opCtx, dbName);
 }
 
@@ -1441,22 +1495,6 @@ void ShardServerCatalogCacheLoader::DbTaskList::pop_front() {
     _activeTaskCompletedCondVar->notify_all();
 }
 
-void ShardServerCatalogCacheLoader::CollAndChunkTaskList::waitForActiveTaskCompletion(
-    stdx::unique_lock<Latch>& lg) {
-    // Increase the use_count of the condition variable shared pointer, because the entire task list
-    // might get deleted during the unlocked interval
-    auto condVar = _activeTaskCompletedCondVar;
-    condVar->wait(lg);
-}
-
-void ShardServerCatalogCacheLoader::DbTaskList::waitForActiveTaskCompletion(
-    stdx::unique_lock<Latch>& lg) {
-    // Increase the use_count of the condition variable shared pointer, because the entire task list
-    // might get deleted during the unlocked interval
-    auto condVar = _activeTaskCompletedCondVar;
-    condVar->wait(lg);
-}
-
 bool ShardServerCatalogCacheLoader::CollAndChunkTaskList::hasTasksFromThisTerm(
     long long term) const {
     invariant(!_tasks.empty());
