@@ -33,12 +33,13 @@
 
 #include "mongo/db/s/transaction_coordinator.h"
 
-#include "mongo/db/logical_clock.h"
+#include "mongo/db/repl/wait_for_majority_service.h"
 #include "mongo/db/s/transaction_coordinator_metrics_observer.h"
-#include "mongo/db/s/wait_for_majority_service.h"
 #include "mongo/db/server_options.h"
+#include "mongo/db/vector_clock_mutable.h"
 #include "mongo/logv2/log.h"
 #include "mongo/s/grid.h"
+#include "mongo/util/cancellation.h"
 #include "mongo/util/fail_point.h"
 
 namespace mongo {
@@ -58,7 +59,9 @@ ExecutorFuture<void> waitForMajorityWithHangFailpoint(ServiceContext* service,
                                                       repl::OpTime opTime) {
     auto executor = Grid::get(service)->getExecutorPool()->getFixedExecutor();
     auto waitForWC = [service, executor](repl::OpTime opTime) {
-        return WaitForMajorityService::get(service).waitUntilMajority(opTime).thenRunOn(executor);
+        return WaitForMajorityService::get(service)
+            .waitUntilMajority(opTime, CancellationToken::uncancelable())
+            .thenRunOn(executor);
     };
 
     if (auto sfp = failpoint.scoped(); MONGO_unlikely(sfp.isActive())) {
@@ -99,6 +102,7 @@ TransactionCoordinator::TransactionCoordinator(OperationContext* operationContex
           std::make_unique<TransactionCoordinatorMetricsObserver>()),
       _deadline(deadline) {
 
+    auto apiParams = APIParameters::get(operationContext);
     auto kickOffCommitPF = makePromiseFuture<void>();
     _kickOffCommitPromise = std::move(kickOffCommitPF.promise);
 
@@ -137,6 +141,10 @@ TransactionCoordinator::TransactionCoordinator(OperationContext* operationContex
     // Two-phase commit phases chain. Once this chain executes, the 2PC sequence has completed
     // either with success or error and the scheduled deadline task above has been joined.
     std::move(kickOffCommitPF.future)
+        .then([this] {
+            return VectorClockMutable::get(_serviceContext)->waitForDurableTopologyTime();
+        })
+        .thenRunOn(Grid::get(_serviceContext)->getExecutorPool()->getFixedExecutor())
         .then([this] {
             // Persist the participants, unless they have been made durable already (which would
             // only be the case if this coordinator was created as part of step-up recovery).
@@ -169,7 +177,7 @@ TransactionCoordinator::TransactionCoordinator(OperationContext* operationContex
                 std::move(opTime));
         })
         .thenRunOn(Grid::get(_serviceContext)->getExecutorPool()->getFixedExecutor())
-        .then([this] {
+        .then([this, apiParams] {
             {
                 stdx::lock_guard<Latch> lg(_mutex);
                 _participantsDurable = true;
@@ -196,8 +204,12 @@ TransactionCoordinator::TransactionCoordinator(OperationContext* operationContex
                     return Future<void>::makeReady();
             }
 
-            return txn::sendPrepare(
-                       _serviceContext, *_sendPrepareScheduler, _lsid, _txnNumber, *_participants)
+            return txn::sendPrepare(_serviceContext,
+                                    *_sendPrepareScheduler,
+                                    _lsid,
+                                    _txnNumber,
+                                    apiParams,
+                                    *_participants)
                 .then([this](PrepareVoteConsensus consensus) mutable {
                     {
                         stdx::lock_guard<Latch> lg(_mutex);
@@ -214,9 +226,8 @@ TransactionCoordinator::TransactionCoordinator(OperationContext* operationContex
                                     "txnNumber"_attr = _txnNumber,
                                     "commitTimestamp"_attr = *_decision->getCommitTimestamp());
 
-                        uassertStatusOK(LogicalClock::get(_serviceContext)
-                                            ->advanceClusterTime(
-                                                LogicalTime(*_decision->getCommitTimestamp())));
+                        VectorClockMutable::get(_serviceContext)
+                            ->tickClusterTimeTo(LogicalTime(*_decision->getCommitTimestamp()));
                     }
                 });
         })
@@ -258,12 +269,25 @@ TransactionCoordinator::TransactionCoordinator(OperationContext* operationContex
             return txn::persistDecision(*_scheduler, _lsid, _txnNumber, *_participants, *_decision);
         })
         .then([this](repl::OpTime opTime) {
+            switch (_decision->getDecision()) {
+                case CommitDecision::kCommit: {
+                    _decisionPromise.emplaceValue(CommitDecision::kCommit);
+                    break;
+                }
+                case CommitDecision::kAbort: {
+                    _decisionPromise.setError(*_decision->getAbortStatus());
+                    break;
+                }
+                default:
+                    MONGO_UNREACHABLE;
+            };
+
             return waitForMajorityWithHangFailpoint(_serviceContext,
                                                     hangBeforeWaitingForDecisionWriteConcern,
                                                     "hangBeforeWaitingForDecisionWriteConcern",
                                                     std::move(opTime));
         })
-        .then([this] {
+        .then([this, apiParams] {
             {
                 stdx::lock_guard<Latch> lg(_mutex);
                 _decisionDurable = true;
@@ -286,20 +310,17 @@ TransactionCoordinator::TransactionCoordinator(OperationContext* operationContex
 
             switch (_decision->getDecision()) {
                 case CommitDecision::kCommit: {
-                    _decisionPromise.emplaceValue(CommitDecision::kCommit);
-
                     return txn::sendCommit(_serviceContext,
                                            *_scheduler,
                                            _lsid,
                                            _txnNumber,
+                                           apiParams,
                                            *_participants,
                                            *_decision->getCommitTimestamp());
                 }
                 case CommitDecision::kAbort: {
-                    _decisionPromise.setError(*_decision->getAbortStatus());
-
                     return txn::sendAbort(
-                        _serviceContext, *_scheduler, _lsid, _txnNumber, *_participants);
+                        _serviceContext, *_scheduler, _lsid, _txnNumber, apiParams, *_participants);
                 }
                 default:
                     MONGO_UNREACHABLE;
@@ -428,10 +449,17 @@ void TransactionCoordinator::_done(Status status) {
     }
 
     ul.unlock();
-    if (!_decisionDurable) {
+
+    if (!_decisionPromise.getFuture().isReady()) {
         _decisionPromise.setError(status);
     }
-    _completionPromise.setFrom(_decisionPromise.getFuture().getNoThrow());
+
+    if (!status.isOK()) {
+        _completionPromise.setError(status);
+    } else {
+        // If the status is OK, the decisionPromise must be set.
+        _completionPromise.setFrom(_decisionPromise.getFuture().getNoThrow());
+    }
 }
 
 void TransactionCoordinator::_logSlowTwoPhaseCommit(
