@@ -146,18 +146,23 @@ IndexDataEntry::IndexDataEntry(const std::string& indexDataEntry, KeyFormat keyF
 
 std::string IndexDataEntry::create(RecordId loc, const KeyString::TypeBits& typeBits) {
     // [RecordId size, RecordId, TypeBits size, TypeBits]
-    StackBufBuilder builder;
+    std::string entry;
+    auto writeSizeT = [&entry](size_t val) {
+        auto data = reinterpret_cast<const char*>(&val);
+        entry.append(data, data + sizeof(val));
+    };
+
     if (loc.isLong()) {
-        builder.appendNum(sizeof(int64_t));
-        builder.appendNum(loc.getLong());
+        writeSizeT(sizeof(int64_t));
+        writeSizeT(loc.getLong());
     } else {
         auto str = loc.getStr();
-        builder.appendNum(str.size());
-        builder.appendBuf(str.rawData(), str.size());
+        writeSizeT(str.size());
+        entry.append(str.rawData(), str.size());
     }
-    builder.appendNum(typeBits.getSize());
-    builder.appendBuf(typeBits.getBuffer(), typeBits.getSize());
-    return std::string(builder.buf(), builder.len());
+    writeSizeT(typeBits.getSize());
+    entry.append(typeBits.getBuffer(), typeBits.getSize());
+    return entry;
 }
 
 const uint8_t* IndexDataEntry::buffer() const {
@@ -1354,9 +1359,9 @@ SortedDataInterfaceUnique::SortedDataInterfaceUnique(const Ordering& ordering, S
     _KSForIdentEnd = createRadixKeyWithoutLocFromObj(BSONObj(), _identEnd, _ordering);
 }
 
-Status SortedDataInterfaceUnique::insert(OperationContext* opCtx,
-                                         const KeyString::Value& keyString,
-                                         bool dupsAllowed) {
+StatusWith<bool> SortedDataInterfaceUnique::insert(OperationContext* opCtx,
+                                                   const KeyString::Value& keyString,
+                                                   bool dupsAllowed) {
     StringStore* workingCopy(RecoveryUnit::get(opCtx)->getHead());
     RecordId loc = decodeRecordId(keyString, rsKeyFormat());
 
@@ -1373,7 +1378,7 @@ Status SortedDataInterfaceUnique::insert(OperationContext* opCtx,
         auto added = data.add(loc, keyString.getTypeBits());
         if (!added) {
             // Already indexed
-            return Status::OK();
+            return false;
         }
 
         workingCopy->update({std::move(key), *added});
@@ -1382,7 +1387,7 @@ Status SortedDataInterfaceUnique::insert(OperationContext* opCtx,
         workingCopy->insert({std::move(key), *data.add(loc, keyString.getTypeBits())});
     }
     RecoveryUnit::get(opCtx)->makeDirty();
-    return Status::OK();
+    return true;
 }
 
 void SortedDataInterfaceUnique::unindex(OperationContext* opCtx,
@@ -1552,9 +1557,9 @@ SortedDataInterfaceStandard::SortedDataInterfaceStandard(const Ordering& orderin
     _KSForIdentEnd = createMinRadixKeyFromObj(BSONObj(), _identEnd, _ordering);
 }
 
-Status SortedDataInterfaceStandard::insert(OperationContext* opCtx,
-                                           const KeyString::Value& keyString,
-                                           bool dupsAllowed) {
+StatusWith<bool> SortedDataInterfaceStandard::insert(OperationContext* opCtx,
+                                                     const KeyString::Value& keyString,
+                                                     bool dupsAllowed) {
     StringStore* workingCopy(RecoveryUnit::get(opCtx)->getHead());
     RecordId loc = decodeRecordId(keyString, rsKeyFormat());
 
@@ -1564,7 +1569,7 @@ Status SortedDataInterfaceStandard::insert(OperationContext* opCtx,
             .second;
     if (inserted)
         RecoveryUnit::get(opCtx)->makeDirty();
-    return Status::OK();
+    return inserted;
 }
 
 void SortedDataInterfaceStandard::unindex(OperationContext* opCtx,
