@@ -81,7 +81,9 @@ public:
     public:
         WriteBatch() = delete;
 
-        WriteBatch(Bucket* bucket, OperationId opId, const std::shared_ptr<ExecutionStats>& stats);
+        WriteBatch(const OID& bucketId,
+                   OperationId opId,
+                   const std::shared_ptr<ExecutionStats>& stats);
 
         /**
          * Attempt to claim the right to commit (or abort) a batch. If it returns true, rights are
@@ -96,7 +98,7 @@ public:
          */
         StatusWith<CommitInfo> getResult() const;
 
-        Bucket* bucket() const;
+        const OID& bucketId() const;
 
         const std::vector<BSONObj>& measurements() const;
         const BSONObj& min() const;
@@ -132,7 +134,7 @@ public:
          * that have previously been committed to the bucket, and marks the batch inactive. Must
          * have commit rights.
          */
-        void _prepareCommit();
+        void _prepareCommit(Bucket* bucket);
 
         /**
          * Report the result and status of a commit, and notify anyone waiting on getResult(). Must
@@ -142,13 +144,12 @@ public:
 
         /**
          * Abandon the write batch and notify any waiters that the bucket has been cleared. Must
-         * have commit rights. Parameter controls whether the function is allowed to access the
-         * bucket.
+         * have commit rights. Parameter 'bucket' provides a pointer to the bucket if still
+         * available, nullptr otherwise.
          */
-        void _abort(const boost::optional<Status>& status, bool canAccessBucket);
+        void _abort(const boost::optional<Status>& status, const Bucket* bucket);
 
-
-        Bucket* _bucket;
+        const OID _bucketId;
         OperationId _opId;
         std::shared_ptr<ExecutionStats> _stats;
 
@@ -180,7 +181,7 @@ public:
      * Returns an empty document if the given bucket cannot be found or if this time-series
      * collection was not created with a metadata field name.
      */
-    BSONObj getMetadata(Bucket* bucket) const;
+    BSONObj getMetadata(const OID& bucketId) const;
 
     /**
      * Returns the WriteBatch into which the document was inserted. Any caller who receives the same
@@ -336,6 +337,8 @@ public:
         friend class BucketAccess;
         friend class BucketCatalog;
 
+        Bucket(const OID& id);
+
         /**
          * Returns the ID for the underlying bucket.
          */
@@ -346,6 +349,11 @@ public:
          */
         bool allCommitted() const;
 
+        /**
+         * Retrieve the time associated with the bucket
+         */
+        Date_t getTime() const;
+
     private:
         /**
          * Determines the effect of adding 'doc' to this bucket. If adding 'doc' causes this bucket
@@ -370,10 +378,11 @@ public:
                                                  const std::shared_ptr<ExecutionStats>& stats);
 
         // Access to the bucket is controlled by this lock
-        mutable Mutex _mutex;
+        mutable Mutex _mutex =
+            MONGO_MAKE_LATCH(HierarchicalAcquisitionLevel(2), "BucketCatalog::Bucket::_mutex");
 
         // The bucket ID for the underlying document
-        OID _id = OID::gen();
+        const OID _id;
 
         // The namespace that this bucket is used for.
         NamespaceString _ns;
@@ -389,6 +398,9 @@ public:
         // Top-level field names of the measurements that have been inserted into the bucket.
         StringSet _fieldNames;
 
+        // Minimum timestamp over contained measurements
+        Date_t _minTime;
+
         // The minimum and maximum values for each field in the bucket.
         timeseries::MinMax _minmax;
 
@@ -551,7 +563,7 @@ private:
                      ExecutionStats* stats,
                      const Date_t& time);
         BucketAccess(BucketCatalog* catalog,
-                     Bucket* bucket,
+                     const OID& bucketId,
                      boost::optional<BucketState> targetState = boost::none);
         ~BucketAccess();
 
@@ -571,9 +583,6 @@ private:
          */
         void rollover(const std::function<bool(BucketAccess*)>& isBucketFull);
 
-        // Retrieve the time associated with the bucket (id)
-        Date_t getTime() const;
-
     private:
         /**
          * Returns the state of the bucket, or boost::none if there is no state for the bucket.
@@ -689,8 +698,6 @@ private:
     std::shared_ptr<ExecutionStats> _getExecutionStats(const NamespaceString& ns);
     const std::shared_ptr<ExecutionStats> _getExecutionStats(const NamespaceString& ns) const;
 
-    void _setIdTimestamp(Bucket* bucket, const Date_t& time, const TimeseriesOptions& options);
-
     /**
      * Changes the bucket state, taking into account the current state, the specified target state,
      * and allowed state transitions. The return value, if set, is the final state of the bucket
@@ -720,7 +727,7 @@ private:
     mutable StripedMutex _bucketMutex;
 
     // All buckets currently in the catalog, including buckets which are full but not yet committed.
-    stdx::unordered_set<std::unique_ptr<Bucket>> _allBuckets;
+    stdx::unordered_map<OID, std::unique_ptr<Bucket>, OID::Hasher> _allBuckets;
 
     // The current open bucket for each namespace and metadata pair.
     stdx::unordered_map<BucketKey, Bucket*, BucketHasher, BucketEq> _openBuckets;
