@@ -113,7 +113,7 @@ TEST_F(AddToSetNodeTest, ApplyFailsOnNonArray) {
     ASSERT_OK(node.init(update["$addToSet"]["a"], expCtx));
 
     mutablebson::Document doc(fromjson("{a: 2}"));
-    setPathTaken("a");
+    setPathTaken(makeRuntimeUpdatePathForTest("a"));
     ASSERT_THROWS_CODE_AND_WHAT(
         node.apply(getApplyParams(doc.root()["a"]), getUpdateNodeApplyParams()),
         AssertionException,
@@ -128,14 +128,16 @@ TEST_F(AddToSetNodeTest, ApplyNonEach) {
     ASSERT_OK(node.init(update["$addToSet"]["a"], expCtx));
 
     mutablebson::Document doc(fromjson("{a: [0]}"));
-    setPathTaken("a");
+    setPathTaken(makeRuntimeUpdatePathForTest("a"));
     addIndexedPath("a");
     auto result = node.apply(getApplyParams(doc.root()["a"]), getUpdateNodeApplyParams());
     ASSERT_FALSE(result.noop);
     ASSERT_TRUE(result.indexesAffected);
     ASSERT_EQUALS(fromjson("{a: [0, 1]}"), doc);
     ASSERT_FALSE(doc.isInPlaceModeEnabled());
-    ASSERT_EQUALS(fromjson("{$set: {a: [0, 1]}}"), getLogDoc());
+
+    assertOplogEntry(fromjson("{$set: {a: [0, 1]}}"), fromjson("{$v: 2, diff: {u: {a: [0, 1]}}}"));
+
     ASSERT_EQUALS(getModifiedPaths(), "{a}");
 }
 
@@ -146,14 +148,16 @@ TEST_F(AddToSetNodeTest, ApplyNonEachArray) {
     ASSERT_OK(node.init(update["$addToSet"]["a"], expCtx));
 
     mutablebson::Document doc(fromjson("{a: [0]}"));
-    setPathTaken("a");
+    setPathTaken(makeRuntimeUpdatePathForTest("a"));
     addIndexedPath("a");
     auto result = node.apply(getApplyParams(doc.root()["a"]), getUpdateNodeApplyParams());
     ASSERT_FALSE(result.noop);
     ASSERT_TRUE(result.indexesAffected);
     ASSERT_EQUALS(fromjson("{a: [0, [1]]}"), doc);
     ASSERT_FALSE(doc.isInPlaceModeEnabled());
-    ASSERT_EQUALS(fromjson("{$set: {a: [0, [1]]}}"), getLogDoc());
+
+    assertOplogEntry(fromjson("{$set: {a: [0, [1]]}}"),
+                     fromjson("{$v: 2, diff: {u: {a: [0, [1]]}}}"));
     ASSERT_EQUALS(getModifiedPaths(), "{a}");
 }
 
@@ -164,14 +168,16 @@ TEST_F(AddToSetNodeTest, ApplyEach) {
     ASSERT_OK(node.init(update["$addToSet"]["a"], expCtx));
 
     mutablebson::Document doc(fromjson("{a: [0]}"));
-    setPathTaken("a");
+    setPathTaken(makeRuntimeUpdatePathForTest("a"));
     addIndexedPath("a");
     auto result = node.apply(getApplyParams(doc.root()["a"]), getUpdateNodeApplyParams());
     ASSERT_FALSE(result.noop);
     ASSERT_TRUE(result.indexesAffected);
     ASSERT_EQUALS(fromjson("{a: [0, 1, 2]}"), doc);
     ASSERT_FALSE(doc.isInPlaceModeEnabled());
-    ASSERT_EQUALS(fromjson("{$set: {a: [0, 1, 2]}}"), getLogDoc());
+
+    assertOplogEntry(fromjson("{$set: {a: [0, 1, 2]}}"),
+                     fromjson("{$v: 2, diff: {u: {a: [0, 1, 2]}}}"));
     ASSERT_EQUALS(getModifiedPaths(), "{a}");
 }
 
@@ -182,14 +188,15 @@ TEST_F(AddToSetNodeTest, ApplyToEmptyArray) {
     ASSERT_OK(node.init(update["$addToSet"]["a"], expCtx));
 
     mutablebson::Document doc(fromjson("{a: []}"));
-    setPathTaken("a");
+    setPathTaken(makeRuntimeUpdatePathForTest("a"));
     addIndexedPath("a");
     auto result = node.apply(getApplyParams(doc.root()["a"]), getUpdateNodeApplyParams());
     ASSERT_FALSE(result.noop);
     ASSERT_TRUE(result.indexesAffected);
     ASSERT_EQUALS(fromjson("{a: [1, 2]}"), doc);
     ASSERT_FALSE(doc.isInPlaceModeEnabled());
-    ASSERT_EQUALS(fromjson("{$set: {a: [1, 2]}}"), getLogDoc());
+
+    assertOplogEntry(fromjson("{$set: {a: [1, 2]}}"), fromjson("{$v: 2, diff: {u: {a: [1, 2]}}}"));
     ASSERT_EQUALS(getModifiedPaths(), "{a}");
 }
 
@@ -200,14 +207,16 @@ TEST_F(AddToSetNodeTest, ApplyDeduplicateElementsToAdd) {
     ASSERT_OK(node.init(update["$addToSet"]["a"], expCtx));
 
     mutablebson::Document doc(fromjson("{a: [0]}"));
-    setPathTaken("a");
+    setPathTaken(makeRuntimeUpdatePathForTest("a"));
     addIndexedPath("a");
     auto result = node.apply(getApplyParams(doc.root()["a"]), getUpdateNodeApplyParams());
     ASSERT_FALSE(result.noop);
     ASSERT_TRUE(result.indexesAffected);
     ASSERT_EQUALS(fromjson("{a: [0, 1]}"), doc);
     ASSERT_FALSE(doc.isInPlaceModeEnabled());
-    ASSERT_EQUALS(fromjson("{$set: {a: [0, 1]}}"), getLogDoc());
+
+    assertOplogEntry(fromjson("{$set: {a: [0, 1]}}"), fromjson("{$v: 2, diff: {u: {a: [0, 1]}}}"));
+
     ASSERT_EQUALS(getModifiedPaths(), "{a}");
 }
 
@@ -218,14 +227,16 @@ TEST_F(AddToSetNodeTest, ApplyDoNotAddExistingElements) {
     ASSERT_OK(node.init(update["$addToSet"]["a"], expCtx));
 
     mutablebson::Document doc(fromjson("{a: [0]}"));
-    setPathTaken("a");
+    setPathTaken(makeRuntimeUpdatePathForTest("a"));
     addIndexedPath("a");
     auto result = node.apply(getApplyParams(doc.root()["a"]), getUpdateNodeApplyParams());
     ASSERT_FALSE(result.noop);
     ASSERT_TRUE(result.indexesAffected);
     ASSERT_EQUALS(fromjson("{a: [0, 1]}"), doc);
     ASSERT_FALSE(doc.isInPlaceModeEnabled());
-    ASSERT_EQUALS(fromjson("{$set: {a: [0, 1]}}"), getLogDoc());
+
+    assertOplogEntry(fromjson("{$set: {a: [0, 1]}}"), fromjson("{$v: 2, diff: {u: {a: [0, 1]}}}"));
+
     ASSERT_EQUALS(getModifiedPaths(), "{a}");
 }
 
@@ -236,14 +247,16 @@ TEST_F(AddToSetNodeTest, ApplyDoNotDeduplicateExistingElements) {
     ASSERT_OK(node.init(update["$addToSet"]["a"], expCtx));
 
     mutablebson::Document doc(fromjson("{a: [0, 0]}"));
-    setPathTaken("a");
+    setPathTaken(makeRuntimeUpdatePathForTest("a"));
     addIndexedPath("a");
     auto result = node.apply(getApplyParams(doc.root()["a"]), getUpdateNodeApplyParams());
     ASSERT_FALSE(result.noop);
     ASSERT_TRUE(result.indexesAffected);
     ASSERT_EQUALS(fromjson("{a: [0, 0, 1]}"), doc);
     ASSERT_FALSE(doc.isInPlaceModeEnabled());
-    ASSERT_EQUALS(fromjson("{$set: {a: [0, 0, 1]}}"), getLogDoc());
+
+    assertOplogEntry(fromjson("{$set: {a: [0, 0, 1]}}"),
+                     fromjson("{$v: 2, diff: {u: {a: [0, 0, 1]}}}"));
     ASSERT_EQUALS(getModifiedPaths(), "{a}");
 }
 
@@ -254,14 +267,15 @@ TEST_F(AddToSetNodeTest, ApplyNoElementsToAdd) {
     ASSERT_OK(node.init(update["$addToSet"]["a"], expCtx));
 
     mutablebson::Document doc(fromjson("{a: [0]}"));
-    setPathTaken("a");
+    setPathTaken(makeRuntimeUpdatePathForTest("a"));
     addIndexedPath("a");
     auto result = node.apply(getApplyParams(doc.root()["a"]), getUpdateNodeApplyParams());
     ASSERT_TRUE(result.noop);
     ASSERT_FALSE(result.indexesAffected);
     ASSERT_EQUALS(fromjson("{a: [0]}"), doc);
     ASSERT_TRUE(doc.isInPlaceModeEnabled());
-    ASSERT_EQUALS(fromjson("{}"), getLogDoc());
+
+    assertOplogEntryIsNoop();
     ASSERT_EQUALS(getModifiedPaths(), "{a}");
 }
 
@@ -272,14 +286,15 @@ TEST_F(AddToSetNodeTest, ApplyNoNonDuplicateElementsToAdd) {
     ASSERT_OK(node.init(update["$addToSet"]["a"], expCtx));
 
     mutablebson::Document doc(fromjson("{a: [0]}"));
-    setPathTaken("a");
+    setPathTaken(makeRuntimeUpdatePathForTest("a"));
     addIndexedPath("a");
     auto result = node.apply(getApplyParams(doc.root()["a"]), getUpdateNodeApplyParams());
     ASSERT_TRUE(result.noop);
     ASSERT_FALSE(result.indexesAffected);
     ASSERT_EQUALS(fromjson("{a: [0]}"), doc);
     ASSERT_TRUE(doc.isInPlaceModeEnabled());
-    ASSERT_EQUALS(fromjson("{}"), getLogDoc());
+
+    assertOplogEntryIsNoop();
     ASSERT_EQUALS(getModifiedPaths(), "{a}");
 }
 
@@ -297,7 +312,8 @@ TEST_F(AddToSetNodeTest, ApplyCreateArray) {
     ASSERT_TRUE(result.indexesAffected);
     ASSERT_EQUALS(fromjson("{a: [0, 1]}"), doc);
     ASSERT_FALSE(doc.isInPlaceModeEnabled());
-    ASSERT_EQUALS(fromjson("{$set: {a: [0, 1]}}"), getLogDoc());
+
+    assertOplogEntry(fromjson("{$set: {a: [0, 1]}}"), fromjson("{$v: 2, diff: {i: {a: [0, 1]}}}"));
     ASSERT_EQUALS(getModifiedPaths(), "{a}");
 }
 
@@ -315,7 +331,8 @@ TEST_F(AddToSetNodeTest, ApplyCreateEmptyArrayIsNotNoop) {
     ASSERT_TRUE(result.indexesAffected);
     ASSERT_EQUALS(fromjson("{a: []}"), doc);
     ASSERT_FALSE(doc.isInPlaceModeEnabled());
-    ASSERT_EQUALS(fromjson("{$set: {a: []}}"), getLogDoc());
+
+    assertOplogEntry(fromjson("{$set: {a: []}}"), fromjson("{$v: 2, diff: {i: {a: []}}}"));
     ASSERT_EQUALS(getModifiedPaths(), "{a}");
 }
 
@@ -329,14 +346,16 @@ TEST_F(AddToSetNodeTest, ApplyDeduplicationOfElementsToAddRespectsCollation) {
     ASSERT_OK(node.init(update["$addToSet"]["a"], expCtx));
 
     mutablebson::Document doc(fromjson("{a: []}"));
-    setPathTaken("a");
+    setPathTaken(makeRuntimeUpdatePathForTest("a"));
     addIndexedPath("a");
     auto result = node.apply(getApplyParams(doc.root()["a"]), getUpdateNodeApplyParams());
     ASSERT_FALSE(result.noop);
     ASSERT_TRUE(result.indexesAffected);
     ASSERT_EQUALS(fromjson("{a: ['abc', 'def']}"), doc);
     ASSERT_FALSE(doc.isInPlaceModeEnabled());
-    ASSERT_EQUALS(fromjson("{$set: {a: ['abc', 'def']}}"), getLogDoc());
+
+    assertOplogEntry(fromjson("{$set: {a: ['abc', 'def']}}"),
+                     fromjson("{$v: 2, diff: {u: {a: ['abc', 'def']}}}"));
     ASSERT_EQUALS(getModifiedPaths(), "{a}");
 }
 
@@ -350,14 +369,16 @@ TEST_F(AddToSetNodeTest, ApplyComparisonToExistingElementsRespectsCollation) {
     ASSERT_OK(node.init(update["$addToSet"]["a"], expCtx));
 
     mutablebson::Document doc(fromjson("{a: ['ABC']}"));
-    setPathTaken("a");
+    setPathTaken(makeRuntimeUpdatePathForTest("a"));
     addIndexedPath("a");
     auto result = node.apply(getApplyParams(doc.root()["a"]), getUpdateNodeApplyParams());
     ASSERT_FALSE(result.noop);
     ASSERT_TRUE(result.indexesAffected);
     ASSERT_EQUALS(fromjson("{a: ['ABC', 'def']}"), doc);
     ASSERT_FALSE(doc.isInPlaceModeEnabled());
-    ASSERT_EQUALS(fromjson("{$set: {a: ['ABC', 'def']}}"), getLogDoc());
+
+    assertOplogEntry(fromjson("{$set: {a: ['ABC', 'def']}}"),
+                     fromjson("{$v: 2, diff: {u: {a: ['ABC', 'def']}}}"));
     ASSERT_EQUALS(getModifiedPaths(), "{a}");
 }
 
@@ -372,14 +393,17 @@ TEST_F(AddToSetNodeTest, ApplyRespectsCollationFromSetCollator) {
     node.setCollator(&caseInsensitiveCollator);
 
     mutablebson::Document doc(fromjson("{a: []}"));
-    setPathTaken("a");
+    setPathTaken(makeRuntimeUpdatePathForTest("a"));
     addIndexedPath("a");
     auto result = node.apply(getApplyParams(doc.root()["a"]), getUpdateNodeApplyParams());
     ASSERT_FALSE(result.noop);
     ASSERT_TRUE(result.indexesAffected);
     ASSERT_EQUALS(fromjson("{a: ['abc', 'def']}"), doc);
     ASSERT_FALSE(doc.isInPlaceModeEnabled());
-    ASSERT_EQUALS(fromjson("{$set: {a: ['abc', 'def']}}"), getLogDoc());
+
+    assertOplogEntry(fromjson("{$set: {a: ['abc', 'def']}}"),
+                     fromjson("{$v: 2, diff: {u: {a: ['abc', 'def']}}}"));
+
     ASSERT_EQUALS(getModifiedPaths(), "{a}");
 }
 
@@ -414,15 +438,18 @@ TEST_F(AddToSetNodeTest, ApplyNestedArray) {
     AddToSetNode node;
     ASSERT_OK(node.init(update["$addToSet"]["a.1"], expCtx));
 
-    mutablebson::Document doc(fromjson("{ _id : 1, a : [ 1, [ ] ] }"));
-    setPathTaken("a.1");
+    mutablebson::Document doc(fromjson("{ _id : 1, a : [1, []] }"));
+    setPathTaken(makeRuntimeUpdatePathForTest("a.1"));
     addIndexedPath("a");
     auto result = node.apply(getApplyParams(doc.root()["a"][1]), getUpdateNodeApplyParams());
     ASSERT_FALSE(result.noop);
     ASSERT_TRUE(result.indexesAffected);
-    ASSERT_EQUALS(fromjson("{ _id : 1, a : [ 1, [ 1 ] ] }"), doc);
+    ASSERT_EQUALS(fromjson("{ _id : 1, a : [1, [1]] }"), doc);
     ASSERT_FALSE(doc.isInPlaceModeEnabled());
-    ASSERT_EQUALS(fromjson("{ $set : { 'a.1' : [ 1 ] } }"), getLogDoc());
+
+    assertOplogEntry(fromjson("{ $set : { 'a.1' : [1] } }"),
+                     fromjson("{$v: 2, diff: {sa: {a: true, u1: [1]}}}"));
+
     ASSERT_EQUALS(getModifiedPaths(), "{a.1}");
 }
 
@@ -433,13 +460,14 @@ TEST_F(AddToSetNodeTest, ApplyIndexesNotAffected) {
     ASSERT_OK(node.init(update["$addToSet"]["a"], expCtx));
 
     mutablebson::Document doc(fromjson("{a: [0]}"));
-    setPathTaken("a");
+    setPathTaken(makeRuntimeUpdatePathForTest("a"));
     addIndexedPath("b");
     auto result = node.apply(getApplyParams(doc.root()["a"]), getUpdateNodeApplyParams());
     ASSERT_FALSE(result.noop);
     ASSERT_FALSE(result.indexesAffected);
     ASSERT_FALSE(doc.isInPlaceModeEnabled());
-    ASSERT_EQUALS(fromjson("{$set: {a: [0, 1]}}"), getLogDoc());
+
+    assertOplogEntry(fromjson("{$set: {a: [0, 1]}}"), fromjson("{$v: 2, diff: {u: {a: [0, 1]}}}"));
     ASSERT_EQUALS(getModifiedPaths(), "{a}");
 }
 
@@ -450,7 +478,7 @@ TEST_F(AddToSetNodeTest, ApplyNoIndexDataOrLogBuilder) {
     ASSERT_OK(node.init(update["$addToSet"]["a"], expCtx));
 
     mutablebson::Document doc(fromjson("{a: [0]}"));
-    setPathTaken("a");
+    setPathTaken(makeRuntimeUpdatePathForTest("a"));
     setLogBuilderToNull();
     auto result = node.apply(getApplyParams(doc.root()["a"]), getUpdateNodeApplyParams());
     ASSERT_FALSE(result.noop);
