@@ -41,8 +41,8 @@ Status UnsetNode::init(BSONElement modExpr, const boost::intrusive_ptr<Expressio
     return Status::OK();
 }
 
-ModifierNode::ModifyResult UnsetNode::updateExistingElement(
-    mutablebson::Element* element, std::shared_ptr<FieldRef> elementPath) const {
+ModifierNode::ModifyResult UnsetNode::updateExistingElement(mutablebson::Element* element,
+                                                            const FieldRef& elementPath) const {
     auto parent = element->parent();
 
     invariant(parent.ok());
@@ -61,7 +61,9 @@ void UnsetNode::validateUpdate(mutablebson::ConstElement updatedElement,
                                mutablebson::ConstElement leftSibling,
                                mutablebson::ConstElement rightSibling,
                                std::uint32_t recursionLevel,
-                               ModifyResult modifyResult) const {
+                               ModifyResult modifyResult,
+                               bool validateForStorage,
+                               bool* containsDotsAndDollarsField) const {
     invariant(modifyResult == ModifyResult::kNormalUpdate);
 
     // We only need to check the left and right sibling to see if the removed element was part of a
@@ -70,21 +72,40 @@ void UnsetNode::validateUpdate(mutablebson::ConstElement updatedElement,
     const uint32_t recursionLevelForCheck = 0;
 
     if (leftSibling.ok()) {
-        storage_validation::storageValid(leftSibling, doRecursiveCheck, recursionLevelForCheck);
+        storage_validation::storageValid(leftSibling,
+                                         doRecursiveCheck,
+                                         recursionLevelForCheck,
+                                         false, /* allowTopLevelDollarPrefixedFields */
+                                         validateForStorage,
+                                         containsDotsAndDollarsField);
     }
 
     if (rightSibling.ok()) {
-        storage_validation::storageValid(rightSibling, doRecursiveCheck, recursionLevelForCheck);
+        storage_validation::storageValid(rightSibling,
+                                         doRecursiveCheck,
+                                         recursionLevelForCheck,
+                                         false, /* allowTopLevelDollarPrefixedFields */
+                                         validateForStorage,
+                                         containsDotsAndDollarsField);
     }
 }
 
-void UnsetNode::logUpdate(LogBuilder* logBuilder,
-                          StringData pathTaken,
+void UnsetNode::logUpdate(LogBuilderInterface* logBuilder,
+                          const RuntimeUpdatePath& pathTaken,
                           mutablebson::Element element,
-                          ModifyResult modifyResult) const {
+                          ModifyResult modifyResult,
+                          boost::optional<int> createdFieldIdx) const {
     invariant(logBuilder);
     invariant(modifyResult == ModifyResult::kNormalUpdate);
-    uassertStatusOK(logBuilder->addToUnsets(pathTaken));
+    invariant(!createdFieldIdx);
+
+    if (pathTaken.types().back() == RuntimeUpdatePath::ComponentType::kArrayIndex) {
+        // If $unset is applied to an array index, the value was set to null.
+        invariant(element.getType() == BSONType::jstNULL);
+        uassertStatusOK(logBuilder->logUpdatedField(pathTaken, element));
+    } else {
+        uassertStatusOK(logBuilder->logDeletedField(pathTaken));
+    }
 }
 
 }  // namespace mongo
