@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kDefault
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include "mongo/platform/basic.h"
 
@@ -39,29 +39,112 @@
 #include "mongo/base/status_with.h"
 #include "mongo/base/string_data.h"
 #include "mongo/bson/util/builder.h"
+#include "mongo/db/api_parameters.h"
+#include "mongo/db/audit.h"
 #include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/catalog/database.h"
-#include "mongo/db/commands/feature_compatibility_version_command_parser.h"
+#include "mongo/db/curop.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
-#include "mongo/db/pipeline/aggregation_request.h"
+#include "mongo/db/pipeline/aggregate_command_gen.h"
 #include "mongo/db/pipeline/document_source.h"
 #include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/lite_parsed_pipeline.h"
 #include "mongo/db/pipeline/pipeline.h"
-#include "mongo/db/pipeline/stub_mongo_process_interface.h"
+#include "mongo/db/pipeline/process_interface/stub_mongo_process_interface.h"
 #include "mongo/db/query/collation/collator_factory_interface.h"
 #include "mongo/db/storage/recovery_unit.h"
 #include "mongo/db/views/resolved_view.h"
 #include "mongo/db/views/view.h"
 #include "mongo/db/views/view_graph.h"
-#include "mongo/util/fail_point_service.h"
-#include "mongo/util/log.h"
+#include "mongo/logv2/log.h"
+#include "mongo/util/fail_point.h"
 
 namespace mongo {
 
 namespace {
-auto getViewCatalog = Database::declareDecoration<std::unique_ptr<ViewCatalog>>();
+/**
+ * Helper class to manage copy-on-write for the ViewCatalog.
+ */
+class ViewCatalogWriter {
+public:
+    ViewCatalogWriter(Mutex& mutex,
+                      std::shared_ptr<const ViewCatalog> instance,
+                      std::shared_ptr<ViewCatalog>* storage)
+        : _mutex(mutex), _read(std::move(instance)), _storage(storage) {}
+
+    ViewCatalogWriter(ViewCatalogWriter&&) = delete;
+    ViewCatalogWriter& operator=(ViewCatalogWriter&&) = delete;
+
+    const ViewCatalog* operator->() const {
+        if (_write)
+            return _write.get();
+
+        return _read.get();
+    }
+
+    ViewCatalog* writable() {
+        if (!_write) {
+            _lock = stdx::unique_lock<Mutex>(_mutex);
+            _write = std::make_shared<ViewCatalog>(*_read);
+        }
+        return _write.get();
+    }
+
+    void commit() {
+        if (_write) {
+            atomic_store(_storage, std::move(_write));
+            _lock.unlock();
+        }
+    }
+
+private:
+    Mutex& _mutex;
+    stdx::unique_lock<Mutex> _lock;
+    std::shared_ptr<const ViewCatalog> _read;
+    std::shared_ptr<ViewCatalog> _write;
+    std::shared_ptr<ViewCatalog>* _storage;
+};
+
+/**
+ * Decoration on the Database object for storing the latest ViewCatalog instance and its associated
+ * write mutex.
+ *
+ * We don't want a non-const Database to be required to do modifications on the ViewCatalog, so all
+ * members of this class are mutable.
+ */
+class ViewCatalogStorage {
+public:
+    std::shared_ptr<const ViewCatalog> get() const {
+        return atomic_load(&_catalog);
+    }
+
+    void set(std::shared_ptr<ViewCatalog> instance) const {
+        atomic_store(&_catalog, std::move(instance));
+    }
+
+    ViewCatalogWriter writer() const {
+        return ViewCatalogWriter(_mutex, get(), &_catalog);
+    }
+
+    void setIgnoreExternalChange(bool value) const {
+        _ignoreExternalChange = value;
+    }
+
+    bool shouldIgnoreExternalChange() const {
+        return _ignoreExternalChange;
+    }
+
+private:
+    mutable std::shared_ptr<ViewCatalog> _catalog;
+    mutable Mutex _mutex = MONGO_MAKE_LATCH("ViewCatalogStorage::mutex");  // Serializes writes
+
+    // This is safe to not be atomic because it is only accessed on the write path for the
+    // 'system.views' collection for this db. Modifications to this collection happens only through
+    // this class and we have it locked with MODE_X.
+    mutable bool _ignoreExternalChange = false;
+};
+auto getViewCatalog = Database::declareDecoration<ViewCatalogStorage>();
 
 StatusWith<std::unique_ptr<CollatorInterface>> parseCollator(OperationContext* opCtx,
                                                              BSONObj collationSpec) {
@@ -74,31 +157,41 @@ StatusWith<std::unique_ptr<CollatorInterface>> parseCollator(OperationContext* o
 }
 }  // namespace
 
-ViewCatalog* ViewCatalog::get(const Database* db) {
+std::shared_ptr<const ViewCatalog> ViewCatalog::get(const Database* db) {
     return getViewCatalog(db).get();
 }
 
 void ViewCatalog::set(Database* db, std::unique_ptr<ViewCatalog> catalog) {
-    getViewCatalog(db) = std::move(catalog);
+    getViewCatalog(db).set(std::move(catalog));
 }
 
-Status ViewCatalog::reload(OperationContext* opCtx, ViewCatalogLookupBehavior lookupBehavior) {
+Status ViewCatalog::reload(OperationContext* opCtx,
+                           const Database* db,
+                           ViewCatalogLookupBehavior lookupBehavior) {
+    auto catalog = getViewCatalog(db).writer();
     Lock::CollectionLock systemViewsLock(
         opCtx,
-        NamespaceString(_durable->getName(), NamespaceString::kSystemDotViewsCollectionName),
+        NamespaceString(catalog->_durable->getName(),
+                        NamespaceString::kSystemDotViewsCollectionName),
         MODE_IS);
-    stdx::unique_lock<Latch> lk(_mutex);
-    return _reload(lk, opCtx, ViewCatalogLookupBehavior::kValidateDurableViews);
+    auto result =
+        catalog.writable()->_reload(opCtx, ViewCatalogLookupBehavior::kValidateDurableViews, true);
+    catalog.commit();
+    return result;
 }
 
-Status ViewCatalog::_reload(WithLock,
-                            OperationContext* opCtx,
-                            ViewCatalogLookupBehavior lookupBehavior) {
-    LOG(1) << "reloading view catalog for database " << _durable->getName();
+Status ViewCatalog::_reload(OperationContext* opCtx,
+                            ViewCatalogLookupBehavior lookupBehavior,
+                            bool reloadForCollectionCatalog) {
+    const auto& dbName = _durable->getName();
+    LOGV2_DEBUG(22546, 1, "Reloading view catalog for database", "db"_attr = dbName);
 
     _viewMap.clear();
     _valid = false;
     _viewGraphNeedsRefresh = true;
+    _stats = {};
+
+    absl::flat_hash_set<NamespaceString> viewsForDb;
 
     auto reloadCallback = [&](const BSONObj& view) -> Status {
         BSONObj collationSpec = view.hasField("collation") ? view["collation"].Obj() : BSONObj();
@@ -119,11 +212,26 @@ Status ViewCatalog::_reload(WithLock,
             }
         }
 
-        _viewMap[viewName.ns()] = std::make_shared<ViewDefinition>(viewName.db(),
-                                                                   viewName.coll(),
-                                                                   view["viewOn"].str(),
-                                                                   pipeline,
-                                                                   std::move(collator.getValue()));
+        auto viewDef = std::make_shared<ViewDefinition>(viewName.db(),
+                                                        viewName.coll(),
+                                                        view["viewOn"].str(),
+                                                        pipeline,
+                                                        std::move(collator.getValue()));
+
+        if (!viewName.isOnInternalDb() && !viewName.isSystem()) {
+            if (viewDef->timeseries()) {
+                _stats.userTimeseries += 1;
+            } else {
+                _stats.userViews += 1;
+            }
+        } else {
+            _stats.internal += 1;
+        }
+
+        _viewMap[viewName.ns()] = std::move(viewDef);
+        if (reloadForCollectionCatalog) {
+            viewsForDb.insert(viewName);
+        }
         return Status::OK();
     };
 
@@ -135,10 +243,18 @@ Status ViewCatalog::_reload(WithLock,
         } else {
             MONGO_UNREACHABLE;
         }
+        if (reloadForCollectionCatalog) {
+            CollectionCatalog::write(
+                opCtx, [&dbName, viewsForDb = std::move(viewsForDb)](CollectionCatalog& catalog) {
+                    catalog.replaceViewsForDatabase(dbName, std::move(viewsForDb));
+                });
+        }
     } catch (const DBException& ex) {
         auto status = ex.toStatus();
-        LOG(0) << "could not load view catalog for database " << _durable->getName() << ": "
-               << status;
+        LOGV2(22547,
+              "Could not load view catalog for database",
+              "db"_attr = _durable->getName(),
+              "error"_attr = status);
         return status;
     }
 
@@ -146,41 +262,52 @@ Status ViewCatalog::_reload(WithLock,
     return Status::OK();
 }
 
-void ViewCatalog::clear() {
-    stdx::unique_lock<Latch> lk(_mutex);
+void ViewCatalog::clear(OperationContext* opCtx, const Database* db) {
+    auto catalog = getViewCatalog(db).writer();
 
-    _viewMap.clear();
-    _viewGraph.clear();
-    _valid = true;
-    _viewGraphNeedsRefresh = false;
+    // First, iterate through the views on this database and audit them before they are dropped.
+    for (auto&& view : catalog->_viewMap) {
+        audit::logDropView(opCtx->getClient(),
+                           (*view.second).name(),
+                           (*view.second).viewOn().ns(),
+                           (*view.second).pipeline(),
+                           ErrorCodes::OK);
+    }
+
+    catalog.writable()->_viewMap.clear();
+    catalog.writable()->_viewGraph.clear();
+    catalog.writable()->_valid = true;
+    catalog.writable()->_viewGraphNeedsRefresh = false;
+    catalog.writable()->_stats = {};
+    CollectionCatalog::write(opCtx, [db](CollectionCatalog& catalog) {
+        catalog.replaceViewsForDatabase(db->name(), {});
+    });
+    catalog.commit();
 }
 
 bool ViewCatalog::shouldIgnoreExternalChange(OperationContext* opCtx,
-                                             const NamespaceString& name) const {
-    return _ignoreExternalChange;
+                                             const Database* db,
+                                             const NamespaceString& name) {
+    return getViewCatalog(db).shouldIgnoreExternalChange();
 }
 
-void ViewCatalog::_requireValidCatalog(WithLock) {
+void ViewCatalog::_requireValidCatalog() const {
     uassert(ErrorCodes::InvalidViewDefinition,
             "Invalid view definition detected in the view catalog. Remove the invalid view "
             "manually to prevent disallowing any further usage of the view catalog.",
             _valid);
 }
 
-void ViewCatalog::iterate(OperationContext* opCtx, ViewIteratorCallback callback) {
-    Lock::CollectionLock systemViewsLock(
-        opCtx,
-        NamespaceString(_durable->getName(), NamespaceString::kSystemDotViewsCollectionName),
-        MODE_IS);
-    stdx::lock_guard<Latch> lk(_mutex);
-    _requireValidCatalog(lk);
+void ViewCatalog::iterate(ViewIteratorCallback callback) const {
+    _requireValidCatalog();
     for (auto&& view : _viewMap) {
-        callback(*view.second);
+        if (!callback(*view.second)) {
+            break;
+        }
     }
 }
 
-Status ViewCatalog::_createOrUpdateView(WithLock lk,
-                                        OperationContext* opCtx,
+Status ViewCatalog::_createOrUpdateView(OperationContext* opCtx,
                                         const NamespaceString& viewName,
                                         const NamespaceString& viewOn,
                                         const BSONArray& pipeline,
@@ -190,11 +317,7 @@ Status ViewCatalog::_createOrUpdateView(WithLock lk,
     invariant(opCtx->lockState()->isCollectionLockedForMode(
         NamespaceString(viewName.db(), NamespaceString::kSystemDotViewsCollectionName), MODE_X));
 
-    _requireValidCatalog(lk);
-
-    ON_BLOCK_EXIT([this] { _ignoreExternalChange = false; });
-
-    _ignoreExternalChange = true;
+    _requireValidCatalog();
 
     // Build the BSON definition for this view to be saved in the durable view catalog. If the
     // collation is empty, omit it from the definition altogether.
@@ -211,7 +334,7 @@ Status ViewCatalog::_createOrUpdateView(WithLock lk,
         viewName.db(), viewName.coll(), viewOn.coll(), ownedPipeline, std::move(collator));
 
     // Check that the resulting dependency graph is acyclic and within the maximum depth.
-    Status graphStatus = _upsertIntoGraph(lk, opCtx, *(view.get()));
+    Status graphStatus = _upsertIntoGraph(opCtx, *(view.get()));
     if (!graphStatus.isOK()) {
         return graphStatus;
     }
@@ -219,32 +342,34 @@ Status ViewCatalog::_createOrUpdateView(WithLock lk,
     _durable->upsert(opCtx, viewName, viewDefBuilder.obj());
     _viewMap[viewName.ns()] = view;
 
-    // Register the view in the CollectionCatalog mapping from ResourceID->namespace
-    CollectionCatalog& catalog = CollectionCatalog::get(opCtx);
-    auto viewRid = ResourceId(RESOURCE_COLLECTION, viewName.ns());
-    catalog.addResource(viewRid, viewName.ns());
-
-    opCtx->recoveryUnit()->onRollback([this, viewName, opCtx, viewRid]() {
-        this->_viewMap.erase(viewName.ns());
-        this->_viewGraphNeedsRefresh = true;
-        CollectionCatalog& catalog = CollectionCatalog::get(opCtx);
-        catalog.removeResource(viewRid, viewName.ns());
-    });
-
     // Reload the view catalog with the changes applied.
-    return _reload(lk, opCtx, ViewCatalogLookupBehavior::kValidateDurableViews);
+    auto res = _reload(opCtx, ViewCatalogLookupBehavior::kValidateDurableViews, false);
+    if (res.isOK()) {
+        // Register the view in the CollectionCatalog mapping from ResourceID->namespace
+        auto viewRid = ResourceId(RESOURCE_COLLECTION, viewName.ns());
+
+        CollectionCatalog::write(opCtx, [&](CollectionCatalog& catalog) {
+            catalog.registerView(viewName);
+            catalog.addResource(viewRid, viewName.ns());
+        });
+
+        opCtx->recoveryUnit()->onRollback([viewName, opCtx, viewRid]() {
+            CollectionCatalog::write(opCtx, [&](CollectionCatalog& catalog) {
+                catalog.removeResource(viewRid, viewName.ns());
+                catalog.deregisterView(viewName);
+            });
+        });
+    }
+    return res;
 }
 
-Status ViewCatalog::_upsertIntoGraph(WithLock lk,
-                                     OperationContext* opCtx,
-                                     const ViewDefinition& viewDef) {
+Status ViewCatalog::_upsertIntoGraph(OperationContext* opCtx, const ViewDefinition& viewDef) {
 
     // Performs the insert into the graph.
-    auto doInsert = [this, &lk, &opCtx](const ViewDefinition& viewDef,
-                                        bool needsValidation) -> Status {
+    auto doInsert = [this, &opCtx](const ViewDefinition& viewDef, bool needsValidation) -> Status {
         // Validate that the pipeline is eligible to serve as a view definition. If it is, this
         // will also return the set of involved namespaces.
-        auto pipelineStatus = _validatePipeline(lk, opCtx, viewDef);
+        auto pipelineStatus = validatePipeline(opCtx, viewDef);
         if (!pipelineStatus.isOK()) {
             if (needsValidation) {
                 uassertStatusOKWithContext(pipelineStatus.getStatus(),
@@ -265,7 +390,7 @@ Status ViewCatalog::_upsertIntoGraph(WithLock lk,
 
         if (needsValidation) {
             // Check the collation of all the dependent namespaces before updating the graph.
-            auto collationStatus = _validateCollation(lk, opCtx, viewDef, refs);
+            auto collationStatus = _validateCollation(opCtx, viewDef, refs);
             if (!collationStatus.isOK()) {
                 return collationStatus;
             }
@@ -286,7 +411,6 @@ Status ViewCatalog::_upsertIntoGraph(WithLock lk,
             }
         }
         // Only if the inserts completed without error will we no longer need a refresh.
-        opCtx->recoveryUnit()->onRollback([this]() { this->_viewGraphNeedsRefresh = true; });
         _viewGraphNeedsRefresh = false;
     }
 
@@ -297,12 +421,20 @@ Status ViewCatalog::_upsertIntoGraph(WithLock lk,
     return doInsert(viewDef, true);
 }
 
-StatusWith<stdx::unordered_set<NamespaceString>> ViewCatalog::_validatePipeline(
-    WithLock lk, OperationContext* opCtx, const ViewDefinition& viewDef) const {
-    AggregationRequest request(viewDef.viewOn(), viewDef.pipeline());
-    const LiteParsedPipeline liteParsedPipeline(request);
+StatusWith<stdx::unordered_set<NamespaceString>> ViewCatalog::validatePipeline(
+    OperationContext* opCtx, const ViewDefinition& viewDef) {
+    const LiteParsedPipeline liteParsedPipeline(viewDef.viewOn(), viewDef.pipeline());
     const auto involvedNamespaces = liteParsedPipeline.getInvolvedNamespaces();
 
+    // The API version pipeline validation should be skipped for time-series view because of
+    // following reasons:
+    //     - the view pipeline is not created by (or visible to) the end-user and should be skipped.
+    //     - the view pipeline can have stages that are not allowed in stable API version '1' eg.
+    //       '$_internalUnpackBucket'.
+    bool performApiVersionChecks = !viewDef.timeseries();
+
+    liteParsedPipeline.validate(opCtx, performApiVersionChecks);
+
     // Verify that this is a legitimate pipeline specification by making sure it parses
     // correctly. In order to parse a pipeline we need to resolve any namespaces involved to a
     // collection and a pipeline, but in this case we don't need this map to be accurate since
@@ -313,7 +445,7 @@ StatusWith<stdx::unordered_set<NamespaceString>> ViewCatalog::_validatePipeline(
     }
     boost::intrusive_ptr<ExpressionContext> expCtx =
         new ExpressionContext(opCtx,
-                              request,
+                              AggregateCommandRequest(viewDef.viewOn(), viewDef.pipeline()),
                               CollatorInterface::cloneCollator(viewDef.defaultCollator()),
                               // We can use a stub MongoProcessInterface because we are only parsing
                               // the Pipeline for validation here. We won't do anything with the
@@ -322,51 +454,64 @@ StatusWith<stdx::unordered_set<NamespaceString>> ViewCatalog::_validatePipeline(
                               std::move(resolvedNamespaces),
                               boost::none);
 
-    // Save this to a variable to avoid reading the atomic variable multiple times.
-    auto currentFCV = serverGlobalParams.featureCompatibility.getVersion();
-
-    // If the feature compatibility version is not 4.2, and we are validating features as master,
-    // ban the use of new agg features introduced in 4.2 to prevent them from being persisted in the
-    // catalog.
-    if (serverGlobalParams.validateFeaturesAsMaster.load() &&
-        currentFCV != ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo42) {
-        expCtx->maxFeatureCompatibilityVersion = currentFCV;
-    }
-    auto pipelineStatus = Pipeline::parse(viewDef.pipeline(), std::move(expCtx));
-    if (!pipelineStatus.isOK()) {
-        return pipelineStatus.getStatus();
+    // If the feature compatibility version is not kLatest, and we are validating features as
+    // primary, ban the use of new agg features introduced in kLatest to prevent them from being
+    // persisted in the catalog.
+    // (Generic FCV reference): This FCV check should exist across LTS binary versions.
+    ServerGlobalParams::FeatureCompatibility::Version fcv;
+    if (serverGlobalParams.validateFeaturesAsPrimary.load() &&
+        serverGlobalParams.featureCompatibility.isLessThan(
+            ServerGlobalParams::FeatureCompatibility::kLatest, &fcv)) {
+        expCtx->maxFeatureCompatibilityVersion = fcv;
     }
 
-    // Validate that the view pipeline does not contain any ineligible stages.
-    const auto& sources = pipelineStatus.getValue()->getSources();
-    if (!sources.empty()) {
-        const auto firstPersistentStage =
-            std::find_if(sources.begin(), sources.end(), [](const auto& source) {
-                return source->constraints().writesPersistentData();
+    // The pipeline parser needs to know that we're parsing a pipeline for a view definition
+    // to apply some additional checks.
+    expCtx->isParsingViewDefinition = true;
+
+    try {
+        auto pipeline =
+            Pipeline::parse(viewDef.pipeline(), std::move(expCtx), [&](const Pipeline& pipeline) {
+                // Validate that the view pipeline does not contain any ineligible stages.
+                const auto& sources = pipeline.getSources();
+                const auto firstPersistentStage =
+                    std::find_if(sources.begin(), sources.end(), [](const auto& source) {
+                        return source->constraints().writesPersistentData();
+                    });
+
+                uassert(ErrorCodes::OptionNotSupportedOnView,
+                        str::stream()
+                            << "The aggregation stage "
+                            << firstPersistentStage->get()->getSourceName() << " in location "
+                            << std::distance(sources.begin(), firstPersistentStage)
+                            << " of the pipeline cannot be used in the view definition of "
+                            << viewDef.name().ns() << " because it writes to disk",
+                        firstPersistentStage == sources.end());
+
+                uassert(ErrorCodes::OptionNotSupportedOnView,
+                        "$changeStream cannot be used in a view definition",
+                        sources.empty() || !sources.front()->constraints().isChangeStreamStage());
+
+                std::for_each(sources.begin(), sources.end(), [](auto& stage) {
+                    uassert(ErrorCodes::InvalidNamespace,
+                            str::stream() << "'" << stage->getSourceName()
+                                          << "' cannot be used in a view definition",
+                            !stage->constraints().isIndependentOfAnyCollection);
+                });
             });
-        if (sources.front()->constraints().isChangeStreamStage()) {
-            return {ErrorCodes::OptionNotSupportedOnView,
-                    "$changeStream cannot be used in a view definition"};
-        } else if (firstPersistentStage != sources.end()) {
-            mongo::StringBuilder errorMessage;
-            errorMessage << "The aggregation stage " << firstPersistentStage->get()->getSourceName()
-                         << " in location " << std::distance(sources.begin(), firstPersistentStage)
-                         << " of the pipeline cannot be used in the view definition of "
-                         << viewDef.name().ns() << " because it writes to disk";
-            return {ErrorCodes::OptionNotSupportedOnView, errorMessage.str()};
-        }
+    } catch (const DBException& ex) {
+        return ex.toStatus();
     }
 
     return std::move(involvedNamespaces);
 }
 
-Status ViewCatalog::_validateCollation(WithLock lk,
-                                       OperationContext* opCtx,
+Status ViewCatalog::_validateCollation(OperationContext* opCtx,
                                        const ViewDefinition& view,
-                                       const std::vector<NamespaceString>& refs) {
+                                       const std::vector<NamespaceString>& refs) const {
     for (auto&& potentialViewNss : refs) {
-        auto otherView = _lookup(
-            lk, opCtx, potentialViewNss.ns(), ViewCatalogLookupBehavior::kValidateDurableViews);
+        auto otherView =
+            _lookup(opCtx, potentialViewNss.ns(), ViewCatalogLookupBehavior::kValidateDurableViews);
         if (otherView &&
             !CollatorInterface::collatorsMatch(view.defaultCollator(),
                                                otherView->defaultCollator())) {
@@ -380,6 +525,7 @@ Status ViewCatalog::_validateCollation(WithLock lk,
 }
 
 Status ViewCatalog::createView(OperationContext* opCtx,
+                               const Database* db,
                                const NamespaceString& viewName,
                                const NamespaceString& viewOn,
                                const BSONArray& pipeline,
@@ -389,34 +535,41 @@ Status ViewCatalog::createView(OperationContext* opCtx,
     invariant(opCtx->lockState()->isCollectionLockedForMode(
         NamespaceString(viewName.db(), NamespaceString::kSystemDotViewsCollectionName), MODE_X));
 
-    stdx::lock_guard<Latch> lk(_mutex);
+    const auto& catalogStorage = getViewCatalog(db);
+    auto catalog = catalogStorage.writer();
 
     if (viewName.db() != viewOn.db())
         return Status(ErrorCodes::BadValue,
                       "View must be created on a view or collection in the same database");
 
-    if (_lookup(
-            lk, opCtx, StringData(viewName.ns()), ViewCatalogLookupBehavior::kValidateDurableViews))
+    if (catalog->_lookup(
+            opCtx, StringData(viewName.ns()), ViewCatalogLookupBehavior::kValidateDurableViews))
         return Status(ErrorCodes::NamespaceExists, "Namespace already exists");
 
     if (!NamespaceString::validCollectionName(viewOn.coll()))
         return Status(ErrorCodes::InvalidNamespace,
                       str::stream() << "invalid name for 'viewOn': " << viewOn.coll());
 
-    if (viewName.isSystem())
-        return Status(
-            ErrorCodes::InvalidNamespace,
-            "View name cannot start with 'system.', which is reserved for system namespaces");
-
     auto collator = parseCollator(opCtx, collation);
     if (!collator.isOK())
         return collator.getStatus();
 
-    return _createOrUpdateView(
-        lk, opCtx, viewName, viewOn, pipeline, std::move(collator.getValue()));
+    Status result = Status::OK();
+    {
+        ON_BLOCK_EXIT([&catalogStorage] { catalogStorage.setIgnoreExternalChange(false); });
+        catalogStorage.setIgnoreExternalChange(true);
+
+        result = catalog.writable()->_createOrUpdateView(
+            opCtx, viewName, viewOn, pipeline, std::move(collator.getValue()));
+    }
+    if (result.isOK()) {
+        catalog.commit();
+    }
+    return result;
 }
 
 Status ViewCatalog::modifyView(OperationContext* opCtx,
+                               const Database* db,
                                const NamespaceString& viewName,
                                const NamespaceString& viewOn,
                                const BSONArray& pipeline) {
@@ -424,14 +577,15 @@ Status ViewCatalog::modifyView(OperationContext* opCtx,
     invariant(opCtx->lockState()->isCollectionLockedForMode(
         NamespaceString(viewName.db(), NamespaceString::kSystemDotViewsCollectionName), MODE_X));
 
-    stdx::lock_guard<Latch> lk(_mutex);
+    const auto& catalogStorage = getViewCatalog(db);
+    auto catalog = catalogStorage.writer();
 
     if (viewName.db() != viewOn.db())
         return Status(ErrorCodes::BadValue,
                       "View must be created on a view or collection in the same database");
 
     auto viewPtr =
-        _lookup(lk, opCtx, viewName.ns(), ViewCatalogLookupBehavior::kValidateDurableViews);
+        catalog->_lookup(opCtx, viewName.ns(), ViewCatalogLookupBehavior::kValidateDurableViews);
     if (!viewPtr)
         return Status(ErrorCodes::NamespaceNotFound,
                       str::stream() << "cannot modify missing view " << viewName.ns());
@@ -440,71 +594,92 @@ Status ViewCatalog::modifyView(OperationContext* opCtx,
         return Status(ErrorCodes::InvalidNamespace,
                       str::stream() << "invalid name for 'viewOn': " << viewOn.coll());
 
-    ViewDefinition savedDefinition = *viewPtr;
-
-    opCtx->recoveryUnit()->onRollback([this, viewName, savedDefinition, opCtx]() {
-        this->_viewMap[viewName.ns()] = std::make_shared<ViewDefinition>(savedDefinition);
+    opCtx->recoveryUnit()->onRollback([viewName, opCtx]() {
         auto viewRid = ResourceId(RESOURCE_COLLECTION, viewName.ns());
-        CollectionCatalog& catalog = CollectionCatalog::get(opCtx);
-        catalog.addResource(viewRid, viewName.ns());
+
+        CollectionCatalog::write(opCtx, [&](CollectionCatalog& catalog) {
+            catalog.addResource(viewRid, viewName.ns());
+        });
     });
 
-    return _createOrUpdateView(lk,
-                               opCtx,
-                               viewName,
-                               viewOn,
-                               pipeline,
-                               CollatorInterface::cloneCollator(savedDefinition.defaultCollator()));
+    Status result = Status::OK();
+    {
+        ON_BLOCK_EXIT([&catalogStorage] { catalogStorage.setIgnoreExternalChange(false); });
+        catalogStorage.setIgnoreExternalChange(true);
+
+        result = catalog.writable()->_createOrUpdateView(
+            opCtx,
+            viewName,
+            viewOn,
+            pipeline,
+            CollatorInterface::cloneCollator(viewPtr->defaultCollator()));
+    }
+
+    if (result.isOK()) {
+        catalog.commit();
+    }
+
+    return result;
 }
 
-Status ViewCatalog::dropView(OperationContext* opCtx, const NamespaceString& viewName) {
+Status ViewCatalog::dropView(OperationContext* opCtx,
+                             const Database* db,
+                             const NamespaceString& viewName) {
     invariant(opCtx->lockState()->isDbLockedForMode(viewName.db(), MODE_IX));
     invariant(opCtx->lockState()->isCollectionLockedForMode(viewName, MODE_IX));
     invariant(opCtx->lockState()->isCollectionLockedForMode(
         NamespaceString(viewName.db(), NamespaceString::kSystemDotViewsCollectionName), MODE_X));
 
-    stdx::lock_guard<Latch> lk(_mutex);
-    _requireValidCatalog(lk);
+    const auto& catalogStorage = getViewCatalog(db);
+    auto catalog = catalogStorage.writer();
+    catalog->_requireValidCatalog();
 
-    ON_BLOCK_EXIT([this] { _ignoreExternalChange = false; });
+    Status result = Status::OK();
 
-    _ignoreExternalChange = true;
+    {
+        ON_BLOCK_EXIT([&catalogStorage] { catalogStorage.setIgnoreExternalChange(false); });
 
-    // Save a copy of the view definition in case we need to roll back.
-    auto viewPtr =
-        _lookup(lk, opCtx, viewName.ns(), ViewCatalogLookupBehavior::kValidateDurableViews);
-    if (!viewPtr) {
-        return {ErrorCodes::NamespaceNotFound,
-                str::stream() << "cannot drop missing view: " << viewName.ns()};
+        catalogStorage.setIgnoreExternalChange(true);
+
+        // Save a copy of the view definition in case we need to roll back.
+        auto viewPtr = catalog->_lookup(
+            opCtx, viewName.ns(), ViewCatalogLookupBehavior::kValidateDurableViews);
+        if (!viewPtr) {
+            return {ErrorCodes::NamespaceNotFound,
+                    str::stream() << "cannot drop missing view: " << viewName.ns()};
+        }
+
+        invariant(catalog->_valid);
+        catalog.writable()->_durable->remove(opCtx, viewName);
+        catalog.writable()->_viewGraph.remove(viewPtr->name());
+        catalog.writable()->_viewMap.erase(viewName.ns());
+
+        auto viewRid = ResourceId(RESOURCE_COLLECTION, viewName.ns());
+        CollectionCatalog::write(opCtx, [&](CollectionCatalog& catalog) {
+            catalog.removeResource(viewRid, viewName.ns());
+        });
+
+        opCtx->recoveryUnit()->onCommit([viewName, opCtx](auto ts) {
+            CollectionCatalog::write(
+                opCtx, [&](CollectionCatalog& catalog) { catalog.deregisterView(viewName); });
+        });
+
+        opCtx->recoveryUnit()->onRollback([viewName, opCtx, viewRid]() {
+            CollectionCatalog::write(opCtx, [&](CollectionCatalog& catalog) {
+                catalog.addResource(viewRid, viewName.ns());
+            });
+        });
+
+        // Reload the view catalog with the changes applied.
+        result = catalog.writable()->_reload(
+            opCtx, ViewCatalogLookupBehavior::kValidateDurableViews, false);
     }
-
-    ViewDefinition savedDefinition = *viewPtr;
-
-    invariant(_valid);
-    _durable->remove(opCtx, viewName);
-    _viewGraph.remove(savedDefinition.name());
-    _viewMap.erase(viewName.ns());
-
-    CollectionCatalog& catalog = CollectionCatalog::get(opCtx);
-    auto viewRid = ResourceId(RESOURCE_COLLECTION, viewName.ns());
-    catalog.removeResource(viewRid, viewName.ns());
-
-    opCtx->recoveryUnit()->onRollback([this, viewName, savedDefinition, opCtx, viewRid]() {
-        this->_viewGraphNeedsRefresh = true;
-        this->_viewMap[viewName.ns()] = std::make_shared<ViewDefinition>(savedDefinition);
-        CollectionCatalog& catalog = CollectionCatalog::get(opCtx);
-        catalog.addResource(viewRid, viewName.ns());
-    });
-
-    // Reload the view catalog with the changes applied.
-    return _reload(lk, opCtx, ViewCatalogLookupBehavior::kValidateDurableViews);
+    catalog.commit();
+    return result;
 }
 
-std::shared_ptr<ViewDefinition> ViewCatalog::_lookup(WithLock lk,
-                                                     OperationContext* opCtx,
-                                                     StringData ns,
-                                                     ViewCatalogLookupBehavior lookupBehavior) {
-
+std::shared_ptr<const ViewDefinition> ViewCatalog::_lookup(
+    OperationContext* opCtx, StringData ns, ViewCatalogLookupBehavior lookupBehavior) const {
     ViewMap::const_iterator it = _viewMap.find(ns);
     if (it != _viewMap.end()) {
         return it->second;
@@ -512,12 +687,15 @@ std::shared_ptr<ViewDefinition> ViewCatalog::_lookup(WithLock lk,
     return nullptr;
 }
 
-std::shared_ptr<ViewDefinition> ViewCatalog::lookup(OperationContext* opCtx, StringData ns) {
-    Lock::CollectionLock systemViewsLock(
-        opCtx,
-        NamespaceString(_durable->getName(), NamespaceString::kSystemDotViewsCollectionName),
-        MODE_IS);
-    stdx::lock_guard<Latch> lk(_mutex);
+std::shared_ptr<ViewDefinition> ViewCatalog::_lookup(OperationContext* opCtx,
+                                                     StringData ns,
+                                                     ViewCatalogLookupBehavior lookupBehavior) {
+    return std::const_pointer_cast<ViewDefinition>(
+        std::as_const(*this)._lookup(opCtx, ns, lookupBehavior));
+}
+
+std::shared_ptr<const ViewDefinition> ViewCatalog::lookup(OperationContext* opCtx,
+                                                          StringData ns) const {
     if (!_valid && opCtx->getClient()->isFromUserConnection()) {
         // We want to avoid lookups on invalid collection names.
         if (!NamespaceString::validCollectionName(ns)) {
@@ -527,31 +705,20 @@ std::shared_ptr<ViewDefinition> ViewCatalog::lookup(OperationContext* opCtx, Str
         // ApplyOps should work on a valid existing collection, despite the presence of bad views
         // otherwise the server would crash. The view catalog will remain invalid until the bad view
         // definitions are removed.
-        _requireValidCatalog(lk);
+        _requireValidCatalog();
     }
 
-    return _lookup(lk, opCtx, ns, ViewCatalogLookupBehavior::kValidateDurableViews);
+    return _lookup(opCtx, ns, ViewCatalogLookupBehavior::kValidateDurableViews);
 }
 
-std::shared_ptr<ViewDefinition> ViewCatalog::lookupWithoutValidatingDurableViews(
-    OperationContext* opCtx, StringData ns) {
-    Lock::CollectionLock systemViewsLock(
-        opCtx,
-        NamespaceString(_durable->getName(), NamespaceString::kSystemDotViewsCollectionName),
-        MODE_IS);
-    stdx::lock_guard<Latch> lk(_mutex);
-    return _lookup(lk, opCtx, ns, ViewCatalogLookupBehavior::kAllowInvalidDurableViews);
+std::shared_ptr<const ViewDefinition> ViewCatalog::lookupWithoutValidatingDurableViews(
+    OperationContext* opCtx, StringData ns) const {
+    return _lookup(opCtx, ns, ViewCatalogLookupBehavior::kAllowInvalidDurableViews);
 }
 
 StatusWith<ResolvedView> ViewCatalog::resolveView(OperationContext* opCtx,
-                                                  const NamespaceString& nss) {
-    Lock::CollectionLock systemViewsLock(
-        opCtx,
-        NamespaceString(_durable->getName(), NamespaceString::kSystemDotViewsCollectionName),
-        MODE_IS);
-    stdx::unique_lock<Latch> lock(_mutex);
-
-    _requireValidCatalog(lock);
+                                                  const NamespaceString& nss) const {
+    _requireValidCatalog();
 
     // Keep looping until the resolution completes. If the catalog is invalidated during the
     // resolution, we start over from the beginning.
@@ -570,10 +737,12 @@ StatusWith<ResolvedView> ViewCatalog::resolveView(OperationContext* opCtx,
         // 'resolvedNss'.
         std::shared_ptr<ViewDefinition> lastViewDefinition;
 
+        std::vector<NamespaceString> dependencyChain{nss};
+
         int depth = 0;
         for (; depth < ViewGraph::kMaxViewDepth; depth++) {
-            auto view = _lookup(
-                lock, opCtx, resolvedNss->ns(), ViewCatalogLookupBehavior::kValidateDurableViews);
+            auto view =
+                _lookup(opCtx, resolvedNss->ns(), ViewCatalogLookupBehavior::kValidateDurableViews);
             if (!view) {
                 // Return error status if pipeline is too large.
                 int pipelineSize = 0;
@@ -585,6 +754,10 @@ StatusWith<ResolvedView> ViewCatalog::resolveView(OperationContext* opCtx,
                             str::stream() << "View pipeline exceeds maximum size; maximum size is "
                                           << ViewGraph::kMaxViewPipelineSizeBytes};
                 }
+
+                auto curOp = CurOp::get(opCtx);
+                curOp->debug().addResolvedViews(dependencyChain, resolvedPipeline);
+
                 return StatusWith<ResolvedView>(
                     {*resolvedNss,
                      std::move(resolvedPipeline),
@@ -592,6 +765,7 @@ StatusWith<ResolvedView> ViewCatalog::resolveView(OperationContext* opCtx,
             }
 
             resolvedNss = &view->viewOn();
+            dependencyChain.push_back(*resolvedNss);
             if (!collation) {
                 collation = view->defaultCollator() ? view->defaultCollator()->getSpec().toBSON()
                                                     : CollationSpec::kSimpleSpec;
@@ -603,6 +777,9 @@ StatusWith<ResolvedView> ViewCatalog::resolveView(OperationContext* opCtx,
 
             // If the first stage is a $collStats, then we return early with the viewOn namespace.
             if (toPrepend.size() > 0 && !toPrepend[0]["$collStats"].eoo()) {
+                auto curOp = CurOp::get(opCtx);
+                curOp->debug().addResolvedViews(dependencyChain, resolvedPipeline);
+
                 return StatusWith<ResolvedView>(
                     {*resolvedNss, std::move(resolvedPipeline), std::move(collation.get())});
             }
@@ -616,4 +793,8 @@ StatusWith<ResolvedView> ViewCatalog::resolveView(OperationContext* opCtx,
     };
     MONGO_UNREACHABLE;
 }
+
+ViewCatalog::Stats ViewCatalog::getStats() const {
+    return _stats;
+}
 }  // namespace mongo
