@@ -96,16 +96,15 @@ public:
     }
 
     ClientCursorParams makeParams(OperationContext* opCtx) {
-        return {
-            makeFakePlanExecutor(opCtx),
-            kTestNss,
-            {},
-            APIParameters(),
-            opCtx->getWriteConcern(),
-            repl::ReadConcernArgs(repl::ReadConcernLevel::kLocalReadConcern),
-            BSONObj(),
-            PrivilegeVector(),
-        };
+        return {makeFakePlanExecutor(opCtx),
+                kTestNss,
+                {},
+                APIParameters(),
+                opCtx->getWriteConcern(),
+                repl::ReadConcernArgs(repl::ReadConcernLevel::kLocalReadConcern),
+                BSONObj(),
+                PrivilegeVector(),
+                false /*isOpQueryExhaust*/};
     }
 
     ClientCursorPin makeCursor(OperationContext* opCtx) {
@@ -155,7 +154,8 @@ TEST_F(CursorManagerTest, ShouldBeAbleToKillPinnedCursor) {
          {},
          repl::ReadConcernArgs(repl::ReadConcernLevel::kLocalReadConcern),
          BSONObj(),
-         PrivilegeVector()});
+         PrivilegeVector(),
+         false /*isOpQueryExhaust*/});
 
     auto cursorId = cursorPin.getCursor()->cursorid();
     ASSERT_OK(cursorManager->killCursor(_opCtx.get(), cursorId));
@@ -181,7 +181,8 @@ TEST_F(CursorManagerTest, ShouldBeAbleToKillPinnedCursorMultiClient) {
          {},
          repl::ReadConcernArgs(repl::ReadConcernLevel::kLocalReadConcern),
          BSONObj(),
-         PrivilegeVector()});
+         PrivilegeVector(),
+         false /*isOpQueryExhaust*/});
 
     auto cursorId = cursorPin.getCursor()->cursorid();
 
@@ -218,7 +219,8 @@ TEST_F(CursorManagerTest, InactiveCursorShouldTimeout) {
                                    {},
                                    repl::ReadConcernArgs(repl::ReadConcernLevel::kLocalReadConcern),
                                    BSONObj(),
-                                   PrivilegeVector()});
+                                   PrivilegeVector(),
+                                   false /*isOpQueryExhaust*/});
 
     ASSERT_EQ(0UL, cursorManager->timeoutCursors(_opCtx.get(), Date_t()));
 
@@ -234,7 +236,8 @@ TEST_F(CursorManagerTest, InactiveCursorShouldTimeout) {
                                    {},
                                    repl::ReadConcernArgs(repl::ReadConcernLevel::kLocalReadConcern),
                                    BSONObj(),
-                                   PrivilegeVector()});
+                                   PrivilegeVector(),
+                                   false /*isOpQueryExhaust*/});
     ASSERT_EQ(1UL, cursorManager->timeoutCursors(_opCtx.get(), Date_t::max()));
     ASSERT_EQ(0UL, cursorManager->numCursors());
 }
@@ -255,7 +258,8 @@ TEST_F(CursorManagerTest, InactivePinnedCursorShouldNotTimeout) {
          {},
          repl::ReadConcernArgs(repl::ReadConcernLevel::kLocalReadConcern),
          BSONObj(),
-         PrivilegeVector()});
+         PrivilegeVector(),
+         false /*isOpQueryExhaust*/});
 
     // The pin is still in scope, so it should not time out.
     clock->advance(getDefaultCursorTimeoutMillis());
@@ -280,7 +284,8 @@ TEST_F(CursorManagerTest, MarkedAsKilledCursorsShouldBeDeletedOnCursorPin) {
          {},
          repl::ReadConcernArgs(repl::ReadConcernLevel::kLocalReadConcern),
          BSONObj(),
-         PrivilegeVector()});
+         PrivilegeVector(),
+         false /*isOpQueryExhaust*/});
     auto cursorId = cursorPin->cursorid();
 
     // A cursor will stay alive, but be marked as killed, if it is interrupted with a code other
@@ -314,7 +319,8 @@ TEST_F(CursorManagerTest, InactiveKilledCursorsShouldTimeout) {
          {},
          repl::ReadConcernArgs(repl::ReadConcernLevel::kLocalReadConcern),
          BSONObj(),
-         PrivilegeVector()});
+         PrivilegeVector(),
+         false /*isOpQueryExhaust*/});
 
     // A cursor will stay alive, but be marked as killed, if it is interrupted with a code other
     // than ErrorCodes::Interrupted or ErrorCodes::CursorKilled and then unpinned.
@@ -347,7 +353,8 @@ TEST_F(CursorManagerTest, UsingACursorShouldUpdateTimeOfLastUse) {
          {},
          repl::ReadConcernArgs(repl::ReadConcernLevel::kLocalReadConcern),
          BSONObj(),
-         PrivilegeVector()});
+         PrivilegeVector(),
+         false /*isOpQueryExhaust*/});
     auto usedCursorId = cursorPin.getCursor()->cursorid();
     cursorPin.release();
 
@@ -361,7 +368,8 @@ TEST_F(CursorManagerTest, UsingACursorShouldUpdateTimeOfLastUse) {
                                    {},
                                    repl::ReadConcernArgs(repl::ReadConcernLevel::kLocalReadConcern),
                                    BSONObj(),
-                                   PrivilegeVector()});
+                                   PrivilegeVector(),
+                                   false /*isOpQueryExhaust*/});
 
     // Advance the clock to simulate time passing.
     clock->advance(Milliseconds(1));
@@ -399,7 +407,8 @@ TEST_F(CursorManagerTest, CursorShouldNotTimeOutUntilIdleForLongEnoughAfterBeing
          {},
          repl::ReadConcernArgs(repl::ReadConcernLevel::kLocalReadConcern),
          BSONObj(),
-         PrivilegeVector()});
+         PrivilegeVector(),
+         false /*isOpQueryExhaust*/});
 
     // Advance the clock to simulate time passing.
     clock->advance(getDefaultCursorTimeoutMillis() + Milliseconds(1));
@@ -441,7 +450,8 @@ TEST_F(CursorManagerTest, CursorStoresAPIParameters) {
          {},
          repl::ReadConcernArgs(repl::ReadConcernLevel::kLocalReadConcern),
          BSONObj(),
-         PrivilegeVector()});
+         PrivilegeVector(),
+         false /*isOpQueryExhaust*/});
 
     auto storedAPIParams = cursorPin->getAPIParameters();
     ASSERT_EQ("2", *storedAPIParams.getAPIVersion());
