@@ -128,6 +128,14 @@ public:
                                           bool upsert,
                                           const WriteConcernOptions& writeConcern) override;
 
+    StatusWith<bool> updateConfigDocument(OperationContext* opCtx,
+                                          const NamespaceString& nss,
+                                          const BSONObj& query,
+                                          const BSONObj& update,
+                                          bool upsert,
+                                          const WriteConcernOptions& writeConcern,
+                                          Milliseconds maxTimeMs) override;
+
     Status removeConfigDocuments(OperationContext* opCtx,
                                  const NamespaceString& nss,
                                  const BSONObj& query,
