@@ -350,16 +350,29 @@ void RoutingTableHistory::setAllShardsRefreshed() {
     }
 }
 
-Chunk ChunkManager::findIntersectingChunk(const BSONObj& shardKey, const BSONObj& collation) const {
+Chunk ChunkManager::findIntersectingChunk(const BSONObj& shardKey,
+                                          const BSONObj& collation,
+                                          bool bypassIsFieldHashedCheck) const {
     const bool hasSimpleCollation = (collation.isEmpty() && !_rt->optRt->getDefaultCollator()) ||
         SimpleBSONObjComparator::kInstance.evaluate(collation == CollationSpec::kSimpleSpec);
     if (!hasSimpleCollation) {
         for (BSONElement elt : shardKey) {
+            // We must assume that if the field is specified as "hashed" in the shard key pattern,
+            // then the hash value could have come from a collatable type.
+            const bool isFieldHashed =
+                (_rt->optRt->getShardKeyPattern().isHashedPattern() &&
+                 _rt->optRt->getShardKeyPattern().getHashedField().fieldNameStringData() ==
+                     elt.fieldNameStringData());
+
+            // If we want to skip the check in the special case where the _id field is hashed and
+            // used as the shard key, set bypassIsFieldHashedCheck. This assumes that a request with
+            // a query that contains an _id field can target a specific shard.
             uassert(ErrorCodes::ShardKeyNotFound,
                     str::stream() << "Cannot target single shard due to collation of key "
                                   << elt.fieldNameStringData() << " for namespace "
                                   << _rt->optRt->nss(),
-                    !CollationIndexKey::isCollatableType(elt.type()));
+                    !CollationIndexKey::isCollatableType(elt.type()) &&
+                        (!isFieldHashed || bypassIsFieldHashedCheck));
         }
     }
 
