@@ -64,7 +64,7 @@ StatusWith<ChunkVersion> ChunkVersion::fromBSON(const BSONObj& obj) {
         if (tsPart.type() != bsonTimestamp)
             return {ErrorCodes::TypeMismatch,
                     str::stream() << "Invalid type " << tsPart.type()
-                                  << " for version timestamp part."};
+                                  << " for version major and minor part."};
 
         version._combined = tsPart.timestamp().asULL();
     }
@@ -80,11 +80,28 @@ StatusWith<ChunkVersion> ChunkVersion::fromBSON(const BSONObj& obj) {
         version._epoch = epochPart.OID();
     }
 
+    // The following code handles the optional fields: canThrowSSVOnIgnored and timestamp. It is a
+    // bit complex because this function relies on the order of the fields, but since both of them
+    // are optional they might/might not be present.
+    BSONElement nextElem = it.next();
+    if (!nextElem.eoo() && nextElem.type() == BSONType::Bool) {
+        const BSONElement& canThrowSSVOnIgnoredPart = nextElem;
+        version._canThrowSSVOnIgnored = canThrowSSVOnIgnoredPart && canThrowSSVOnIgnoredPart.Bool();
+        nextElem = it.next();
+    }
+
+    if (!nextElem.eoo()) {
+        if (nextElem.type() != BSONType::bsonTimestamp)
+            return {ErrorCodes::TypeMismatch,
+                    str::stream() << "Invalid type " << nextElem.type()
+                                  << " for version timestamp part."};
+        version._timestamp = nextElem.timestamp();
+    }
+
     return version;
 }
 
 StatusWith<ChunkVersion> ChunkVersion::parseLegacyWithField(const BSONObj& obj, StringData field) {
-    // Expect the timestamp
     auto versionElem = obj[field];
     if (versionElem.eoo())
         return {ErrorCodes::NoSuchKey,
@@ -92,22 +109,41 @@ StatusWith<ChunkVersion> ChunkVersion::parseLegacyWithField(const BSONObj& obj,
 
     ChunkVersion version;
 
-    if (versionElem.type() == bsonTimestamp || versionElem.type() == Date) {
-        version._combined = versionElem._numberLong();
-    } else {
-        return {ErrorCodes::TypeMismatch,
-                str::stream() << "Invalid type " << versionElem.type()
-                              << " for version timestamp part."};
+    // Expect the major and minor
+    {
+        if (versionElem.type() == bsonTimestamp || versionElem.type() == Date) {
+            version._combined = versionElem._numberLong();
+        } else {
+            return {ErrorCodes::TypeMismatch,
+                    str::stream() << "Invalid type " << versionElem.type()
+                                  << " for version major and minor part."};
+        }
     }
 
-    // Expect the optional epoch OID
-    const auto epochField = field + "Epoch";
-    auto epochElem = obj[epochField];
-    if (epochElem.type() == jstOID) {
-        version._epoch = epochElem.OID();
-    } else if (!epochElem.eoo()) {
-        return {ErrorCodes::TypeMismatch,
-                str::stream() << "Invalid type " << epochElem.type() << " for version epoch part."};
+    // Expect the epoch OID
+    {
+        const auto epochField = field + "Epoch";
+        auto epochElem = obj[epochField];
+        if (epochElem.type() == jstOID) {
+            version._epoch = epochElem.OID();
+        } else if (!epochElem.eoo()) {
+            return {ErrorCodes::TypeMismatch,
+                    str::stream() << "Invalid type " << epochElem.type()
+                                  << " for version epoch part."};
+        }
+    }
+
+    // Handle the timestamp if present
+    {
+        const auto timestampField = field + "Timestamp";
+        auto timestampElem = obj[timestampField];
+        if (timestampElem.type() == bsonTimestamp) {
+            version._timestamp = timestampElem.timestamp();
+        } else if (!timestampElem.eoo()) {
+            return {ErrorCodes::TypeMismatch,
+                    str::stream() << "Invalid type " << timestampElem.type()
+                                  << " for version timestamp part."};
+        }
     }
 
     return version;
@@ -117,22 +153,43 @@ void ChunkVersion::appendWithField(BSONObjBuilder* out, StringData field) const
     BSONArrayBuilder arr(out->subarrayStart(field));
     arr.appendTimestamp(_combined);
     arr.append(_epoch);
+    if (_canThrowSSVOnIgnored) {
+        arr.append(_canThrowSSVOnIgnored);
+    }
+    if (_timestamp) {
+        arr.append(*_timestamp);
+    }
 }
 
 void ChunkVersion::appendLegacyWithField(BSONObjBuilder* out, StringData field) const {
     out->appendTimestamp(field, _combined);
     out->append(field + "Epoch", _epoch);
+    if (_timestamp) {
+        out->append(field + "Timestamp", *_timestamp);
+    }
 }
 
 BSONObj ChunkVersion::toBSON() const {
     BSONArrayBuilder b;
     b.appendTimestamp(_combined);
     b.append(_epoch);
+    if (_canThrowSSVOnIgnored) {
+        b.append(_canThrowSSVOnIgnored);
+    }
+    if (_timestamp) {
+        b.append(*_timestamp);
+    }
     return b.arr();
 }
 
+void ChunkVersion::legacyToBSON(StringData field, BSONObjBuilder* out) const {
+    out->appendTimestamp(field, this->toLong());
+}
+
 std::string ChunkVersion::toString() const {
-    return str::stream() << majorVersion() << "|" << minorVersion() << "||" << _epoch;
+    return str::stream() << majorVersion() << "|" << minorVersion() << "||" << _epoch
+                         << (_timestamp ? "||" + _timestamp->toString() : "")
+                         << (_canThrowSSVOnIgnored ? "|||canThrowSSVOnIgnored" : "");
 }
 
 }  // namespace mongo
