@@ -330,7 +330,17 @@ StatusWith<Shard::QueryResponse> ShardRemote::_runExhaustiveCursorCommand(
         return scheduleStatus;
     }
 
-    fetcher.join();
+    Status joinStatus = fetcher.join(opCtx);
+    if (!joinStatus.isOK()) {
+        if (ErrorCodes::isExceededTimeLimitError(joinStatus.code())) {
+            LOGV2(6195000,
+                  "Operation timed out {error}",
+                  "Operation timed out",
+                  "error"_attr = joinStatus);
+        }
+
+        return joinStatus;
+    }
 
     updateReplSetMonitor(host.getValue(), status);
 
@@ -424,7 +434,8 @@ void ShardRemote::runFireAndForgetCommand(OperationContext* opCtx,
 Status ShardRemote::runAggregation(
     OperationContext* opCtx,
     const AggregateCommandRequest& aggRequest,
-    std::function<bool(const std::vector<BSONObj>& batch)> callback) {
+    std::function<bool(const std::vector<BSONObj>& batch,
+                       const boost::optional<BSONObj>& postBatchResumeToken)> callback) {
 
     BSONObj readPrefMetadata;
 
@@ -467,7 +478,9 @@ Status ShardRemote::runAggregation(
         }
 
         try {
-            if (!callback(data.documents)) {
+            boost::optional<BSONObj> postBatchResumeToken =
+                data.documents.empty() ? data.otherFields.postBatchResumeToken : boost::none;
+            if (!callback(data.documents, postBatchResumeToken)) {
                 *nextAction = Fetcher::NextAction::kNoAction;
             }
         } catch (...) {
@@ -504,7 +517,10 @@ Status ShardRemote::runAggregation(
         return scheduleStatus;
     }
 
-    fetcher.join();
+    Status joinStatus = fetcher.join(opCtx);
+    if (!joinStatus.isOK()) {
+        return joinStatus;
+    }
 
     updateReplSetMonitor(host, status);
 
