@@ -325,7 +325,11 @@ public:
             const BSONObj shardKey =
                 getShardKey(opCtx, cm, nss, query, collation, boost::none, let, rc);
 
-            auto chunk = cm.findIntersectingChunk(shardKey, collation);
+            // For now, set bypassIsFieldHashedCheck to be true in order to skip the
+            // isFieldHashedCheck in the special case where _id is hashed and used as the shard key.
+            // This means that we always assume that a findAndModify request using _id is targetable
+            // to a single shard.
+            auto chunk = cm.findIntersectingChunk(shardKey, collation, true);
 
             _runCommand(opCtx,
                         chunk.getShardId(),
