@@ -37,20 +37,21 @@
 #include "mongo/db/catalog/document_validation.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/commands/update_metrics.h"
-#include "mongo/db/commands/write_commands/write_commands_common.h"
+#include "mongo/db/commands/write_commands_common.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/lasterror.h"
-#include "mongo/db/logical_clock.h"
 #include "mongo/db/pipeline/lite_parsed_pipeline.h"
 #include "mongo/db/stats/counters.h"
 #include "mongo/db/storage/duplicate_key_error_info.h"
 #include "mongo/executor/task_executor_pool.h"
 #include "mongo/logv2/log.h"
 #include "mongo/rpc/get_status_from_command_result.h"
+#include "mongo/s/chunk_manager_targeter.h"
 #include "mongo/s/client/num_hosts_targeted_metrics.h"
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/s/cluster_commands_helpers.h"
 #include "mongo/s/cluster_last_error_info.h"
+#include "mongo/s/cluster_write.h"
 #include "mongo/s/commands/cluster_explain.h"
 #include "mongo/s/commands/document_shard_key_update_util.h"
 #include "mongo/s/grid.h"
@@ -60,8 +61,6 @@
 #include "mongo/s/would_change_owning_shard_exception.h"
 #include "mongo/s/write_ops/batched_command_request.h"
 #include "mongo/s/write_ops/batched_command_response.h"
-#include "mongo/s/write_ops/chunk_manager_targeter.h"
-#include "mongo/s/write_ops/cluster_write.h"
 #include "mongo/util/timer.h"
 
 namespace mongo {
@@ -225,7 +224,7 @@ bool handleWouldChangeOwningShardError(OperationContext* opCtx,
             documentShardKeyUpdateUtil::startTransactionForShardKeyUpdate(opCtx);
             // Clear the error details from the response object before sending the write again
             response->unsetErrDetails();
-            ClusterWriter::write(opCtx, *request, &stats, response);
+            cluster::write(opCtx, *request, &stats, response);
             wouldChangeOwningShardErrorInfo =
                 getWouldChangeOwningShardErrorInfo(opCtx, *request, response, !isRetryableWrite);
             if (!wouldChangeOwningShardErrorInfo)
@@ -360,45 +359,37 @@ private:
      *
      * Does *not* retry or retarget if the metadata is stale.
      */
-    static Status _commandOpWrite(OperationContext* opCtx,
-                                  const std::string& dbName,
-                                  const BSONObj& command,
-                                  BatchItemRef targetingBatchItem,
-                                  std::vector<Strategy::CommandResult>* results) {
-        // Note that this implementation will not handle targeting retries and does not completely
-        // emulate write behavior
-        ChunkManagerTargeter targeter(targetingBatchItem.getRequest()->getNS());
-        Status status = targeter.init(opCtx);
-        if (!status.isOK())
-            return status;
+    static void _commandOpWrite(OperationContext* opCtx,
+                                const NamespaceString& nss,
+                                const BSONObj& command,
+                                BatchItemRef targetingBatchItem,
+                                std::vector<AsyncRequestsSender::Response>* results) {
+        auto endpoints = [&] {
+            // Note that this implementation will not handle targeting retries and does not
+            // completely emulate write behavior
+            ChunkManagerTargeter targeter(opCtx, nss);
 
-        auto swEndpoints = [&]() -> StatusWith<std::vector<ShardEndpoint>> {
             if (targetingBatchItem.getOpType() == BatchedCommandRequest::BatchType_Insert) {
-                auto swEndpoint = targeter.targetInsert(opCtx, targetingBatchItem.getDocument());
-                if (!swEndpoint.isOK())
-                    return swEndpoint.getStatus();
-                return std::vector<ShardEndpoint>{std::move(swEndpoint.getValue())};
+                return std::vector{targeter.targetInsert(opCtx, targetingBatchItem.getDocument())};
             } else if (targetingBatchItem.getOpType() == BatchedCommandRequest::BatchType_Update) {
-                return targeter.targetUpdate(opCtx, targetingBatchItem.getUpdate());
+                return targeter.targetUpdate(opCtx, targetingBatchItem);
             } else if (targetingBatchItem.getOpType() == BatchedCommandRequest::BatchType_Delete) {
-                return targeter.targetDelete(opCtx, targetingBatchItem.getDelete());
-            } else {
-                MONGO_UNREACHABLE;
+                return targeter.targetDelete(opCtx, targetingBatchItem);
             }
+            MONGO_UNREACHABLE;
         }();
 
-        if (!swEndpoints.isOK())
-            return swEndpoints.getStatus();
-
         // Assemble requests
         std::vector<AsyncRequestsSender::Request> requests;
-        for (const auto& endpoint : swEndpoints.getValue()) {
+        for (const auto& endpoint : endpoints) {
             BSONObj cmdObjWithVersions = BSONObj(command);
             if (endpoint.databaseVersion) {
                 cmdObjWithVersions =
                     appendDbVersionIfPresent(cmdObjWithVersions, *endpoint.databaseVersion);
             }
-            cmdObjWithVersions = appendShardVersion(cmdObjWithVersions, endpoint.shardVersion);
+            if (endpoint.shardVersion) {
+                cmdObjWithVersions = appendShardVersion(cmdObjWithVersions, *endpoint.shardVersion);
+            }
             requests.emplace_back(endpoint.shardName, cmdObjWithVersions);
         }
 
@@ -408,36 +399,20 @@ private:
         MultiStatementTransactionRequestsSender ars(
             opCtx,
             Grid::get(opCtx)->getExecutorPool()->getArbitraryExecutor(),
-            dbName,
+            nss.db(),
             requests,
             readPref,
             Shard::RetryPolicy::kNoRetry);
 
-        // Receive the responses.
-
-        Status dispatchStatus = Status::OK();
         while (!ars.done()) {
             // Block until a response is available.
             auto response = ars.next();
-
-            if (!response.swResponse.isOK()) {
-                dispatchStatus = std::move(response.swResponse.getStatus());
-                break;
-            }
-
-            Strategy::CommandResult result;
+            uassertStatusOK(response.swResponse);
 
             // If the response status was OK, the response must contain which host was targeted.
             invariant(response.shardHostAndPort);
-            result.target = ConnectionString(std::move(*response.shardHostAndPort));
-
-            result.shardTargetId = std::move(response.shardId);
-            result.result = std::move(response.swResponse.getValue().data);
-
-            results->push_back(result);
+            results->push_back(response);
         }
-
-        return dispatchStatus;
     }
 };
 
@@ -448,7 +423,6 @@ public:
                    BatchedCommandRequest batchedRequest,
                    UpdateMetrics* updateMetrics = nullptr)
         : CommandInvocation(command),
-          _bypass{shouldBypassDocumentValidationForCommand(request.body)},
           _request{&request},
           _batchedRequest{std::move(batchedRequest)},
           _updateMetrics{updateMetrics} {}
@@ -458,7 +432,7 @@ public:
     }
 
     bool getBypass() const {
-        return _bypass;
+        return _batchedRequest.getBypassDocumentValidation();
     }
 
 private:
@@ -474,7 +448,7 @@ private:
         // The batched request will only have WC if it was supplied by the client. Otherwise, the
         // batched request should use the WC from the opCtx.
         if (!batchedRequest.hasWriteConcern()) {
-            if (opCtx->getWriteConcern().usedDefault) {
+            if (opCtx->getWriteConcern().usedDefaultConstructedWC) {
                 // Pass writeConcern: {}, rather than {w: 1, wtimeout: 0}, so as to not override the
                 // configsvr w:majority upconvert.
                 batchedRequest.setWriteConcern(BSONObj());
@@ -494,7 +468,7 @@ private:
             batchedRequest.unsetWriteConcern();
         }
 
-        ClusterWriter::write(opCtx, batchedRequest, &stats, &response);
+        cluster::write(opCtx, batchedRequest, &stats, &response);
 
         bool updatedShardKey = false;
         if (_batchedRequest.getBatchType() == BatchedCommandRequest::BatchType_Update) {
@@ -509,7 +483,7 @@ private:
 
         if (!response.getOk()) {
             numAttempts = 0;
-        } else if (batchedRequest.getWriteCommandBase().getOrdered() &&
+        } else if (batchedRequest.getWriteCommandRequestBase().getOrdered() &&
                    response.isErrDetailsSet()) {
             // Add one failed attempt
             numAttempts = response.getErrDetailsAt(0)->getIndex() + 1;
@@ -535,9 +509,14 @@ private:
                 }
                 catalogCache->checkAndRecordOperationBlockedByRefresh(opCtx,
                                                                       mongo::LogicalOp::opUpdate);
-                debug.upsert = response.isUpsertDetailsSet();
-                debug.additiveMetrics.nMatched =
-                    response.getN() - (debug.upsert ? response.sizeUpsertDetails() : 0);
+
+                // The response.getN() count is the sum of documents matched and upserted.
+                if (response.isUpsertDetailsSet()) {
+                    debug.additiveMetrics.nMatched = response.getN() - response.sizeUpsertDetails();
+                    debug.additiveMetrics.nUpserted = response.sizeUpsertDetails();
+                } else {
+                    debug.additiveMetrics.nMatched = response.getN();
+                }
                 debug.additiveMetrics.nModified = response.getNModified();
 
                 invariant(_updateMetrics);
@@ -614,15 +593,16 @@ private:
 
         // Target the command to the shards based on the singleton batch item.
         BatchItemRef targetingBatchItem(&_batchedRequest, 0);
-        std::vector<Strategy::CommandResult> shardResults;
-        uassertStatusOK(_commandOpWrite(opCtx,
-                                        _request->getDatabase().toString(),
-                                        explainCmd,
-                                        targetingBatchItem,
-                                        &shardResults));
+        std::vector<AsyncRequestsSender::Response> shardResponses;
+        _commandOpWrite(
+            opCtx, _batchedRequest.getNS(), explainCmd, targetingBatchItem, &shardResponses);
         auto bodyBuilder = result->getBodyBuilder();
-        uassertStatusOK(ClusterExplain::buildExplainResult(
-            opCtx, shardResults, ClusterExplain::kWriteOnShards, timer.millis(), &bodyBuilder));
+        uassertStatusOK(ClusterExplain::buildExplainResult(opCtx,
+                                                           shardResponses,
+                                                           ClusterExplain::kWriteOnShards,
+                                                           timer.millis(),
+                                                           _request->body,
+                                                           &bodyBuilder));
     }
 
     NamespaceString ns() const override {
@@ -633,10 +613,6 @@ private:
         return true;
     }
 
-    ReadConcernSupportResult supportsReadConcern(repl::ReadConcernLevel level) const final {
-        return ReadConcernSupportResult::allSupportedAndDefaultPermitted();
-    }
-
     void doCheckAuthorization(OperationContext* opCtx) const final {
         try {
             doCheckAuthorizationHook(AuthorizationSession::get(opCtx->getClient()));
@@ -662,6 +638,10 @@ class ClusterInsertCmd final : public ClusterWriteCmd {
 public:
     ClusterInsertCmd() : ClusterWriteCmd("insert") {}
 
+    const std::set<std::string>& apiVersions() const {
+        return kApiVersions1;
+    }
+
 private:
     class Invocation final : public InvocationBase {
     public:
@@ -689,12 +669,20 @@ private:
     LogicalOp getLogicalOp() const override {
         return LogicalOp::opInsert;
     }
+
+    const AuthorizationContract* getAuthorizationContract() const final {
+        return &::mongo::write_ops::InsertCommandRequest::kAuthorizationContract;
+    }
 } clusterInsertCmd;
 
 class ClusterUpdateCmd final : public ClusterWriteCmd {
 public:
     ClusterUpdateCmd() : ClusterWriteCmd("update"), _updateMetrics{"update"} {}
 
+    const std::set<std::string>& apiVersions() const {
+        return kApiVersions1;
+    }
+
 private:
     class Invocation final : public InvocationBase {
     public:
@@ -712,8 +700,8 @@ private:
         auto parsedRequest = BatchedCommandRequest::parseUpdate(request);
         uassert(51195,
                 "Cannot specify runtime constants option to a mongos",
-                !parsedRequest.hasRuntimeConstants());
-        parsedRequest.setRuntimeConstants(Variables::generateRuntimeConstants(opCtx));
+                !parsedRequest.hasLegacyRuntimeConstants());
+        parsedRequest.setLegacyRuntimeConstants(Variables::generateRuntimeConstants(opCtx));
         return std::make_unique<Invocation>(
             this, request, std::move(parsedRequest), &_updateMetrics);
     }
@@ -726,6 +714,10 @@ private:
         return LogicalOp::opUpdate;
     }
 
+    const AuthorizationContract* getAuthorizationContract() const final {
+        return &::mongo::write_ops::UpdateCommandRequest::kAuthorizationContract;
+    }
+
     // Update related command execution metrics.
     UpdateMetrics _updateMetrics;
 } clusterUpdateCmd;
@@ -734,6 +726,10 @@ class ClusterDeleteCmd final : public ClusterWriteCmd {
 public:
     ClusterDeleteCmd() : ClusterWriteCmd("delete") {}
 
+    const std::set<std::string>& apiVersions() const {
+        return kApiVersions1;
+    }
+
 private:
     class Invocation final : public InvocationBase {
     public:
@@ -759,6 +755,11 @@ private:
     LogicalOp getLogicalOp() const override {
         return LogicalOp::opDelete;
     }
+
+    const AuthorizationContract* getAuthorizationContract() const final {
+        return &::mongo::write_ops::DeleteCommandRequest::kAuthorizationContract;
+    }
+
 } clusterDeleteCmd;
 
 }  // namespace
