@@ -27,16 +27,16 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kQuery
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/client/dbclient_base.h"
+#include "mongo/logv2/log.h"
 #include "mongo/scripting/mozjs/cursor_handle.h"
 #include "mongo/scripting/mozjs/implscope.h"
 #include "mongo/scripting/mozjs/scripting_util_gen.h"
 #include "mongo/scripting/mozjs/wrapconstrainedmethod.h"
-#include "mongo/util/log.h"
 
 namespace mongo {
 namespace mozjs {
@@ -77,7 +77,10 @@ void CursorHandleInfo::finalize(js::FreeOp* fop, JSObject* obj) {
                 auto status = exceptionToStatus();
 
                 try {
-                    LOG(0) << "Failed to kill cursor " << cursorId << " due to " << status;
+                    LOGV2_INFO(22782,
+                               "Failed to kill cursor",
+                               "cursorId"_attr = cursorId,
+                               "error"_attr = status);
                 } catch (...) {
                     // This is here in case logging fails.
                 }
