@@ -108,18 +108,8 @@ public:
     }
 
     Status initialize() {
-        auto status = _initializeGlobal();
-        if (!status.isOK()) {
-            return status;
-        }
-
-        return Status::OK();
-    }
-
-private:
-    Status _initializeGlobal() {
         if (_initialized) {
-            return Status::OK();
+            return {ErrorCodes::AlreadyInitialized, "CurlLibraryManager already initialized."};
         }
 
         CURLcode ret = curl_global_init(CURL_GLOBAL_ALL);
@@ -137,6 +127,10 @@ private:
         return Status::OK();
     }
 
+    bool isInitialized() const {
+        return _initialized;
+    }
+
 private:
     bool _initialized = false;
 
@@ -488,8 +482,10 @@ public:
     CurlHandle(executor::ConnectionPool::ConnectionHandle handle, CURL* curlHandle)
         : _poolHandle(std::move(handle)), _handle(curlHandle) {}
 
+    CurlHandle(CurlHandle&& other) = default;
+
     ~CurlHandle() {
-        if (!_success) {
+        if (!_success && _poolHandle.get() != nullptr) {
             _poolHandle->indicateFailure(
                 Status(ErrorCodes::HostUnreachable, "unknown curl handle failure"));
         }
@@ -528,25 +524,27 @@ public:
           _pool(std::make_shared<executor::ConnectionPool>(
               _typeFactory, "Curl", makePoolOptions(Seconds(60)))) {}
 
-    CurlHandle get(HostAndPort server, Protocols protocol);
+    StatusWith<CurlHandle> get(HostAndPort server, Protocols protocol);
 
 private:
     std::shared_ptr<CurlHandleTypeFactory> _typeFactory;
     std::shared_ptr<executor::ConnectionPool> _pool;
 };
 
-CurlHandle CurlPool::get(HostAndPort server, Protocols protocol) {
+StatusWith<CurlHandle> CurlPool::get(HostAndPort server, Protocols protocol) {
 
     auto sslMode = mapProtocolToSSLMode(protocol);
 
     auto semi = _pool->get(server, sslMode, Seconds(60));
 
     StatusWith<executor::ConnectionPool::ConnectionHandle> swHandle = std::move(semi).getNoThrow();
-    invariant(swHandle.isOK());
+    if (!swHandle.isOK()) {
+        return swHandle.getStatus();
+    }
 
     auto curlHandle = static_cast<PooledCurlHandle*>(swHandle.getValue().get())->get();
 
-    return CurlHandle(std::move(swHandle.getValue()), curlHandle);
+    return {CurlHandle(std::move(swHandle.getValue()), curlHandle)};
 }
 
 HostAndPort exactHostAndPortFromUrl(StringData url) {
@@ -598,14 +596,6 @@ public:
         _headers = headers;
     }
 
-    void setTimeout(Seconds timeout) final {
-        _timeout = timeout;
-    }
-
-    void setConnectTimeout(Seconds timeout) final {
-        _connectTimeout = timeout;
-    }
-
     HttpReply request(HttpMethod method,
                       StringData url,
                       ConstDataRange cdr = {nullptr, 0}) const final {
@@ -615,13 +605,15 @@ public:
 
             auto server = exactHostAndPortFromUrl(url);
 
-            CurlHandle handle(factory.get(server, protocol));
+            auto swHandle(factory.get(server, protocol));
+            uassertStatusOK(swHandle.getStatus());
+
+            CurlHandle handle(std::move(swHandle.getValue()));
             auto reply = request(handle.get(), method, url, cdr);
 
             // indidicateFailure will be called if indicateSuccess is not called.
             handle.indicateSuccess();
             return reply;
-
         } else {
             // Make a request with a non-pooled handle. This is needed during server startup when
             // thread spawning is not allowed which is required by the thread pool.
@@ -705,8 +697,6 @@ private:
     HttpConnectionPool _pool;
 
     bool _allowInsecure{false};
-    Seconds _timeout;
-    Seconds _connectTimeout;
 };
 
 class HttpClientProviderImpl : public HttpClientProvider {
@@ -716,17 +706,17 @@ public:
     }
 
     std::unique_ptr<HttpClient> create() final {
-        uassertStatusOK(curlLibraryManager.initialize());
+        invariant(curlLibraryManager.isInitialized());
         return std::make_unique<CurlHttpClient>(HttpConnectionPool::kUse);
     }
 
     std::unique_ptr<HttpClient> createWithoutConnectionPool() final {
-        uassertStatusOK(curlLibraryManager.initialize());
+        invariant(curlLibraryManager.isInitialized());
         return std::make_unique<CurlHttpClient>(HttpConnectionPool::kDoNotUse);
     }
 
     BSONObj getServerStatus() final {
-
+        invariant(curlLibraryManager.isInitialized());
         BSONObjBuilder info;
         info.append("type", "curl");
 
@@ -751,10 +741,11 @@ public:
 
 }  // namespace
 
-// Transitional API used by blockstore to trigger libcurl init
-// until it's been migrated to use the HTTPClient API.
-Status curlLibraryManager_initialize() {
-    return curlLibraryManager.initialize();
+MONGO_INITIALIZER_GENERAL(CurlLibraryManager,
+                          (),
+                          ("BeginStartupOptionParsing", "NativeSaslClientContext"))
+(InitializerContext* context) {
+    uassertStatusOK(curlLibraryManager.initialize());
 }
 
 
