@@ -38,13 +38,13 @@
 
 #include "mongo/base/checked_cast.h"
 #include "mongo/base/init.h"
-#include "mongo/base/initializer_context.h"
 #include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
 #include "mongo/crypto/sha1_block.h"
 #include "mongo/crypto/sha256_block.h"
 #include "mongo/logv2/log.h"
 #include "mongo/platform/random.h"
+#include "mongo/transport/ssl_connection_context.h"
 #include "mongo/util/base64.h"
 #include "mongo/util/concurrency/mutex.h"
 #include "mongo/util/fail_point.h"
@@ -55,6 +55,7 @@
 #include "mongo/util/net/ssl_manager.h"
 #include "mongo/util/net/ssl_options.h"
 #include "mongo/util/net/ssl_parameters_gen.h"
+#include "mongo/util/net/ssl_peer_info.h"
 
 using asio::ssl::apple::CFUniquePtr;
 
@@ -69,6 +70,8 @@ extern "C" SecIdentityRef SecIdentityCreate(CFAllocatorRef, SecCertificateRef, S
 
 namespace mongo {
 
+using transport::SSLConnectionContext;
+
 namespace {
 
 // This failpoint is a no-op on OSX.
@@ -332,15 +335,14 @@ StatusWith<SSLX509Name::Entry> extractSingleOIDEntry(::CFDictionaryRef entry) {
         std::move(swLabelStr.getValue()), 19, std::move(swValueStr.getValue()));
 }
 
-// Translate a raw DER subject sequence into a structured subject name.
-StatusWith<SSLX509Name> extractSubjectName(::CFDictionaryRef dict) {
-    auto swSubject = extractDictionaryValue<::CFDictionaryRef>(dict, ::kSecOIDX509V1SubjectName);
-    if (!swSubject.isOK()) {
-        return swSubject.getStatus();
+// Extract a name value from the dictionary for the given property key.
+StatusWith<SSLX509Name> extractPropertyName(::CFDictionaryRef dict, CFStringRef property) {
+    auto swProp = extractDictionaryValue<::CFDictionaryRef>(dict, property);
+    if (!swProp.isOK()) {
+        return swProp.getStatus();
     }
 
-    auto swElems =
-        extractDictionaryValue<::CFArrayRef>(swSubject.getValue(), ::kSecPropertyKeyValue);
+    auto swElems = extractDictionaryValue<::CFArrayRef>(swProp.getValue(), ::kSecPropertyKeyValue);
     if (!swElems.isOK()) {
         return swElems.getStatus();
     }
@@ -353,7 +355,7 @@ StatusWith<SSLX509Name> extractSubjectName(::CFDictionaryRef dict) {
         invariant(elem);
         if (::CFGetTypeID(elem) != ::CFDictionaryGetTypeID()) {
             return {ErrorCodes::InvalidSSLConfiguration,
-                    "Subject name element is not a dictionary"};
+                    str::stream() << toString(property) << " element is not a dictionary"};
         }
 
         auto swType = extractDictionaryValue<::CFStringRef>(elem, ::kSecPropertyKeyType);
@@ -375,7 +377,8 @@ StatusWith<SSLX509Name> extractSubjectName(::CFDictionaryRef dict) {
                 invariant(rdnElem);
                 if (::CFGetTypeID(rdnElem) != ::CFDictionaryGetTypeID()) {
                     return {ErrorCodes::InvalidSSLConfiguration,
-                            "Subject name sub-element is not a dictionary"};
+                            str::stream()
+                                << toString(property) << " sub-element is not a dictionary"};
                 }
                 auto swEntry = extractSingleOIDEntry(rdnElem);
                 if (!swEntry.isOK()) {
@@ -395,12 +398,17 @@ StatusWith<SSLX509Name> extractSubjectName(::CFDictionaryRef dict) {
         }
     }
 
-    SSLX509Name subjectName = SSLX509Name(std::move(ret));
-    Status normalize = subjectName.normalizeStrings();
+    SSLX509Name propertyName = SSLX509Name(std::move(ret));
+    Status normalize = propertyName.normalizeStrings();
     if (!normalize.isOK()) {
         return normalize;
     }
-    return subjectName;
+    return propertyName;
+}
+
+// Translate a raw DER subject sequence into a structured subject name.
+StatusWith<SSLX509Name> extractSubjectName(::CFDictionaryRef dict) {
+    return extractPropertyName(dict, ::kSecOIDX509V1SubjectName);
 }
 
 StatusWith<mongo::Date_t> extractValidityDate(::CFDictionaryRef dict,
@@ -541,6 +549,29 @@ StatusWith<std::vector<std::string>> extractSubjectAlternateNames(::CFDictionary
     return ret;
 }
 
+StatusWith<::SecCertificateRef> getCertificate(::CFArrayRef certs);
+
+StatusWith<std::vector<std::string>> extractSubjectAlternateNames(::CFArrayRef certs) {
+    auto swCert = getCertificate(certs);
+    if (!swCert.isOK()) {
+        return swCert.getStatus();
+    }
+    CFUniquePtr<::SecCertificateRef> cert(swCert.getValue());
+
+    CFUniquePtr<::CFMutableArrayRef> oids(
+        ::CFArrayCreateMutable(nullptr, 1, &::kCFTypeArrayCallBacks));
+    ::CFArrayAppendValue(oids.get(), ::kSecOIDSubjectAltName);
+
+    ::CFErrorRef err = nullptr;
+    CFUniquePtr<::CFDictionaryRef> cfdict(::SecCertificateCopyValues(cert.get(), oids.get(), &err));
+    CFUniquePtr<::CFErrorRef> cferror(err);
+    if (cferror) {
+        return Status(ErrorCodes::NoSuchKey, "Could not find Subject Alternative Name");
+    }
+
+    return extractSubjectAlternateNames(cfdict.get());
+}
+
 bool isCFDataEqual(::CFDataRef a, ::CFDataRef b) {
     const auto len = ::CFDataGetLength(a);
     if (::CFDataGetLength(b) != len) {
@@ -818,11 +849,13 @@ StatusWith<CFUniquePtr<::CFArrayRef>> loadPEM(const std::string& keyfilepath,
     return std::move(cfcerts);
 }
 
-StatusWith<SSLX509Name> certificateGetSubject(::SecCertificateRef cert, Date_t* expire = nullptr) {
-    // Fetch expiry range and full subject name.
+StatusWith<SSLX509Name> certificateGetPropertyName(::SecCertificateRef cert,
+                                                   CFStringRef property,
+                                                   Date_t* expire = nullptr) {
+    // Fetch expiry range and property name.
     CFUniquePtr<::CFMutableArrayRef> oids(
         ::CFArrayCreateMutable(nullptr, expire ? 3 : 1, &::kCFTypeArrayCallBacks));
-    ::CFArrayAppendValue(oids.get(), ::kSecOIDX509V1SubjectName);
+    ::CFArrayAppendValue(oids.get(), property);
     if (expire) {
         ::CFArrayAppendValue(oids.get(), ::kSecOIDX509V1ValidityNotBefore);
         ::CFArrayAppendValue(oids.get(), ::kSecOIDX509V1ValidityNotAfter);
@@ -836,14 +869,14 @@ StatusWith<SSLX509Name> certificateGetSubject(::SecCertificateRef cert, Date_t*
                 str::stream() << "Unable to determine certificate validity: " << cferror};
     }
 
-    auto swSubjectName = extractSubjectName(cfdict.get());
-    if (!swSubjectName.isOK()) {
-        return swSubjectName.getStatus();
+    auto swPropertyName = extractPropertyName(cfdict.get(), property);
+    if (!swPropertyName.isOK()) {
+        return swPropertyName.getStatus();
     }
-    auto subject = swSubjectName.getValue();
+    auto propertyName = swPropertyName.getValue();
 
     if (!expire) {
-        return subject;
+        return propertyName;
     }
 
     // Marshal expiration.
@@ -862,10 +895,11 @@ StatusWith<SSLX509Name> certificateGetSubject(::SecCertificateRef cert, Date_t*
                 "The provided SSL certificate is expired or not yet valid"};
     }
 
-    return subject;
+    return propertyName;
 }
 
-StatusWith<SSLX509Name> certificateGetSubject(::CFArrayRef certs, Date_t* expire = nullptr) {
+// Get the root certificate from an array of certs.
+StatusWith<::SecCertificateRef> getCertificate(::CFArrayRef certs) {
     if (::CFArrayGetCount(certs) <= 0) {
         return {ErrorCodes::InvalidSSLConfiguration, "No certificates in certificate list"};
     }
@@ -882,8 +916,26 @@ StatusWith<SSLX509Name> certificateGetSubject(::CFArrayRef certs, Date_t* expire
                 str::stream() << "Unable to get certificate from identity: "
                               << stringFromOSStatus(status)};
     }
-    CFUniquePtr<::SecCertificateRef> cert(idcert);
-    return certificateGetSubject(cert.get(), expire);
+    return idcert;
+}
+
+StatusWith<SSLX509Name> certificateGetPropertyName(::CFArrayRef certs,
+                                                   CFStringRef property,
+                                                   Date_t* expire = nullptr) {
+    auto swCert = getCertificate(certs);
+    if (!swCert.isOK()) {
+        return swCert.getStatus();
+    }
+    CFUniquePtr<::SecCertificateRef> cert(swCert.getValue());
+    return certificateGetPropertyName(cert.get(), property, expire);
+}
+
+StatusWith<SSLX509Name> certificateGetSubject(::SecCertificateRef cert, Date_t* expire = nullptr) {
+    return certificateGetPropertyName(cert, ::kSecOIDX509V1SubjectName, expire);
+}
+
+StatusWith<SSLX509Name> certificateGetSubject(::CFArrayRef certs, Date_t* expire = nullptr) {
+    return certificateGetPropertyName(certs, ::kSecOIDX509V1SubjectName, expire);
 }
 
 StatusWith<CFUniquePtr<::CFArrayRef>> copyMatchingCertificate(
@@ -1226,6 +1278,9 @@ public:
                           const SSLParams& params,
                           ConnectionDirection direction) final;
 
+    void registerOwnedBySSLContext(
+        std::weak_ptr<const transport::SSLConnectionContext> ownedByContext) final;
+
     SSLConnectionInterface* connect(Socket* socket) final;
     SSLConnectionInterface* accept(Socket* socket, const char* initialBytes, int len) final;
 
@@ -1239,7 +1294,7 @@ public:
                                                         const HostAndPort& hostForLogging,
                                                         const ExecutorPtr& reactor) final;
 
-    Status stapleOCSPResponse(asio::ssl::apple::Context* context) final;
+    Status stapleOCSPResponse(asio::ssl::apple::Context* context, bool asyncOCSPStaple) final;
 
     const SSLConfiguration& getSSLConfiguration() const final {
         return _sslConfiguration;
@@ -1249,6 +1304,10 @@ public:
     int SSL_write(SSLConnectionInterface* conn, const void* buf, int num) final;
     int SSL_shutdown(SSLConnectionInterface* conn) final;
 
+    SSLInformationToLog getSSLInformationToLog() const final;
+
+    void stopJobs() final;
+
 private:
     bool _weakValidation;
     bool _allowInvalidCertificates;
@@ -1271,6 +1330,10 @@ private:
     CFUniquePtr<::CFArrayRef> _serverCA;
 
     SSLConfiguration _sslConfiguration;
+
+    // Weak pointer to verify that this manager is still owned by this context.
+    // Will be used if stapling is implemented.
+    synchronized_value<std::weak_ptr<const SSLConnectionContext>> _ownedByContext;
 };
 
 SSLManagerApple::SSLManagerApple(const SSLParams& params, bool isServer)
@@ -1291,8 +1354,18 @@ SSLManagerApple::SSLManagerApple(const SSLParams& params, bool isServer)
             uassertStatusOK(
                 _sslConfiguration.setServerSubjectName(uassertStatusOK(certificateGetSubject(
                     _serverCtx.certs.get(), &_sslConfiguration.serverCertificateExpirationDate))));
-            static auto task =
-                CertificateExpirationMonitor(_sslConfiguration.serverCertificateExpirationDate);
+            CertificateExpirationMonitor::get()->updateExpirationDeadline(
+                _sslConfiguration.serverCertificateExpirationDate);
+
+            auto swSans = extractSubjectAlternateNames(_serverCtx.certs.get());
+            const bool hasSan = swSans.isOK() && (0 != swSans.getValue().size());
+            if (!hasSan) {
+                LOGV2_WARNING_OPTIONS(
+                    551191,
+                    {logv2::LogTag::kStartupWarnings},
+                    "Server certificate has no compatible Subject Alternative Name. "
+                    "This may prevent TLS clients from connecting");
+            }
         }
     }
 
@@ -1411,6 +1484,11 @@ Status SSLManagerApple::initSSLContext(asio::ssl::apple::Context* context,
         params.sslCertificateSelector, params.sslPEMKeyFile, params.sslPEMKeyPassword);
 }
 
+void SSLManagerApple::registerOwnedBySSLContext(
+    std::weak_ptr<const transport::SSLConnectionContext> ownedByContext) {
+    _ownedByContext = ownedByContext;
+}
+
 SSLConnectionInterface* SSLManagerApple::connect(Socket* socket) {
     return new SSLConnectionApple(
         &_clientCtx, socket, ::kSSLClientSide, socket->remoteAddr().hostOrIp());
@@ -1456,10 +1534,13 @@ StatusWith<TLSVersion> mapTLSVersion(SSLContextRef ssl) {
     }
 }
 
-Status SSLManagerApple::stapleOCSPResponse(asio::ssl::apple::Context* context) {
+Status SSLManagerApple::stapleOCSPResponse(asio::ssl::apple::Context* context,
+                                           bool asyncOCSPStaple) {
     return Status::OK();
 }
 
+void SSLManagerApple::stopJobs() {}
+
 Future<SSLPeerInfo> SSLManagerApple::parseAndValidatePeerCertificate(
     ::SSLContextRef ssl,
     boost::optional<std::string> sniName,
@@ -1701,8 +1782,12 @@ Future<SSLPeerInfo> SSLManagerApple::parseAndValidatePeerCertificate(
 int SSLManagerApple::SSL_read(SSLConnectionInterface* conn, void* buf, int num) {
     auto ssl = checked_cast<SSLConnectionApple*>(conn)->get();
     size_t read = 0;
-    uassertOSStatusOK(::SSLRead(ssl, static_cast<uint8_t*>(buf), num, &read),
-                      SocketErrorKind::RECV_ERROR);
+
+    const auto status = ::SSLRead(ssl, static_cast<uint8_t*>(buf), num, &read);
+    if (status != ::errSSLWouldBlock) {
+        uassertOSStatusOK(status, SocketErrorKind::RECV_ERROR);
+    }
+
     return read;
 }
 
@@ -1724,6 +1809,57 @@ int SSLManagerApple::SSL_shutdown(SSLConnectionInterface* conn) {
     return 1;
 }
 
+void getCertInfo(CertInformationToLog* info, const ::CFArrayRef cert) {
+    info->subject = uassertStatusOK(certificateGetSubject(cert));
+    info->issuer = uassertStatusOK(certificateGetPropertyName(cert, ::kSecOIDX509V1IssuerName));
+
+    // Get validity dates via SecCertificateCopyValues
+    CFUniquePtr<::CFMutableArrayRef> oids(
+        ::CFArrayCreateMutable(nullptr, 2, &::kCFTypeArrayCallBacks));
+    ::CFArrayAppendValue(oids.get(), ::kSecOIDX509V1ValidityNotBefore);
+    ::CFArrayAppendValue(oids.get(), ::kSecOIDX509V1ValidityNotAfter);
+
+    CFUniquePtr<::SecCertificateRef> rootCert(uassertStatusOK(getCertificate(cert)));
+    ::CFErrorRef cferror = nullptr;
+    CFUniquePtr<::CFDictionaryRef> cfdict(
+        ::SecCertificateCopyValues(rootCert.get(), oids.get(), &cferror));
+    if (cferror) {
+        CFUniquePtr<::CFErrorRef> deleter(cferror);
+        uasserted(ErrorCodes::InvalidSSLConfiguration,
+                  str::stream() << "Failed to get thumbprint: " << cferror);
+    }
+
+    info->validityNotBefore = uassertStatusOK(
+        extractValidityDate(cfdict.get(), ::kSecOIDX509V1ValidityNotBefore, "valid-from"));
+    info->validityNotAfter = uassertStatusOK(
+        extractValidityDate(cfdict.get(), ::kSecOIDX509V1ValidityNotAfter, "valid-until"));
+
+    // Compute certificate thumbprint
+    CFUniquePtr<::CFDataRef> cfCertData(::SecCertificateCopyData(rootCert.get()));
+    ConstDataRange certData(reinterpret_cast<const char*>(::CFDataGetBytePtr(cfCertData.get())),
+                            ::CFDataGetLength(cfCertData.get()));
+
+    // Comupte hash from bytes of certificate
+    const auto certSha1 = SHA1Block::computeHash({certData});
+    info->thumbprint =
+        std::vector<char>((char*)certSha1.data(), (char*)certSha1.data() + certSha1.kHashLength);
+    info->hexEncodedThumbprint = hexblob::encode(info->thumbprint.data(), info->thumbprint.size());
+}
+
+SSLInformationToLog SSLManagerApple::getSSLInformationToLog() const {
+    SSLInformationToLog info;
+    // server CA should definitely exist, client CA is optional
+    if (_serverCtx.certs.get()) {
+        getCertInfo(&info.server, _serverCtx.certs.get());
+    }
+    if (_clientCtx.certs.get()) {
+        CertInformationToLog clientInfo;
+        getCertInfo(&clientInfo, _clientCtx.certs.get());
+        info.cluster = clientInfo;
+    }
+    return info;
+}
+
 }  // namespace
 
 /////////////////////////////////////////////////////////////////////////////
@@ -1732,10 +1868,18 @@ int SSLManagerApple::SSL_shutdown(SSLConnectionInterface* conn) {
 bool isSSLServer = false;
 
 extern SSLManagerInterface* theSSLManager;
+extern SSLManagerCoordinator* theSSLManagerCoordinator;
 
-std::unique_ptr<SSLManagerInterface> SSLManagerInterface::create(const SSLParams& params,
+std::shared_ptr<SSLManagerInterface> SSLManagerInterface::create(
+    const SSLParams& params,
+    const std::optional<TransientSSLParams>& transientSSLParams,
+    bool isServer) {
+    return std::make_shared<SSLManagerApple>(params, isServer);
+}
+
+std::shared_ptr<SSLManagerInterface> SSLManagerInterface::create(const SSLParams& params,
                                                                  bool isServer) {
-    return std::make_unique<SSLManagerApple>(params, isServer);
+    return std::make_shared<SSLManagerApple>(params, isServer);
 }
 
 MONGO_INITIALIZER_WITH_PREREQUISITES(SSLManager, ("EndStartupOptionHandling"))
@@ -1744,9 +1888,8 @@ MONGO_INITIALIZER_WITH_PREREQUISITES(SSLManager, ("EndStartupOptionHandling"))
         nullptr, mongodbRolesOID.identifier.c_str(), ::kCFStringEncodingUTF8);
 
     if (!isSSLServer || (sslGlobalParams.sslMode.load() != SSLParams::SSLMode_disabled)) {
-        theSSLManager = new SSLManagerApple(sslGlobalParams, isSSLServer);
+        theSSLManagerCoordinator = new SSLManagerCoordinator();
     }
-    return Status::OK();
 }
 
 }  // namespace mongo
